"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.shallowCompare = exports.is_different_array = exports.is_different = exports.field_cmp = exports.timestamp_cmp = exports.cmp_array = exports.cmp_moment = exports.cmp_Date = exports.cmp = void 0;
var lodash_1 = require("lodash");
function cmp(a, b) {
    if (a < b) {
        return -1;
    }
    else if (a > b) {
        return 1;
    }
    return 0;
}
exports.cmp = cmp;
/*
compare two Date | undefined | null objects.

null and undefined are considered equal to each other.

null_last:
  - true: nulls are infinitely in the future
  - false: nulls are the dawn of mankind
*/
function cmp_Date(a, b, null_last) {
    if (null_last === void 0) { null_last = false; }
    if (a == null) {
        if (b == null) {
            return 0;
        }
        return null_last ? 1 : -1;
    }
    // a != null
    if (b == null) {
        return null_last ? -1 : 1;
    }
    if (a < b)
        return -1;
    if (a > b)
        return 1;
    return 0; // note: a == b for Date objects doesn't work as expected, but that's OK here.
}
exports.cmp_Date = cmp_Date;
function cmp_moment(a, b, null_last) {
    if (null_last === void 0) { null_last = false; }
    return cmp_Date(a === null || a === void 0 ? void 0 : a.toDate(), b === null || b === void 0 ? void 0 : b.toDate(), null_last);
}
exports.cmp_moment = cmp_moment;
function cmp_array(a, b) {
    var end = Math.max(a.length, b.length);
    for (var i = 0; i < end; i++) {
        var c = cmp(a[i], b[i]);
        if (c) {
            return c;
        }
    }
    return 0;
}
exports.cmp_array = cmp_array;
function timestamp_cmp(a, b, field) {
    if (field == null) {
        field = "timestamp";
    }
    return -cmp_Date(a[field], b[field]);
}
exports.timestamp_cmp = timestamp_cmp;
function field_cmp(field) {
    return function (a, b) { return cmp(a[field], b[field]); };
}
exports.field_cmp = field_cmp;
function is_different(a, b, fields, verbose) {
    var e_1, _a, e_2, _b, e_3, _c;
    if (verbose != null) {
        return is_different_verbose(a, b, fields, verbose);
    }
    var field;
    if (a == null) {
        if (b == null) {
            return false; // they are the same
        }
        try {
            // a not defined but b is
            for (var fields_1 = __values(fields), fields_1_1 = fields_1.next(); !fields_1_1.done; fields_1_1 = fields_1.next()) {
                field = fields_1_1.value;
                if (b[field] != null) {
                    return true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (fields_1_1 && !fields_1_1.done && (_a = fields_1.return)) _a.call(fields_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return false;
    }
    if (b == null) {
        try {
            // a is defined or would be handled above
            for (var fields_2 = __values(fields), fields_2_1 = fields_2.next(); !fields_2_1.done; fields_2_1 = fields_2.next()) {
                field = fields_2_1.value;
                if (a[field] != null) {
                    return true; // different
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (fields_2_1 && !fields_2_1.done && (_b = fields_2.return)) _b.call(fields_2);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return false; // same
    }
    try {
        for (var fields_3 = __values(fields), fields_3_1 = fields_3.next(); !fields_3_1.done; fields_3_1 = fields_3.next()) {
            field = fields_3_1.value;
            if (a[field] !== b[field]) {
                return true;
            }
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (fields_3_1 && !fields_3_1.done && (_c = fields_3.return)) _c.call(fields_3);
        }
        finally { if (e_3) throw e_3.error; }
    }
    return false;
}
exports.is_different = is_different;
// Use for debugging purposes only -- copy code from above to avoid making that
// code more complicated and possibly slower.
function is_different_verbose(a, b, fields, verbose) {
    var e_4, _a, e_5, _b, e_6, _c;
    function log() {
        var x = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            x[_i] = arguments[_i];
        }
        console.log.apply(console, __spread(["is_different_verbose", verbose], x));
    }
    var field;
    if (a == null) {
        if (b == null) {
            log("both null");
            return false; // they are the same
        }
        try {
            // a not defined but b is
            for (var fields_4 = __values(fields), fields_4_1 = fields_4.next(); !fields_4_1.done; fields_4_1 = fields_4.next()) {
                field = fields_4_1.value;
                if (b[field] != null) {
                    log("a not defined but b is");
                    return true;
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (fields_4_1 && !fields_4_1.done && (_a = fields_4.return)) _a.call(fields_4);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return false;
    }
    if (b == null) {
        try {
            // a is defined or would be handled above
            for (var fields_5 = __values(fields), fields_5_1 = fields_5.next(); !fields_5_1.done; fields_5_1 = fields_5.next()) {
                field = fields_5_1.value;
                if (a[field] != null) {
                    log("b null and \"" + field + "\" of a is not null");
                    return true; // different
                }
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (fields_5_1 && !fields_5_1.done && (_b = fields_5.return)) _b.call(fields_5);
            }
            finally { if (e_5) throw e_5.error; }
        }
        return false; // same
    }
    try {
        for (var fields_6 = __values(fields), fields_6_1 = fields_6.next(); !fields_6_1.done; fields_6_1 = fields_6.next()) {
            field = fields_6_1.value;
            if (a[field] !== b[field]) {
                log("field \"" + field + "\" differs", a[field], b[field]);
                return true;
            }
        }
    }
    catch (e_6_1) { e_6 = { error: e_6_1 }; }
    finally {
        try {
            if (fields_6_1 && !fields_6_1.done && (_c = fields_6.return)) _c.call(fields_6);
        }
        finally { if (e_6) throw e_6.error; }
    }
    log("same");
    return false;
}
var is_different_array = function (a, b) { return !lodash_1.isEqual(a, b); };
exports.is_different_array = is_different_array;
// See https://stackoverflow.com/questions/22266826/how-can-i-do-a-shallow-comparison-of-the-properties-of-two-objects-with-javascri/22266891#22266891
var shallowCompare = function (obj1, obj2) {
    return Object.keys(obj1).length === Object.keys(obj2).length &&
        Object.keys(obj1).every(function (key) { return obj2.hasOwnProperty(key) && obj1[key] === obj2[key]; });
};
exports.shallowCompare = shallowCompare;
