"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var e_1, _a, e_2, _b;
Object.defineProperty(exports, "__esModule", { value: true });
exports.tool2display = exports.parser2display = exports.syntax2display = exports.parser2tool = exports.syntax2tool = exports.ext2parser = exports.ext2syntax = exports.file_extensions = void 0;
// common configuration for mapping programming languages (lower case) to formatters
// this is used by webapp and the project
var misc_1 = require("./misc");
// the set of file extensions where we want to have formatting support
exports.file_extensions = misc_1.tuple([
    "js",
    "jsx",
    "ts",
    "tsx",
    "json",
    "md",
    "css",
    "py",
    "r",
    "rs",
    "go",
    "yml",
    "yaml",
    "xml",
    "cml" /* that's xml */,
    "kml" /* geodata keyhole markup, also xml */,
    "xsl",
    "ptx",
    "c",
    "c++",
    "cc",
    "cpp",
    "h",
    "bib",
]);
exports.ext2syntax = Object.freeze({
    js: "JavaScript",
    jsx: "jsx",
    md: "Markdown",
    rmd: "RMarkdown",
    css: "CSS",
    ts: "TypeScript",
    tsx: "tsx",
    json: "JSON",
    yaml: "yaml",
    yml: "yaml",
    py: "python3",
    tex: "latex",
    html: "html",
    r: "R",
    rs: "rust",
    go: "go",
    c: "clang",
    cc: "clang",
    "c++": "clang",
    cpp: "clang",
    h: "clang",
    xml: "xml",
    cml: "xml",
    kml: "xml",
    xsl: "xml",
    ptx: "xml",
    bib: "bibtex",
});
exports.ext2parser = exports.ext2syntax;
exports.syntax2tool = Object.freeze({
    py: "python",
    python: "python",
    python3: "python",
    R: "r",
    r: "r",
    JavaScript: "babel",
    jsx: "babel",
    tsx: "typescript",
    TypeScript: "typescript",
    typescript: "typescript",
    CSS: "css",
    json: "json",
    JSON: "json",
    yaml: "yaml",
    markdown: "markdown",
    Markdown: "markdown",
    RMarkdown: "markdown",
    c: "clang-format",
    clang: "clang-format",
    "clang-format": "clang-format",
    "c++": "clang-format",
    babel: "prettier",
    latex: "latex",
    go: "gofmt",
    gofmt: "gofmt",
    rust: "rustfmt",
    rustfmt: "rustfmt",
    bibtex: "bib-biber",
    xml: "xml-tidy",
    html: "html-tidy",
});
exports.parser2tool = exports.syntax2tool;
exports.syntax2display = Object.freeze({
    r: "R Language",
    c: "C",
    "c++": "C++",
    latex: "LaTeX",
    "bib-biber": "Bibtex",
    json: "JSON",
    yaml: "YAML",
    py: "Python",
    gofmt: "Go",
    rust: "Rust",
    rustfmt: "Rust",
    markdown: "Markdown",
    typescript: "TypeScript",
    html: "HTML",
    xml: "XML",
    css: "CSS",
    javascript: "JavaScript",
});
exports.parser2display = exports.syntax2display;
var t2d = {};
try {
    for (var _c = __values(Object.keys(exports.parser2tool)), _d = _c.next(); !_d.done; _d = _c.next()) {
        var parser = _d.value;
        var tool = exports.parser2tool[parser];
        if (t2d[tool] == null)
            t2d[tool] = [];
        var lang = exports.parser2display[parser];
        if (lang != null)
            t2d[tool].push(lang);
    }
}
catch (e_1_1) { e_1 = { error: e_1_1 }; }
finally {
    try {
        if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
    }
    finally { if (e_1) throw e_1.error; }
}
try {
    for (var _e = __values(Object.keys(t2d)), _f = _e.next(); !_f.done; _f = _e.next()) {
        var tool = _f.value;
        t2d[tool] = t2d[tool].sort();
    }
}
catch (e_2_1) { e_2 = { error: e_2_1 }; }
finally {
    try {
        if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
    }
    finally { if (e_2) throw e_2.error; }
}
exports.tool2display = Object.freeze(t2d);
