"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("./types");
types_1.Table({
    name: "blobs",
    fields: {
        id: {
            type: "uuid",
            desc: "The uuid of this blob, which is a uuid derived from the Sha1 hash of the blob content.",
        },
        blob: {
            type: "Buffer",
            desc: "The actual blob content",
        },
        expire: {
            type: "timestamp",
            desc: "When to expire this blob (when delete_expired is called on the database).",
        },
        created: {
            type: "timestamp",
            desc: "When the blob was created.",
        },
        project_id: {
            type: "string",
            desc: "The uuid of the project that created the blob.",
        },
        last_active: {
            type: "timestamp",
            desc: "When the blob was last pulled from the database.",
        },
        count: {
            type: "number",
            desc: "How many times the blob has been pulled from the database.",
        },
        size: {
            type: "number",
            desc: "The size in bytes of the blob.",
        },
        gcloud: {
            type: "string",
            desc: "name of a bucket that contains the actual blob, if available.",
        },
        backup: {
            type: "boolean",
            desc: "if true, then this blob was saved to an offsite backup",
        },
        compress: {
            type: "string",
            desc: "optional compression used: 'gzip', 'zlib', 'snappy'",
        },
    },
    rules: {
        desc: "Table that stores blobs mainly generated as output of Sage worksheets.",
        primary_key: "id",
        // these indices speed up the search been done in 'copy_all_blobs_to_gcloud'
        // less important to make this query fast, but we want to avoid thrashing cache
        pg_indexes: ["((expire IS NULL))", "((gcloud IS NULL))", "last_active"],
        user_query: {
            get: {
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var obj;
                        return __generator(this, function (_a) {
                            obj = Object.assign({}, opts.query);
                            if (obj == null || obj.id == null) {
                                cb("id must be specified");
                                return [2 /*return*/];
                            }
                            database.get_blob({
                                uuid: obj.id,
                                cb: function (err, blob) {
                                    if (err) {
                                        cb(err);
                                    }
                                    else {
                                        cb(undefined, { id: obj.id, blob: blob });
                                    }
                                },
                            });
                            return [2 /*return*/];
                        });
                    });
                },
                fields: {
                    id: null,
                    blob: null,
                },
            },
            set: {
                // NOTE: we put "as any" for fields below because ttl is not an actual field but
                // it is allowed for set queries and determine the expire field.  I would rather
                // do this (which *is* supported by the backend) then not restrict the fields keys
                // for other schema entries.  Alternatively, we could have a special kind of field
                // above that is "virtual", but that requires writing more code in the backend. We'll
                // do that if necessary.
                fields: {
                    id: true,
                    blob: true,
                    project_id: "project_write",
                    ttl: 0,
                },
                required_fields: {
                    id: true,
                    blob: true,
                    project_id: true,
                },
                instead_of_change: function (database, _old_value, new_val, _account_id, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        return __generator(this, function (_a) {
                            database.save_blob({
                                uuid: new_val.id,
                                blob: new_val.blob,
                                ttl: new_val.ttl,
                                project_id: new_val.project_id,
                                check: true,
                                cb: cb,
                            });
                            return [2 /*return*/];
                        });
                    });
                },
            },
        },
    },
});
