"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.client_db = void 0;
// Client side versions of some db functions, which are used, e.g., when setting fields.
var misc_1 = require("../misc");
var index_1 = require("./index");
var sha1 = require("sha1");
var ClientDB = /** @class */ (function () {
    function ClientDB() {
        this.sha1 = this.sha1.bind(this);
        this._user_set_query_project_users = this._user_set_query_project_users.bind(this);
        this._user_set_query_project_change_after = this._user_set_query_project_change_after.bind(this);
        this._user_set_query_project_change_before = this._user_set_query_project_change_before.bind(this);
        this.primary_keys = this.primary_keys.bind(this);
        this.r = {};
    }
    ClientDB.prototype.sha1 = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var v;
        try {
            v = args
                .map(function (x) { return (typeof x === "string" ? x : JSON.stringify(x)); })
                .join("");
        }
        catch (err) {
            if (console != null && console.warn != null) {
                console.warn("args=", args);
            }
            throw err;
        }
        return sha1(v);
    };
    ClientDB.prototype._user_set_query_project_users = function (obj) {
        // client allows anything; server may be more stringent
        return obj.users;
    };
    ClientDB.prototype._user_set_query_project_change_after = function (_obj, _old_val, _new_val, cb) {
        cb();
    };
    ClientDB.prototype._user_set_query_project_change_before = function (_obj, _old_val, _new_val, cb) {
        cb();
    };
    ClientDB.prototype.primary_keys = function (table) {
        if (this._primary_keys_cache == null) {
            this._primary_keys_cache = {};
        }
        if (this._primary_keys_cache[table] != null) {
            return this._primary_keys_cache[table];
        }
        var t = index_1.SCHEMA[table];
        if (typeof t.virtual == "string") {
            t = index_1.SCHEMA[t.virtual];
        }
        var v = t != null ? t.primary_key : undefined;
        if (v == null) {
            throw Error("primary key for table '" + table + "' must be explicitly specified in schema");
        }
        if (typeof v === "string") {
            return (this._primary_keys_cache[table] = [v]);
        }
        else if (misc_1.is_array(v) && typeof v == "object") {
            // the typeof is just to make typescript happy
            if (v.length === 0) {
                throw Error("at least one primary key must specified");
            }
            return (this._primary_keys_cache[table] = v);
        }
        else {
            throw Error("primary key must be a string or array of strings");
        }
    };
    return ClientDB;
}());
exports.client_db = new ClientDB();
