"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("./types");
/*
This is a virtual table to make it simple from clients to
query about all usage ever of a specific file or directory
tree by a given user.

The usage times are the union of times both when patches
and times when the file was accessed.  Thus this would be
useful to see if a user opened a PDF file, say, even though
they didn't edit it.
*/
var LIMIT = 1000;
types_1.Table({
    name: "file_use_times",
    fields: {
        project_id: {
            type: "uuid",
            desc: "id of a project",
        },
        account_id: {
            type: "uuid",
            desc: "id of a user",
        },
        path: {
            type: "string",
            desc: "path to a specific file in the project",
        },
        edit_times: {
            type: "array",
            desc: "array of times (as ms since epoch) when the file was edited by the given account_id, sorted from newest to oldest. At most " + LIMIT + " values are returned.",
        },
        access_times: {
            type: "array",
            desc: "array of times (as ms since epoch) when the file was accessed by the given account_id, sorted from newest to oldest.   At most " + LIMIT + " values are returned.",
        },
    },
    rules: {
        virtual: true,
        desc: "File usage information.",
        anonymous: false,
        primary_key: ["project_id", "path"],
        user_query: {
            get: {
                options: [{ limit: LIMIT }],
                fields: {
                    project_id: null,
                    account_id: null,
                    path: null,
                    access_times: null,
                    edit_times: null,
                },
                // Actual query is implemented using this code below rather than an actual query directly.
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var obj, project_id, account_id, path, edit_times, access_times, limit, x, err_1;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    obj = Object.assign({}, opts.query);
                                    project_id = obj.project_id, account_id = obj.account_id, path = obj.path;
                                    if (project_id == null || account_id == null || path == null) {
                                        cb("project_id, account_id, and path must all be specified");
                                        return [2 /*return*/];
                                    }
                                    edit_times = obj.edit_times === null;
                                    access_times = obj.access_times === null;
                                    if (!edit_times && !access_times) {
                                        // dumb -- nothing to do; but let's make this edge case work, of course.
                                        cb(undefined, obj);
                                        return [2 /*return*/];
                                    }
                                    limit = LIMIT;
                                    if (opts.options && opts.options[0] && opts.options[0].limit) {
                                        // hackishly only support limit option.
                                        limit = opts.options[0].limit;
                                    }
                                    _a.label = 1;
                                case 1:
                                    _a.trys.push([1, 3, , 4]);
                                    return [4 /*yield*/, database.file_use_times({
                                            project_id: project_id,
                                            account_id: account_id,
                                            user_account_id: opts.account_id,
                                            path: path,
                                            limit: limit,
                                            access_times: access_times,
                                            edit_times: edit_times,
                                        })];
                                case 2:
                                    x = _a.sent();
                                    if (access_times) {
                                        obj.access_times = x.access_times;
                                    }
                                    if (edit_times) {
                                        obj.edit_times = x.edit_times;
                                    }
                                    cb(undefined, obj);
                                    return [3 /*break*/, 4];
                                case 3:
                                    err_1 = _a.sent();
                                    cb(err_1);
                                    return [3 /*break*/, 4];
                                case 4: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
        },
    },
});
