"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
var misc_1 = require("../misc");
var index_1 = require("./index");
var types_1 = require("./types");
types_1.Table({
    name: "project_log",
    rules: {
        primary_key: "id",
        // db_standby feels too slow for this, since the user only
        // does this query when they actually click to show the log.
        //db_standby: "unsafe",
        durability: "soft",
        pg_indexes: ["project_id", "time"],
        user_query: {
            get: {
                pg_where: ["time >= NOW() - interval '2 months'", "projects"],
                pg_changefeed: "projects",
                options: [{ order_by: "-time" }, { limit: 300 }],
                throttle_changes: 2000,
                fields: {
                    id: null,
                    project_id: null,
                    time: null,
                    account_id: null,
                    event: null,
                },
            },
            set: {
                fields: {
                    id: function (obj) {
                        return obj.id != null ? obj.id : misc_1.uuid();
                    },
                    project_id: "project_write",
                    account_id: "account_id",
                    time: true,
                    event: true,
                },
            },
        },
    },
    fields: {
        id: {
            type: "uuid",
            desc: "which",
        },
        project_id: {
            type: "uuid",
            desc: "where",
        },
        time: {
            type: "timestamp",
            desc: "when",
        },
        account_id: {
            type: "uuid",
            desc: "who",
        },
        event: {
            type: "map",
            desc: "what",
        },
    },
});
// project_log_all -- exactly like project_log, but loads up
// to the most recent **many** log entries (so a LOT).
index_1.SCHEMA.project_log_all = misc_1.deep_copy(index_1.SCHEMA.project_log);
// This happens rarely, and user is prepared to wait.
index_1.SCHEMA.project_log_all.db_standby = "unsafe";
index_1.SCHEMA.project_log_all.virtual = "project_log";
// no time constraint:
if (((_a = index_1.SCHEMA.project_log_all.user_query) === null || _a === void 0 ? void 0 : _a.get) == null) {
    throw Error("make typescript happy");
}
index_1.SCHEMA.project_log_all.user_query.get.pg_where = ["projects"];
index_1.SCHEMA.project_log_all.user_query.get.options = [
    { order_by: "-time" },
    { limit: 7500 },
];
