"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var misc_1 = require("../misc");
var index_1 = require("./index");
var types_1 = require("./types");
// Get publicly available information about a project.
types_1.Table({
    name: "public_projects",
    rules: {
        anonymous: true,
        virtual: "projects",
        user_query: {
            get: {
                pg_where: [{ "project_id = $::UUID": "project_id-public" }],
                fields: {
                    project_id: true,
                    title: true,
                    description: true,
                },
            },
        },
    },
});
types_1.Table({
    name: "public_paths",
    fields: {
        id: {
            type: "string",
            pg_type: "CHAR(40)",
            desc: "sha1 hash derived from project_id and path",
        },
        project_id: {
            type: "uuid",
        },
        path: {
            type: "string",
        },
        description: {
            type: "string",
        },
        disabled: {
            type: "boolean",
            desc: "if true then disabled",
        },
        unlisted: {
            type: "boolean",
            desc: "if true then unlisted, so does not appear in /share listing page.",
        },
        license: {
            type: "string",
            desc: "The license that the content of the share is made available under.",
        },
        created: {
            type: "timestamp",
            desc: "when this path was created",
        },
        last_edited: {
            type: "timestamp",
            desc: "when this path was last edited",
        },
        last_saved: {
            type: "timestamp",
            desc: "when this path was last saved (or deleted if disabled) by manage-storage",
        },
        counter: {
            type: "number",
            desc: "the number of times this public path has been accessed",
        },
        vhost: {
            // For now, this will only be used *manually* for now; at some point users will be able to specify this,
            // though maybe they have to prove they own it.
            // For now we will only serve the vhost files statically with no special support, except we do support
            // basic http auth.   However, we will add
            // special server support for certain file types (e.g., math typesetting, markdown, sagews, ipynb, etc.)
            // so static websites can just be written in a mix of md, html, ipynb, etc. files with no javascript needed.
            // This could be a non-default option.
            // IMPORTANT: right now if vhost is set, then the share is not visible at all to the normal share server.
            type: "string",
            desc: 'Request for the given host (which must not container "cocalc") will be served by this public share. Only one public path can have a given vhost.  The vhost field can be a comma-separated string for multiple vhosts.',
            unique: true,
        },
        auth: {
            type: "map",
            desc: "Map from relative path inside the share to array of {path:[{name:[string], pass:[password-hash]}, ...], ...}.  Used both by vhost and share server, but not user editable yet.  Later it will be user editable.  The password hash is from smc-hub/auth.password_hash (so 1000 iterations of sha512)",
        },
        token: {
            type: "string",
            desc: "Random token that must be passed in as query parameter to see this share; this increases security.  Only used for unlisted shares.",
        },
        compute_image: {
            type: "string",
            desc: "The underlying compute image, which defines the associated software stack. e.g. 'default', 'custom/some-id/latest', ...",
        },
    },
    rules: {
        primary_key: "id",
        db_standby: "unsafe",
        anonymous: true,
        pg_indexes: [
            "project_id",
            "(substring(project_id::text from 1 for 1))",
            "(substring(project_id::text from 1 for 2))",
        ],
        user_query: {
            get: {
                pg_where: [{ "project_id = $::UUID": "project_id" }],
                throttle_changes: 2000,
                fields: {
                    id: null,
                    project_id: null,
                    path: null,
                    description: null,
                    disabled: null,
                    unlisted: null,
                    license: null,
                    last_edited: null,
                    created: null,
                    last_saved: null,
                    counter: null,
                    // don't use DEFAULT_COMPUTE_IMAGE, because old shares without that val set will always be "default"!
                    compute_image: "default",
                },
            },
            set: {
                fields: {
                    id: function (obj, db) {
                        return db.sha1(obj.project_id, obj.path);
                    },
                    project_id: "project_write",
                    path: true,
                    description: true,
                    disabled: true,
                    unlisted: true,
                    license: true,
                    last_edited: true,
                    created: true,
                    compute_image: true,
                },
                required_fields: {
                    id: true,
                    project_id: true,
                    path: true,
                },
            },
        },
    },
});
index_1.SCHEMA.public_paths.project_query = misc_1.deep_copy(index_1.SCHEMA.public_paths.user_query);
/* Look up a single public path by its id. */
types_1.Table({
    name: "public_paths_by_id",
    rules: {
        anonymous: true,
        virtual: "public_paths",
        user_query: {
            get: {
                check_hook: function (_db, obj, _account_id, _project_id, cb) {
                    if (typeof obj.id == "string" && obj.id.length == 40) {
                        cb(); // good
                    }
                    else {
                        cb("id must be a sha1 hash");
                    }
                },
                fields: {
                    id: null,
                    project_id: null,
                    path: null,
                    description: null,
                    disabled: null,
                    unlisted: null,
                    license: null,
                    last_edited: null,
                    created: null,
                    last_saved: null,
                    counter: null,
                    compute_image: null,
                },
            },
        },
    },
});
// WARNING: the fields in queries to all_publics_paths are ignored; all of them are always returned.
types_1.Table({
    name: "all_public_paths",
    rules: {
        anonymous: true,
        virtual: "public_paths",
        user_query: {
            get: {
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var _a, _b, err_1;
                        return __generator(this, function (_c) {
                            switch (_c.label) {
                                case 0:
                                    _c.trys.push([0, 2, , 3]);
                                    _a = cb;
                                    _b = [undefined];
                                    return [4 /*yield*/, database.get_all_public_paths(opts.account_id)];
                                case 1:
                                    _a.apply(void 0, _b.concat([_c.sent()]));
                                    return [3 /*break*/, 3];
                                case 2:
                                    err_1 = _c.sent();
                                    cb(err_1);
                                    return [3 /*break*/, 3];
                                case 3: return [2 /*return*/];
                            }
                        });
                    });
                },
                fields: {
                    id: null,
                    project_id: null,
                    path: null,
                    description: null,
                    disabled: null,
                    unlisted: null,
                    license: null,
                    last_edited: null,
                    created: null,
                    last_saved: null,
                    counter: null,
                    compute_image: null,
                },
            },
        },
    },
});
