"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.site_settings_conf = exports.KUCALC_ON_PREMISES = exports.KUCALC_COCALC_COM = exports.KUCALC_DISABLED = exports.split_iframe_comm_hosts = exports.only_nonneg_int = exports.only_ints = exports.to_int = exports.only_booleans = exports.to_bool = exports.only_commercial = exports.show_theming_vars = exports.not_cocalc_com = exports.only_cocalc_com = exports.only_onprem = exports.only_for_password_reset_smtp = exports.only_for_sendgrid = exports.only_for_smtp = exports.is_email_enabled = void 0;
// Default settings to customize a given site, typically a private install of CoCalc.
// The following two requires *should* be imports for better
// typing info.  Unfortunately, this currently breaks starting projects.
//import { is_valid_email_address } from "smc-util/misc";
//import { DNS } from "../theme";
var is_valid_email_address = require("smc-util/misc").is_valid_email_address;
var DNS = require("../theme").DNS;
var fallback = function (conf, name) { var _a; return (_a = conf[name]) !== null && _a !== void 0 ? _a : exports.site_settings_conf[name].default; };
// little helper fuctions, used in the site settings & site settings extras
var is_email_enabled = function (conf) {
    return exports.to_bool(conf.email_enabled) && conf.email_backend !== "none";
};
exports.is_email_enabled = is_email_enabled;
var only_for_smtp = function (conf) {
    return exports.is_email_enabled(conf) && conf.email_backend === "smtp";
};
exports.only_for_smtp = only_for_smtp;
var only_for_sendgrid = function (conf) {
    return exports.is_email_enabled(conf) && conf.email_backend === "sendgrid";
};
exports.only_for_sendgrid = only_for_sendgrid;
var only_for_password_reset_smtp = function (conf) {
    return exports.to_bool(conf.email_enabled) && conf.password_reset_override === "smtp";
};
exports.only_for_password_reset_smtp = only_for_password_reset_smtp;
var only_onprem = function (conf) {
    return conf.kucalc === exports.KUCALC_ON_PREMISES;
};
exports.only_onprem = only_onprem;
var only_cocalc_com = function (conf) {
    return conf.kucalc === exports.KUCALC_COCALC_COM;
};
exports.only_cocalc_com = only_cocalc_com;
var not_cocalc_com = function (conf) { return !exports.only_cocalc_com(conf); };
exports.not_cocalc_com = not_cocalc_com;
var show_theming_vars = function (conf) {
    return exports.to_bool(fallback(conf, "theming"));
};
exports.show_theming_vars = show_theming_vars;
var only_commercial = function (conf) {
    return exports.to_bool(fallback(conf, "commercial"));
};
exports.only_commercial = only_commercial;
var to_bool = function (val) { return val === "true" || val === "yes"; };
exports.to_bool = to_bool;
exports.only_booleans = ["yes", "no"]; // we also understand true and false
var to_int = function (val) { return parseInt(val); };
exports.to_int = to_int;
var only_ints = function (val) {
    return (function (v) { return !isNaN(v) && Number.isFinite(v) && Number.isInteger(val); })(exports.to_int(val));
};
exports.only_ints = only_ints;
var only_nonneg_int = function (val) {
    return (function (v) { return exports.only_ints(v) && v >= 0; })(exports.to_int(val));
};
exports.only_nonneg_int = only_nonneg_int;
// TODO a cheap'n'dirty validation is good enough
var valid_dns_name = function (val) { return val.match(/^[a-zA-Z0-9.-]+$/g); };
var split_iframe_comm_hosts = function (hosts) {
    return hosts.match(/[a-z0-9.-]+/g) || [];
};
exports.split_iframe_comm_hosts = split_iframe_comm_hosts;
var split_strings = function (str) { return str.match(/[a-zA-Z0-9]+/g) || []; };
function num_dns_hosts(val) {
    return "Found " + exports.split_iframe_comm_hosts(val).length + " hosts.";
}
exports.KUCALC_DISABLED = "no";
exports.KUCALC_COCALC_COM = "yes";
exports.KUCALC_ON_PREMISES = "onprem";
var KUCALC_VALID_VALS = [
    exports.KUCALC_COCALC_COM,
    exports.KUCALC_ON_PREMISES,
    exports.KUCALC_DISABLED,
];
var help_email_name = "Help email";
var organization_email_desc = "How to contact your organization (fallback: '" + help_email_name + "').";
exports.site_settings_conf = {
    // ========= THEMING ===============
    dns: {
        name: "Domain name",
        desc: "DNS for your server, e.g. cocalc.universe.edu",
        default: "",
        valid: valid_dns_name,
        // to make sure if dns isn't set or an empty string, it falls back to a known name
        to_val: function (val) { return val || DNS; },
    },
    theming: {
        name: "Show Theming",
        desc: "If 'No', the fields below are hidden, not disabled!",
        default: "yes",
        valid: exports.only_booleans,
        to_val: exports.to_bool,
    },
    site_name: {
        name: "Site name",
        desc: "The heading name of your CoCalc site.",
        default: "Open CoCalc",
        clearable: true,
        show: exports.show_theming_vars,
    },
    site_description: {
        name: "Site description",
        desc: "A tagline describing your site.",
        default: "Collaborative Calculation Online",
        clearable: true,
        show: exports.show_theming_vars,
    },
    help_email: {
        name: help_email_name,
        desc: "Email address that user is directed to use for support requests",
        default: "",
        valid: is_valid_email_address,
        clearable: true,
        show: exports.show_theming_vars,
    },
    terms_of_service_url: {
        name: "Terms of Service URL",
        desc: "URL to the page describing ToS, Policies, etc. (leave empty to not require)",
        default: "",
        clearable: true,
        show: exports.show_theming_vars,
    },
    terms_of_service: {
        name: "ToS information",
        desc: "The text displayed for the terms of service link (empty falls back a boilerplate using the URL).",
        default: "By creating an account you agree to the <em>Terms of Service</em>.",
        clearable: true,
        show: exports.show_theming_vars,
    },
    account_creation_email_instructions: {
        name: "Account creation",
        desc: "Instructions displayed next to the box where a user creates their account using their name and email address.",
        default: "Create an Account",
        clearable: true,
        show: exports.show_theming_vars,
    },
    organization_name: {
        name: "Organization name",
        desc: "The name of your organization, e.g. 'Hogwarts School of Witchcraft and Wizardry'.",
        default: "",
        clearable: true,
        show: exports.show_theming_vars,
    },
    organization_email: {
        name: "Contact email address",
        desc: organization_email_desc,
        default: "",
        clearable: true,
        valid: is_valid_email_address,
        show: exports.show_theming_vars,
    },
    organization_url: {
        name: "Organization website",
        desc: "URL link to your organization",
        default: "",
        clearable: true,
        show: exports.show_theming_vars,
    },
    logo_square: {
        name: "Logo (square)",
        desc: "URL of a square logo (SVG or PNG, about 200x200 px)",
        default: "",
        clearable: true,
        show: exports.show_theming_vars,
    },
    logo_rectangular: {
        name: "Logo (rectangular)",
        desc: "URL of a rectangular logo (about 450x75 px, SVG or PNG)",
        default: "",
        clearable: true,
        show: exports.show_theming_vars,
    },
    splash_image: {
        name: "Index page picture",
        desc: "URL of an image displayed on the index page (about 1200x800 px)",
        default: "",
        clearable: true,
        show: exports.show_theming_vars,
    },
    index_info_html: {
        name: "Index page info",
        desc: "An HTML string displayed on the index page.",
        default: "",
        clearable: true,
        show: exports.show_theming_vars,
        multiline: 5,
    },
    // ============== END THEMING ============
    versions: {
        name: "Client Versions",
        desc: "",
        default: "",
        type: "header",
    },
    version_min_project: {
        name: "Required project version",
        desc: "Minimal version required by projects (if project older, will be force restarted).",
        default: "0",
        valid: exports.only_nonneg_int,
        show: function () { return true; },
    },
    version_min_browser: {
        name: "Required browser version",
        desc: "Minimal version required for browser clients (if older, forced disconnect).",
        default: "0",
        valid: exports.only_nonneg_int,
        show: function () { return true; },
    },
    version_recommended_browser: {
        name: "Recommended version",
        desc: "Older clients receive an upgrade warning.",
        default: "0",
        valid: exports.only_nonneg_int,
        show: function () { return true; },
    },
    kucalc: {
        name: "KuCalc UI",
        desc: "Configure which UI elements to show in order to match the Kubernetes backend. '" + exports.KUCALC_COCALC_COM + "' for cocalc.com production site, '" + exports.KUCALC_ON_PREMISES + "' for on-premises Kubernetes, or '" + exports.KUCALC_DISABLED + "' for Docker",
        default: exports.KUCALC_DISABLED,
        valid: KUCALC_VALID_VALS,
    },
    google_analytics: {
        name: "Google Analytics",
        desc: "The GA tag, only for the cocalc.com production site",
        default: "",
        show: exports.only_cocalc_com,
    },
    commercial: {
        name: "Commercial",
        desc: "Whether or not to include user interface elements related to for-pay upgrades and other features.  Set to 'yes' to include these elements.",
        default: "no",
        valid: exports.only_booleans,
        to_val: exports.to_bool,
        show: exports.only_cocalc_com,
    },
    max_trial_projects: {
        name: "Maximum Trial Projects",
        desc: "Limit where we start blocking trial projects from running in nonfree countries. (0 means disabled)",
        default: "0",
        to_val: exports.to_int,
        valid: exports.only_nonneg_int,
        show: exports.only_cocalc_com,
    },
    nonfree_countries: {
        name: "Nonfree Countries",
        desc: "ISO 3166-1 Alpha 2 country codes where extra usage restrictions apply",
        default: "",
        to_val: split_strings,
        show: exports.only_cocalc_com,
    },
    onprem_quota_heading: {
        name: "On-prem Quotas",
        desc: "",
        default: "",
        show: exports.only_onprem,
        type: "header",
    },
    default_quotas: {
        name: "Default Quotas",
        desc: "A JSON-formatted default quota for projects. This is only for on-prem setups. The fields actual meaning is defined in hub's quota.ts code",
        default: "{}",
        show: exports.only_onprem,
    },
    max_upgrades: {
        name: "Maximum Quota Upgrades",
        desc: "A JSON-formatted upper limit of all quotas. This is only for on-prem setups. The fields are defined in the upgrade spec.",
        default: "{}",
        show: exports.only_onprem,
    },
    ssh_gateway: {
        name: "SSH Gateway",
        desc: "Show corresponding UI elements",
        default: "no",
        valid: exports.only_booleans,
        to_val: exports.to_bool,
    },
    iframe_comm_hosts: {
        name: "IFrame communication hosts",
        desc: "List of allowed DNS names, which are allowed to communicate back and forth with an embedded CoCalc instance. If starting with a dot, also all subdomains. It picks all matching '[a-zA-Z0-9.-]+'",
        default: "",
        to_val: exports.split_iframe_comm_hosts,
        to_display: num_dns_hosts,
    },
    email_enabled: {
        name: "Email sending enabled",
        desc: "Controls visibility of UI elements and if any emails are sent. This is independent of any particular email configuration!",
        default: "no",
        valid: exports.only_booleans,
        to_val: exports.to_bool,
    },
    verify_emails: {
        name: "Verify email addresses",
        desc: "If 'true', email verification tokens are sent out + account settings UI shows it – email sending must be enabled",
        default: "no",
        show: exports.is_email_enabled,
        valid: exports.only_booleans,
        to_val: exports.to_bool,
    },
    email_signup: {
        name: "Allow email signup",
        desc: "Users can sign up via email & password. Could be subject to an 'account creation token'.",
        default: "yes",
        valid: exports.only_booleans,
        to_val: exports.to_bool,
    },
};
