"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.describe_quota = void 0;
/*
Table of all site licenses.

From wikipedia https://en.wikipedia.org/wiki/Site_license: "A site license is a type of software license that allows the user to install a software package in several computers simultaneously, such as at a particular site (facility) or across a corporation.[1] Depending on the amount of fees paid, the license may be unlimited or may limit simultaneous access to a certain number of users. The latter is called a concurrent site license.[2]"

That's sort of what we're doing here.  We've defined a license that let's users
*use* several upgraded projects with simultaneous access limits
(e.g., number of projects at once).

A possibly confusing point is that a single University might have many site
licenses for different purposes (e.g., one license for faculty and one license
for students).
*/
var misc_1 = require("../misc");
var types_1 = require("./types");
var index_1 = require("./index");
function describe_quota(quota, short) {
    var desc = "";
    if (!short) {
        desc =
            (quota.user == "business" ? "Business" : "Academic") +
                " license providing ";
    }
    var v = [];
    if (quota.ram) {
        v.push(quota.ram + "GB RAM");
    }
    if (quota.cpu) {
        v.push(quota.cpu + " shared " + misc_1.plural(quota.cpu, "CPU"));
    }
    if (quota.disk) {
        v.push(quota.disk + "GB disk");
    }
    if (quota.dedicated_ram) {
        v.push(quota.dedicated_ram + "GB dedicated RAM");
    }
    if (quota.dedicated_cpu) {
        v.push(quota.dedicated_cpu + " dedicated " + misc_1.plural(quota.dedicated_cpu, "CPU"));
    }
    if (quota.member) {
        v.push("member" + (short ? "" : " hosting"));
    }
    if (quota.always_running) {
        v.push("always running");
    }
    v.push("network"); // always provided, because we trust customers.
    desc += v.join(", ");
    return desc;
}
exports.describe_quota = describe_quota;
types_1.Table({
    name: "site_licenses",
    fields: {
        id: {
            type: "uuid",
            desc: "ID that determines the license.",
        },
        title: {
            type: "string",
            desc: "Descriptive name of the license, e.g., the class and university or other information.",
        },
        description: {
            type: "string",
            desc: "Longer description of the license, extra notes, etc.",
        },
        info: {
            type: "map",
            desc: "Structured object for admins to store structured information about this license.  This serves a similar purpose to description, but must be a valid JSON object map.",
        },
        expires: {
            type: "timestamp",
            desc: "Date when the license expires.  At this point in time the license no longer upgrades projects, and any running upgraded projects have their upgrades removed, which may result in thosoe projects being stoped.",
        },
        activates: {
            type: "timestamp",
            desc: "Date when this license starts working.  Before this date, the license can be applied to projects, but nothing happens.",
        },
        created: {
            type: "timestamp",
            desc: "when this license was first created",
        },
        last_used: {
            type: "timestamp",
            desc: "when this license was last used to upgrade a project when the project was starting.  Obviously, we don't update this *every* single time a project starts - it's throttled, so it'll only be updated periodically (maybe once per minute).",
        },
        managers: {
            type: "array",
            pg_type: "TEXT[]" /* TODO/NOTE: I made a mistake -- this should have been UUID[]! */,
            desc: "A list of the account_id's of users that are allowed to manage how this site license is being used.",
        },
        restricted: {
            type: "boolean",
            desc: "NOTE IMPLEMENTED YET: If true, then only managers are allowed to add this site license to a project.  If false, anybody who knows the license key can use it on projects.",
        },
        upgrades: {
            type: "map",
            desc: "Map of the upgrades that are applied to a project when it has this site license; this is the same as the settings field of a project, so e.g., {cores: 1.5, cpu_shares: 768, disk_quota: 1000, memory: 2000, mintime: 36000000, network: 0}.  This matches with our older purchases and our internal system.  Instead of this one can give quota.",
        },
        quota: {
            type: "map",
            desc: "The exact quota a project using this license gets -- {ram: total amount of memory in GB, cpu: total number of shared vCPUs, disk:total GB of disk space, always_running:true/false, member:true/false, user:'academic'|'business'}.  (Plan is) that such a license does not provide upgrades, but instead a fixed quota.",
        },
        run_limit: {
            type: "integer",
            desc: "The maximum number of running projects that may be simultaneously upgraded using this license.  When this is exceeded, older projects have the license automatically removed.  If removal changes project upgrades, then those projects have the upgrades removed and are stopped.",
        },
        apply_limit: {
            type: "integer",
            desc: "The maximum number of projects that may simultaneously have this license applied to them.  When this is exceeded, older projects have the license automatically removed.  If this changes how the projects are upgraded, then those projects are stopped.",
        },
    },
    rules: {
        desc: "Site Licenses",
        anonymous: false,
        primary_key: "id",
        pg_indexes: [],
        user_query: {
            get: {
                pg_where: [],
                admin: true,
                options: [{ order_by: "-last_used" }, { limit: 500 }],
                fields: {
                    id: null,
                    title: null,
                    description: null,
                    info: null,
                    expires: null,
                    activates: null,
                    created: null,
                    last_used: null,
                    managers: null,
                    restricted: null,
                    upgrades: null,
                    quota: null,
                    run_limit: null,
                    apply_limit: null,
                },
            },
            set: {
                admin: true,
                fields: {
                    id: null,
                    title: null,
                    description: null,
                    info: null,
                    expires: null,
                    activates: null,
                    created: null,
                    last_used: null,
                    managers: null,
                    restricted: null,
                    upgrades: null,
                    quota: null,
                    run_limit: null,
                    apply_limit: null,
                },
            },
        },
    },
});
var MATCHING_SITE_LICENSES_LIMIT = 20; // arbitrary limit.
types_1.Table({
    name: "matching_site_licenses",
    fields: {
        search: {
            type: "string",
            desc: "A search query",
        },
        id: true,
        title: true,
        description: true,
        info: true,
        expires: true,
        activates: true,
        created: true,
        last_used: true,
        managers: true,
        restricted: true,
        upgrades: true,
        quota: true,
        run_limit: true,
        apply_limit: true,
    },
    rules: {
        virtual: true,
        desc: "Site Licenses that match a query (default limit of ${MATCHING_SITE_LICENSES_LIMIT} most active)",
        anonymous: false,
        primary_key: ["id"],
        user_query: {
            get: {
                admin: true,
                fields: {
                    search: null,
                    id: null,
                    title: null,
                    description: null,
                    info: null,
                    expires: null,
                    activates: null,
                    created: null,
                    last_used: null,
                    managers: null,
                    restricted: null,
                    upgrades: null,
                    quota: null,
                    run_limit: null,
                    apply_limit: null,
                },
                // Actual query is implemented using this code below rather than an actual query directly.
                // We also completely ignore the user-requested fields and just return everything, since
                // in our application this is fine right now (since all fields are always requested).
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var limit, _a, _b, option, _c, _d, err_1;
                        var e_1, _e;
                        return __generator(this, function (_f) {
                            switch (_f.label) {
                                case 0:
                                    _f.trys.push([0, 2, , 3]);
                                    if (!opts.multi) {
                                        throw Error("only query requesting multiple results is implemented");
                                    }
                                    limit = MATCHING_SITE_LICENSES_LIMIT;
                                    if (opts.options != null) {
                                        try {
                                            for (_a = __values(opts.options), _b = _a.next(); !_b.done; _b = _a.next()) {
                                                option = _b.value;
                                                if (option["limit"]) {
                                                    limit = option["limit"];
                                                }
                                            }
                                        }
                                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                                        finally {
                                            try {
                                                if (_b && !_b.done && (_e = _a.return)) _e.call(_a);
                                            }
                                            finally { if (e_1) throw e_1.error; }
                                        }
                                    }
                                    _c = cb;
                                    _d = [undefined];
                                    return [4 /*yield*/, database.matching_site_licenses(opts.query.search, limit)];
                                case 1:
                                    _c.apply(void 0, _d.concat([_f.sent()]));
                                    return [3 /*break*/, 3];
                                case 2:
                                    err_1 = _f.sent();
                                    cb(err_1);
                                    return [3 /*break*/, 3];
                                case 3: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
        },
    },
});
// A virtual table that can be queried only by admins and gets global information
// about how site licenses are currently being used by active projects.
types_1.Table({
    name: "site_license_usage_stats",
    fields: {
        running: {
            type: "map",
            desc: "Map from license_id to a count of how many *running* projects are using that license right now.  Only includes licenses that are being used.",
        },
        time: {
            type: "timestamp",
            desc: "When the data was grabbed.",
        },
    },
    rules: {
        virtual: true,
        desc: "Site License usage information for running projects",
        anonymous: false,
        primary_key: ["time"],
        user_query: {
            get: {
                admin: true,
                fields: {
                    running: null,
                    time: null,
                },
                // Actual query is implemented using this code below rather than an actual query directly.
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var obj, _a, err_2;
                        return __generator(this, function (_b) {
                            switch (_b.label) {
                                case 0:
                                    obj = Object.assign({}, opts.query);
                                    _b.label = 1;
                                case 1:
                                    _b.trys.push([1, 3, , 4]);
                                    _a = obj;
                                    return [4 /*yield*/, database.site_license_usage_stats()];
                                case 2:
                                    _a.running = _b.sent();
                                    obj.time = new Date();
                                    cb(undefined, obj);
                                    return [3 /*break*/, 4];
                                case 3:
                                    err_2 = _b.sent();
                                    cb(err_2);
                                    return [3 /*break*/, 4];
                                case 4: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
        },
    },
});
types_1.Table({
    name: "projects_using_site_license",
    fields: {
        license_id: {
            type: "string",
            desc: "the id of the license -- must be specified",
        },
        cutoff: {
            type: "timestamp",
            desc: "include projects that were running with this license applied at some point since cutoff; E.g., if cutoff is right now, then we get the currently running projects, and if cuttoff is a timestamp a week ago, we get all projects that ran using this license during the last week.  Default: NOW().",
        },
        limit: {
            type: "integer",
            desc: "limit on the number of results to return, to avoid overloading things. Default: 1000.  This is only used by admins so for now having a large limit and no paging is probably fine.",
        },
        project_id: index_1.SCHEMA.projects.fields.project_id,
        title: index_1.SCHEMA.projects.fields.title,
        description: index_1.SCHEMA.projects.fields.description,
        users: index_1.SCHEMA.projects.fields.users,
        last_active: index_1.SCHEMA.projects.fields.last_active,
        last_edited: index_1.SCHEMA.projects.fields.last_edited,
    },
    rules: {
        virtual: true,
        desc: "Site License usage information for running projects with a particular license",
        anonymous: false,
        primary_key: ["license_id", "project_id"],
        user_query: {
            get: {
                admin: true,
                fields: {
                    license_id: null,
                    cutoff: null,
                    limit: null,
                    project_id: null,
                    title: null,
                    description: null,
                    users: null,
                    last_active: null,
                    last_edited: null,
                },
                // Actual query is implemented using this code below rather than an actual query directly.
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var obj, fields, _a, _b, field, projects, projects_1, projects_1_1, project, err_3;
                        var e_2, _c, e_3, _d;
                        return __generator(this, function (_e) {
                            switch (_e.label) {
                                case 0:
                                    if (!opts.multi) {
                                        cb("query must be an array (you must request to get multiple values back)");
                                        return [2 /*return*/];
                                    }
                                    obj = opts.query;
                                    if (typeof obj != "object" || !misc_1.is_valid_uuid_string(obj.license_id)) {
                                        cb("query must be of the form [{license_id:uuid, ...}]");
                                        return [2 /*return*/];
                                    }
                                    if (!obj.limit) {
                                        obj.limit = 1000;
                                    }
                                    fields = [];
                                    try {
                                        for (_a = __values([
                                            // this approach ensures requests for bad fields don't cause SQL injection...
                                            "project_id",
                                            "title",
                                            "description",
                                            "users",
                                            "last_active",
                                            "last_edited",
                                        ]), _b = _a.next(); !_b.done; _b = _a.next()) {
                                            field = _b.value;
                                            if (obj[field] === null) {
                                                // === is important here since we don't want to pick up not set field!
                                                fields.push(field);
                                            }
                                        }
                                    }
                                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                                    finally {
                                        try {
                                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                                        }
                                        finally { if (e_2) throw e_2.error; }
                                    }
                                    _e.label = 1;
                                case 1:
                                    _e.trys.push([1, 3, , 4]);
                                    return [4 /*yield*/, database.projects_using_site_license({
                                            license_id: obj.license_id,
                                            fields: fields,
                                            cutoff: obj.cutoff,
                                            limit: obj.limit,
                                            truncate: 80,
                                        })];
                                case 2:
                                    projects = _e.sent();
                                    try {
                                        for (projects_1 = __values(projects), projects_1_1 = projects_1.next(); !projects_1_1.done; projects_1_1 = projects_1.next()) {
                                            project = projects_1_1.value;
                                            // for consistency with how queries work, we fill this in.
                                            project.license_id = obj.license_id;
                                        }
                                    }
                                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                                    finally {
                                        try {
                                            if (projects_1_1 && !projects_1_1.done && (_d = projects_1.return)) _d.call(projects_1);
                                        }
                                        finally { if (e_3) throw e_3.error; }
                                    }
                                    cb(undefined, projects);
                                    return [3 /*break*/, 4];
                                case 3:
                                    err_3 = _e.sent();
                                    cb(err_3);
                                    return [3 /*break*/, 4];
                                case 4: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
        },
    },
});
types_1.Table({
    name: "number_of_projects_using_site_license",
    fields: {
        license_id: {
            type: "string",
            desc: "the id of the license -- must be specified",
        },
        cutoff: {
            type: "timestamp",
            desc: "include projects that were running with this license applied at some point since cutoff; E.g., if cutoff is right now, then we get the currently running projects, and if cuttoff is a timestamp a week ago, we get all projects that ran using this license during the last week.  Default: NOW().",
        },
        number: {
            type: "integer",
            desc: "how many projects using the site license at some point since cutoff",
        },
    },
    rules: {
        virtual: true,
        desc: "Virtual table for determining the number of projects that recently used a given site license",
        anonymous: false,
        primary_key: ["license_id", "cutoff"],
        user_query: {
            get: {
                admin: true,
                fields: {
                    license_id: null,
                    cutoff: null,
                    number: null,
                },
                // Actual query is implemented using this code below rather than an actual query directly.
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var obj, _a, err_4;
                        return __generator(this, function (_b) {
                            switch (_b.label) {
                                case 0:
                                    if (opts.multi) {
                                        cb("query must NOT be an array (do not request multiple values back)");
                                        return [2 /*return*/];
                                    }
                                    obj = opts.query;
                                    if (typeof obj != "object" ||
                                        !misc_1.is_valid_uuid_string(obj.license_id) ||
                                        obj.number != null) {
                                        cb("query must be of the form {license_id:uuid, cutoff?:<date>, count:null...}]");
                                        return [2 /*return*/];
                                    }
                                    _b.label = 1;
                                case 1:
                                    _b.trys.push([1, 3, , 4]);
                                    _a = obj;
                                    return [4 /*yield*/, database.number_of_projects_using_site_license({
                                            license_id: obj.license_id,
                                            cutoff: obj.cutoff,
                                        })];
                                case 2:
                                    _a.number = _b.sent();
                                    cb(undefined, obj);
                                    return [3 /*break*/, 4];
                                case 3:
                                    err_4 = _b.sent();
                                    cb(err_4);
                                    return [3 /*break*/, 4];
                                case 4: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
        },
    },
});
// Get publicly available information about a site license.
// User just has to know the license id to get this info.
//
if (((_a = index_1.SCHEMA === null || index_1.SCHEMA === void 0 ? void 0 : index_1.SCHEMA.site_licenses) === null || _a === void 0 ? void 0 : _a.fields) == null)
    throw Error("bug"); // for typescript
types_1.Table({
    name: "site_license_public_info",
    fields: {
        id: index_1.SCHEMA.site_licenses.fields.id,
        title: index_1.SCHEMA.site_licenses.fields.title,
        description: index_1.SCHEMA.site_licenses.fields.description,
        expires: index_1.SCHEMA.site_licenses.fields.expires,
        activates: index_1.SCHEMA.site_licenses.fields.activates,
        upgrades: index_1.SCHEMA.site_licenses.fields.upgrades,
        quota: index_1.SCHEMA.site_licenses.fields.quota,
        run_limit: index_1.SCHEMA.site_licenses.fields.run_limit,
        managers: index_1.SCHEMA.site_licenses.fields.managers,
        running: {
            type: "integer",
            desc: "Number of running projects currently using this license.   Regarding security, we assume that if the user knows the license id, then they are allowed to know how many projects are using it.",
        },
        is_manager: {
            type: "boolean",
            desc: "True if user making the query is a manager of this license.  Frontend UI might tell them this and show license code and other links.",
        },
    },
    rules: {
        desc: "Publicly available information about site licenses",
        anonymous: false,
        primary_key: ["id"],
        virtual: true,
        user_query: {
            get: {
                admin: false,
                fields: {
                    id: null,
                    title: null,
                    description: null,
                    expires: null,
                    activates: null,
                    upgrades: null,
                    quota: null,
                    run_limit: null,
                    managers: null,
                    running: null,
                    is_manager: null,
                },
                // Actual query is implemented using this code below rather than an
                // actual query directly.  TODO: Also, we're lazy and return all fields we
                // know, even if user doesn't request them all.
                // If the user making the query is a manager of this license they get a list of
                // the managers (otherwise managers isn't set for them.)
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var id, info, err_5;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    id = opts.query.id;
                                    if (!(typeof id != "string" || !misc_1.is_valid_uuid_string(id))) return [3 /*break*/, 1];
                                    cb("must be a single object query with id specified");
                                    return [3 /*break*/, 4];
                                case 1:
                                    _a.trys.push([1, 3, , 4]);
                                    return [4 /*yield*/, database.site_license_public_info(id)];
                                case 2:
                                    info = _a.sent();
                                    info.is_manager =
                                        info.managers != null &&
                                            info.managers.includes(opts.account_id);
                                    cb(undefined, info);
                                    return [3 /*break*/, 4];
                                case 3:
                                    err_5 = _a.sent();
                                    cb(err_5);
                                    return [3 /*break*/, 4];
                                case 4: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
        },
    },
});
types_1.Table({
    name: "site_license_usage_log",
    fields: {
        license_id: {
            type: "uuid",
            desc: "id of the site license",
        },
        project_id: {
            type: "uuid",
            desc: "id of the project",
        },
        start: {
            type: "timestamp",
            desc: "When the project started running using this site license",
        },
        stop: {
            type: "timestamp",
            desc: "When the project stopped running using this site license",
        },
    },
    rules: {
        desc: "Table for logging when site licenses are used to upgrade running projects.",
        primary_key: ["license_id", "project_id", "start"],
        pg_indexes: ["license_id"],
    },
});
/* Way to get all the licenses that a given user is a manager of. */
types_1.Table({
    name: "manager_site_licenses",
    fields: {
        id: true,
        title: true,
        description: true,
        info: true,
        expires: true,
        activates: true,
        created: true,
        last_used: true,
        managers: true,
        upgrades: true,
        quota: true,
        run_limit: true,
        apply_limit: true,
    },
    rules: {
        virtual: "site_licenses",
        desc: "Licenses that user doing the query is a manager of.",
        anonymous: false,
        primary_key: ["id"],
        user_query: {
            get: {
                admin: false,
                fields: {
                    id: null,
                    title: null,
                    description: null,
                    info: null,
                    expires: null,
                    activates: null,
                    created: null,
                    last_used: null,
                    managers: null,
                    upgrades: null,
                    quota: null,
                    run_limit: null,
                    apply_limit: null,
                },
                // Actual query is implemented using this code below rather than an actual query directly.
                // We also completely ignore the user-requested fields and just return everything, since
                // in our application this is fine right now (since all fields are always requested).
                instead_of_query: function (database, opts, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var _a, _b, err_6;
                        return __generator(this, function (_c) {
                            switch (_c.label) {
                                case 0:
                                    _c.trys.push([0, 2, , 3]);
                                    if (!opts.multi) {
                                        throw Error("only query requesting multiple results is implemented");
                                    }
                                    _a = cb;
                                    _b = [undefined];
                                    return [4 /*yield*/, database.manager_site_licenses(opts.account_id)];
                                case 1:
                                    _a.apply(void 0, _b.concat([_c.sent()]));
                                    return [3 /*break*/, 3];
                                case 2:
                                    err_6 = _c.sent();
                                    cb(err_6);
                                    return [3 /*break*/, 3];
                                case 3: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
            set: {
                // set is so managers of a license can easily change the title/description at any time
                admin: false,
                fields: {
                    id: true,
                    title: true,
                    description: true,
                    managers: true,
                },
                instead_of_change: function (database, _old_value, new_val, account_id, cb) {
                    return __awaiter(this, void 0, void 0, function () {
                        var err_7;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    _a.trys.push([0, 2, , 3]);
                                    return [4 /*yield*/, database.site_license_manager_set(account_id, new_val)];
                                case 1:
                                    _a.sent();
                                    cb();
                                    return [3 /*break*/, 3];
                                case 2:
                                    err_7 = _a.sent();
                                    cb(err_7);
                                    return [3 /*break*/, 3];
                                case 3: return [2 /*return*/];
                            }
                        });
                    });
                },
            },
        },
    },
});
