"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EXTRAS = exports.pii_retention_parse = void 0;
// Site Settings Config for the servers (hubs)
// They are only visible and editable for admins and services.
// In particular, this includes the email backend config, Stripe, etc.
var site_defaults_1 = require("./site-defaults");
var _a = require("smc-util/misc"), is_valid_email_address = _a.is_valid_email_address, expire_time = _a.expire_time;
var pii_retention_parse = function (retention) {
    if (retention == "never" || retention == null)
        return false;
    var _a = __read(retention.split(" "), 2), num_str = _a[0], mult_str = _a[1];
    var num = parseInt(num_str);
    var mult = (function () {
        var m = mult_str.toLowerCase();
        if (m.startsWith("year"))
            return 365;
        if (m.startsWith("month"))
            return 30;
        if (m.startsWith("day"))
            return 1;
        throw new Error("unknown multiplyer \"" + m + "\"");
    })();
    var secs = num * (mult * 24 * 60 * 60);
    if (isNaN(secs) || secs == null) {
        throw new Error("pii_expire problem: cannot derive future time from \"{retention}\"");
    }
    return secs;
};
exports.pii_retention_parse = pii_retention_parse;
var pii_retention_display = function (retention) {
    var secs = exports.pii_retention_parse(retention);
    if (secs === false) {
        return "will never expire";
    }
    else {
        return "Future date " + expire_time(secs).toLocaleString();
    }
};
// not public, but admins can edit them
exports.EXTRAS = {
    pii_retention: {
        name: "PII Retention",
        desc: "How long to keep personally identifiable information (deletes certain database entries)",
        default: "never",
        // values must be understood by smc-hub/utils.ts pii_expire
        valid: [
            "never",
            "30 days",
            "3 month",
            "6 month",
            "1 year",
            "2 years",
            "5 years",
            "10 years",
        ],
        to_val: exports.pii_retention_parse,
        to_display: pii_retention_display,
    },
    stripe_heading: {
        // this is consmetic, otherwise it looks weird.
        name: "Stripe Keys",
        desc: "",
        default: "",
        show: site_defaults_1.only_commercial,
        type: "header",
    },
    stripe_publishable_key: {
        name: "Stripe Publishable",
        desc: "Stripe calls this key 'publishable'",
        default: "",
        password: false,
        show: site_defaults_1.only_commercial,
    },
    stripe_secret_key: {
        name: "Stripe Secret",
        desc: "Stripe calls this key 'secret'",
        default: "",
        show: site_defaults_1.only_commercial,
        password: true,
    },
    email_backend: {
        name: "Email backend type",
        desc: "The type of backend for sending emails ('none' means there is none).",
        default: "",
        valid: ["none", "sendgrid", "smtp"],
        show: site_defaults_1.is_email_enabled,
    },
    sendgrid_key: {
        name: "Sendgrid API key",
        desc: "You need a Sendgrid account and then enter a valid API key here",
        password: true,
        default: "",
        show: site_defaults_1.only_for_sendgrid,
    },
    email_smtp_server: {
        name: "SMTP server",
        desc: "the hostname to talk to",
        default: "",
        show: site_defaults_1.only_for_smtp,
    },
    email_smtp_from: {
        name: "SMTP server FROM",
        desc: "the FROM and REPLYTO email address",
        default: "",
        valid: is_valid_email_address,
        show: site_defaults_1.only_for_smtp,
    },
    email_smtp_login: {
        name: "SMTP username",
        desc: "the username, for PLAIN login",
        default: "",
        show: site_defaults_1.only_for_smtp,
    },
    email_smtp_password: {
        name: "SMTP password",
        desc: "the password, for PLAIN login",
        default: "",
        show: site_defaults_1.only_for_smtp,
        password: true,
    },
    email_smtp_port: {
        name: "SMTP port",
        desc: "Usually: For secure==true use port 465, otherwise port 587 or 25",
        default: "465",
        to_val: site_defaults_1.to_int,
        valid: site_defaults_1.only_nonneg_int,
        show: site_defaults_1.only_for_smtp,
    },
    email_smtp_secure: {
        name: "SMTP secure",
        desc: "Usually 'true'",
        default: "true",
        valid: site_defaults_1.only_booleans,
        to_val: site_defaults_1.to_bool,
        show: site_defaults_1.only_for_smtp,
    },
    // bad name, historic baggage, used in smc-hub/email.ts
    password_reset_override: {
        name: "Override email backend",
        desc: "For 'smtp', password reset and email verification emails are sent via the 'Secondary SMTP' configuration",
        default: "default",
        valid: ["default", "smtp"],
        show: site_defaults_1.is_email_enabled,
    },
    password_reset_smtp_server: {
        name: "Secondary SMTP server",
        desc: "hostname sending password reset emails",
        default: "",
        show: site_defaults_1.only_for_password_reset_smtp,
    },
    password_reset_smtp_from: {
        name: "Secondary SMTP FROM",
        desc: "This sets the FROM and REPLYTO email address",
        default: "",
        valid: is_valid_email_address,
        show: site_defaults_1.only_for_password_reset_smtp,
    },
    password_reset_smtp_login: {
        name: "Secondary SMTP username",
        desc: "username, PLAIN auth",
        default: "",
        show: site_defaults_1.only_for_password_reset_smtp,
    },
    password_reset_smtp_password: {
        name: "Secondary SMTP password",
        desc: "password, PLAIN auth",
        default: "",
        show: site_defaults_1.only_for_password_reset_smtp,
        password: true,
    },
    password_reset_smtp_port: {
        name: "Secondary SMTP port",
        desc: "Usually: For secure==true use port 465, otherwise port 587 or 25",
        default: "465",
        to_val: site_defaults_1.to_int,
        valid: site_defaults_1.only_nonneg_int,
        show: site_defaults_1.only_for_password_reset_smtp,
    },
    password_reset_smtp_secure: {
        name: "Secondary SMTP secure",
        desc: "Usually 'true'",
        default: "true",
        valid: site_defaults_1.only_booleans,
        to_val: site_defaults_1.to_bool,
        show: site_defaults_1.only_for_password_reset_smtp,
    },
};
