"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.failing_to_save = void 0;
/*
   If we are having repeated hash mismatches when trying to save a particular path,
   then this function will return true.  This can be used by the caller to
   reconnect and fix everything.  This is basically a stupid workaround for a subtle
   but very annoying bug that I don't know how to reproduce or fix...
*/
// Trigger a failure if there have been THRESHOLD.fails during
// the last THRESHOLD.interval_s seconds.
var THRESHOLD = { fails: 3, interval_s: 60 };
function is_failing(x) {
    var e_1, _a;
    var cutoff = new Date().getTime() - THRESHOLD.interval_s * 1000;
    var failures = [];
    var t;
    try {
        for (var _b = __values(x.failures), _c = _b.next(); !_c.done; _c = _b.next()) {
            t = _c.value;
            if (t >= cutoff) {
                failures.push(t);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    if (failures.length >= THRESHOLD.fails) {
        x.failures = [];
        return true;
    }
    else {
        x.failures = failures;
        return false;
    }
}
var state = {};
function failing_to_save(path, hash, expected_hash) {
    if (expected_hash == undefined) {
        return false;
    }
    if (!state[path]) {
        state[path] = { path: path, failures: [] };
    }
    if (hash != expected_hash) {
        state[path].failures.push(new Date().getTime());
        return is_failing(state[path]);
    }
    // definitely NOT failing.
    return false;
}
exports.failing_to_save = failing_to_save;
