"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.define = exports.required = void 0;
exports.required = "__!!!!!!this is a required property!!!!!!__";
function define(props, definition, allow_extra, strict) {
    if (allow_extra === void 0) { allow_extra = false; }
    if (strict === void 0) { strict = false; }
    // We put explicit traces before the errors in this function,
    // since otherwise they can be very hard to debug.
    function maybe_error(message) {
        var err = message + " " + error_addendum(props, definition);
        if (strict) {
            throw new Error(err);
        }
        else {
            console.log(err);
            console.trace();
            return definition;
        }
    }
    if (props == undefined) {
        props = {};
    }
    // Undefined was checked above but TS 3.6.3 is having none of it.
    // Checking here makes TS work as expected below
    if (typeof props !== "object" || props == undefined) {
        return maybe_error("BUG -- Traceback -- misc.defaults -- TypeError: function takes inputs as an object");
    }
    var result = {};
    for (var key in definition) {
        if (props.hasOwnProperty(key) && props[key] != undefined) {
            if (definition[key] === exports.required && props[key] == undefined) {
                return maybe_error("misc.defaults -- TypeError: property '" + key + "' must be specified on props:");
            }
            result[key] = props[key];
        }
        else if (definition[key] != undefined) {
            if (definition[key] == exports.required) {
                maybe_error("misc.defaults -- TypeError: property '" + key + "' must be specified:");
            }
            else {
                result[key] = definition[key];
            }
        }
    }
    if (!allow_extra) {
        for (var key in props) {
            if (!definition.hasOwnProperty(key)) {
                return maybe_error("misc.defaults -- TypeError: got an unexpected argument '" + key + "'");
            }
        }
    }
    return result;
}
exports.define = define;
function error_addendum(props, definition) {
    try {
        return "(obj1=" + exports.trunc(exports.to_json(props), 1024) + ", obj2=" + exports.trunc(exports.to_json(definition), 1024) + ")";
    }
    catch (err) {
        return "";
    }
}
