"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.fill = void 0;
/**
 * Given an object: T with some optional parameters
 * and defaults: a subset of optional parameters from T.
 *
 * Explicitly setting a default to `undefined` is not recommended
 *
 * @return T except provided defaults are guaranteed
 *
 * @example
 *     props: {foo: string; bar?: string},
 *     defaults: {bar: "good stuff"}
 *        => {foo: string; bar: string} // <- Note bar is defined
 *
 *
 *     props: {foo: string; bar?: string; baz?: number},
 *     defaults: {bar: "good stuff"}
 *        => {foo: string; bar: string; baz?: number} // <- Note baz is optional
 * .
 **/
function fill(props, defaults) {
    var ret = {};
    for (var key in defaults) {
        if (!props.hasOwnProperty(key) || props[key] == undefined) {
            ret[key] = defaults[key];
        }
    }
    return Object.assign({}, props, ret);
}
exports.fill = fill;
