"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
var fill_1 = require("./fill");
var tsd_1 = require("tsd");
test("Supplied default should be merged in to target even if marked undefined", function () {
    var opts = {
        name: "jack",
        height: undefined,
    };
    var actual = fill_1.fill(opts, { height: 20 });
    expect(actual).toStrictEqual({ name: "jack", height: 20 });
});
test("Defaults should not overwrite already defined optional params", function () {
    var opts = {
        name: "jack",
        height: 20,
    };
    var actual = fill_1.fill(opts, { height: 30 });
    expect(actual).toStrictEqual({ name: "jack", height: 20 });
});
test("Missing optional params should not appear if not given defaults", function () {
    var opts = {
        name: "jack",
    };
    var actual = fill_1.fill(opts, { height: 20 });
    expect(actual).toStrictEqual({ name: "jack", height: 20 });
});
test("Supplied default should guarantee type existance", function () {
    var opts = { name: "foo", direction: "up" };
    var actual = fill_1.fill(opts, { highlight: false });
    tsd_1.expectType(actual);
});
test("strings", function () {
    function filled(props) {
        // This should not end up narrowing to the fixed value
        return fill_1.fill(props, { highlight: "fixed_string" });
    }
    var a = filled({ name: "foo", direction: "up" });
    tsd_1.expectType(a.name);
    tsd_1.expectType(a.direction);
    tsd_1.expectType(a.highlight);
    tsd_1.expectType(a.animate);
});
// tsd expectError doesn't integrate into Jest
test("Errors", function () {
    /*
    function prop_typed_errors(props: {
      name: string;
      direction: "up" | "down" | "left" | "right";
      highlight?: boolean;
      animate?: boolean;
    }) {
      // Don't allow requireds to even be listed
      return fill(props, { name: "undefined", highlight: false });
    }
    */
});
