/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
/*
Very, very simple key:value store.

The keys can be arbitrary json-able objects.
A frozen copy of the object is saved in the key:value store,
so it won't get mutated.
*/
var json = require("json-stable-stringify");
exports.key_value_store = function () { return new KeyValueStore(); };
var KeyValueStore = /** @class */ (function () {
    function KeyValueStore() {
        this.set = this.set.bind(this);
        this.get = this.get.bind(this);
        this.delete = this.delete.bind(this);
        this.close = this.close.bind(this);
        this._data = {};
    }
    KeyValueStore.prototype.assert_not_closed = function () {
        if (this._data == null) {
            throw Error("closed");
        }
    };
    KeyValueStore.prototype.set = function (key, value) {
        this.assert_not_closed();
        if (value.freeze != null) {
            // supported by modern browsers
            value = value.freeze(); // so doesn't get mutated
        }
        this._data[json(key)] = value;
    };
    KeyValueStore.prototype.get = function (key) {
        this.assert_not_closed();
        return this._data[json(key)];
    };
    KeyValueStore.prototype.delete = function (key) {
        this.assert_not_closed();
        delete this._data[json(key)];
    };
    KeyValueStore.prototype.close = function () {
        delete this._data;
    };
    return KeyValueStore;
}());
