"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.local_storage_length = exports.has_local_storage = exports.delete_local_storage = exports.get_local_storage = exports.set_local_storage = void 0;
// Wrapper around localStorage, so we can safely touch it without raising an
// exception if it is banned (like in some browser modes) or doesn't exist.
// See https://github.com/sagemathinc/cocalc/issues/237
function set_local_storage(key, val) {
    try {
        localStorage[key] = val;
    }
    catch (e) {
        console.warn("localStorage set error -- " + e);
    }
}
exports.set_local_storage = set_local_storage;
function get_local_storage(key) {
    try {
        return localStorage[key];
    }
    catch (e) {
        console.warn("localStorage get error -- " + e);
        return undefined;
    }
}
exports.get_local_storage = get_local_storage;
/**
 * Deletes key from local storage
 * FRONT END ONLY
 */
function delete_local_storage(key) {
    try {
        delete localStorage[key];
    }
    catch (e) {
        console.warn("localStorage delete error -- " + e);
    }
}
exports.delete_local_storage = delete_local_storage;
function has_local_storage() {
    try {
        var TEST = "__smc_test__";
        localStorage[TEST] = "x";
        delete localStorage[TEST];
        return true;
    }
    catch (e) {
        return false;
    }
}
exports.has_local_storage = has_local_storage;
function local_storage_length() {
    try {
        return localStorage.length;
    }
    catch (e) {
        return 0;
    }
}
exports.local_storage_length = local_storage_length;
