"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.map_mutate_out_undefined = exports.map_without_undefined = exports.is_zero_map = exports.sum = exports.map_max = exports.map_min = exports.map_limit = void 0;
var lodash_1 = require("lodash");
var type_checking_1 = require("./type-checking");
// compare the values in a map a by the values of b
// or just by b if b is a number, using func(a, b)
function map_comp_fn(func, fallback) {
    return function (a, b) {
        var _a;
        var c = {};
        if (typeof b === "number") {
            for (var k in a) {
                var v = a[k];
                c[k] = func(v, b);
            }
        }
        else {
            for (var k in a) {
                var v = a[k];
                c[k] = func(v, (_a = b[k]) !== null && _a !== void 0 ? _a : fallback);
            }
        }
        return c;
    };
}
exports.map_limit = map_comp_fn(Math.min, Number.MAX_VALUE);
exports.map_min = exports.map_limit;
exports.map_max = map_comp_fn(Math.max, Number.MIN_VALUE);
// arithmetic sum of an array
function sum(arr, start) {
    if (start === void 0) { start = 0; }
    if (start == null) {
        start = 0;
    }
    return lodash_1.reduce(arr, function (a, b) { return a + b; }, start);
}
exports.sum = sum;
// returns true if the given map is undefined or empty, or all the values are falsy
function is_zero_map(map) {
    if (map == null) {
        return true;
    }
    for (var k in map) {
        if (map[k]) {
            return false;
        }
    }
    return true;
}
exports.is_zero_map = is_zero_map;
// Returns copy of map with no undefined/null values (recursive).
// Doesn't modify map.  If map is an array, just returns it
// with no change even if it has undefined values.
function map_without_undefined(map) {
    if (map == null) {
        return;
    }
    if (type_checking_1.is_array(map)) {
        return map;
    }
    var new_map = {};
    for (var k in map) {
        var v = map[k];
        if (v == null) {
            continue;
        }
        else {
            new_map[k] = type_checking_1.is_object(v) ? map_without_undefined(v) : v;
        }
    }
    return new_map;
}
exports.map_without_undefined = map_without_undefined;
// modify map in places deleting keys with null or undefined
// values; NOT recursive.
function map_mutate_out_undefined(map) {
    for (var k in map) {
        var v = map[k];
        if (v == null) {
            delete map[k];
        }
    }
}
exports.map_mutate_out_undefined = map_mutate_out_undefined;
