"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.YEAR = exports.expire_time = exports.months_before = exports.weeks_before = exports.days_before = exports.hours_before = exports.minutes_before = exports.seconds_before = exports.milliseconds_before = exports.server_months_ago = exports.server_weeks_ago = exports.server_days_ago = exports.server_hours_ago = exports.server_minutes_ago = exports.server_seconds_ago = exports.server_milliseconds_ago = exports.server_time = exports.months_ago = exports.weeks_ago = exports.days_ago = exports.hours_ago = exports.minutes_ago = exports.seconds_ago = exports.milliseconds_ago = void 0;
var local_storage_1 = require("./local-storage");
// Specific, easy to read: describe amount of time before right now
// Use negative input for after now (i.e., in the future).
function milliseconds_ago(ms) {
    return new Date(new Date().valueOf() - ms);
}
exports.milliseconds_ago = milliseconds_ago;
function seconds_ago(s) {
    return milliseconds_ago(1000 * s);
}
exports.seconds_ago = seconds_ago;
function minutes_ago(m) {
    return seconds_ago(60 * m);
}
exports.minutes_ago = minutes_ago;
function hours_ago(h) {
    return minutes_ago(60 * h);
}
exports.hours_ago = hours_ago;
function days_ago(d) {
    return hours_ago(24 * d);
}
exports.days_ago = days_ago;
function weeks_ago(w) {
    return days_ago(7 * w);
}
exports.weeks_ago = weeks_ago;
function months_ago(m) {
    return days_ago(30.5 * m);
}
exports.months_ago = months_ago;
function server_time() {
    var _a;
    if (typeof window != "undefined") {
        return new Date(new Date().valueOf() - parseFloat((_a = local_storage_1.get_local_storage("clock_skew")) !== null && _a !== void 0 ? _a : "0"));
    }
    else {
        // On the server, we assume that the server clocks are sufficiently
        // accurate.  Providing these functions makes it simpler to write code
        // that runs on both the frontend and the backend.
        return new Date();
    }
}
exports.server_time = server_time;
function server_milliseconds_ago(ms) {
    var _a;
    if (typeof window != "undefined") {
        return new Date(new Date().valueOf() -
            ms -
            parseFloat((_a = local_storage_1.get_local_storage("clock_skew")) !== null && _a !== void 0 ? _a : "0"));
    }
    else {
        return milliseconds_ago(ms);
    }
}
exports.server_milliseconds_ago = server_milliseconds_ago;
function server_seconds_ago(s) {
    return server_milliseconds_ago(1000 * s);
}
exports.server_seconds_ago = server_seconds_ago;
function server_minutes_ago(m) {
    return server_seconds_ago(60 * m);
}
exports.server_minutes_ago = server_minutes_ago;
function server_hours_ago(h) {
    return server_minutes_ago(60 * h);
}
exports.server_hours_ago = server_hours_ago;
function server_days_ago(d) {
    return server_hours_ago(24 * d);
}
exports.server_days_ago = server_days_ago;
function server_weeks_ago(w) {
    return server_days_ago(7 * w);
}
exports.server_weeks_ago = server_weeks_ago;
function server_months_ago(m) {
    return server_days_ago(30.5 * m);
}
exports.server_months_ago = server_months_ago;
// Specific easy to read and describe point in time before another point in time tm.
// (The following work exactly as above if the second argument is excluded.)
// Use negative input for first argument for that amount of time after tm.
function milliseconds_before(ms, tm) {
    return new Date((tm !== null && tm !== void 0 ? tm : new Date()).valueOf() - ms);
}
exports.milliseconds_before = milliseconds_before;
function seconds_before(s, tm) {
    return milliseconds_before(1000 * s, tm);
}
exports.seconds_before = seconds_before;
function minutes_before(m, tm) {
    return seconds_before(60 * m, tm);
}
exports.minutes_before = minutes_before;
function hours_before(h, tm) {
    return minutes_before(60 * h, tm);
}
exports.hours_before = hours_before;
function days_before(d, tm) {
    return hours_before(24 * d, tm);
}
exports.days_before = days_before;
function weeks_before(d, tm) {
    return days_before(7 * d, tm);
}
exports.weeks_before = weeks_before;
function months_before(d, tm) {
    return days_before(30.5 * d, tm);
}
exports.months_before = months_before;
// time this many seconds in the future (or undefined)
function expire_time(s) {
    // @ts-ignore: due to possible non-TS clients
    if (s == null)
        return;
    return new Date(new Date().valueOf() + s * 1000);
}
exports.expire_time = expire_time;
exports.YEAR = new Date().getFullYear();
