"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sagews = exports.SageWS = exports.uuids_of_linked_files = exports.ACTION_SESSION_FLAGS = exports.ACTION_FLAGS = exports.FLAGS = exports.MARKERS = void 0;
/*
Some functions for working with Sage worksheets (sagews files) --
*/
//---------------------------------------------------------------------------------------------------------
// Support for using synchronized docs to represent Sage Worksheets (i.e., live compute documents)
//---------------------------------------------------------------------------------------------------------
// WARNING: in Codemirror, to avoid issues with parsing I also set the output marker to be a comment character
// by modifying the python mode as follows:     if (ch == "#"  || ch == "\uFE21") {
exports.MARKERS = {
    cell: "\uFE20",
    output: "\uFE21",
};
exports.FLAGS = {
    execute: "x",
    waiting: "w",
    running: "r",
    interrupt: "c",
    this_session: "s",
    hide_input: "i",
    hide_output: "o",
};
exports.ACTION_FLAGS = [
    exports.FLAGS.execute,
    exports.FLAGS.running,
    exports.FLAGS.waiting,
    exports.FLAGS.interrupt,
];
exports.ACTION_SESSION_FLAGS = [
    exports.FLAGS.execute,
    exports.FLAGS.running,
    exports.FLAGS.waiting,
    exports.FLAGS.interrupt,
    exports.FLAGS.this_session,
];
/*
Return a list of the uuids of files that are displayed in the given document,
where doc is the string representation of a worksheet.
At present, this function finds all output messages of the form
  {"file":{"uuid":"806f4f54-96c8-47f0-9af3-74b5d48d0a70",...}}
but it could do more at some point in the future.
*/
function uuids_of_linked_files(doc) {
    var e_1, _a;
    var uuids = [];
    var i = 0;
    while (true) {
        i = doc.indexOf(exports.MARKERS.output, i);
        if (i === -1) {
            return uuids;
        }
        var j = doc.indexOf("\n", i);
        if (j === -1) {
            j = doc.length;
        }
        var line = doc.slice(i, j);
        try {
            for (var _b = (e_1 = void 0, __values(line.split(exports.MARKERS.output).slice(1))), _c = _b.next(); !_c.done; _c = _b.next()) {
                var m = _c.value;
                // Only bother to run the possibly slow JSON.parse on file messages; since
                // this function would block the global hub server, this is important.
                if (m.slice(0, 8) === '{"file":') {
                    var mesg = JSON.parse(m);
                    if (mesg.file != null) {
                        var uuid = mesg.file.uuid;
                        if (uuid != null) {
                            uuids.push(uuid);
                        }
                    }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        i = j;
    }
}
exports.uuids_of_linked_files = uuids_of_linked_files;
var SageWS = /** @class */ (function () {
    function SageWS(content) {
        this.content = content;
    }
    SageWS.prototype.find_cell_meta = function (id, start) {
        var i = this.content.indexOf(exports.MARKERS.cell + id, start);
        var j = this.content.indexOf(exports.MARKERS.cell, i + 1);
        if (j === -1) {
            return undefined;
        }
        return { start: i, end: j };
    };
    SageWS.prototype.get_cell_flagstring = function (id) {
        var pos = this.find_cell_meta(id);
        if (pos != null) {
            return this.content.slice(pos.start + 37, pos.end);
        }
    };
    SageWS.prototype.set_cell_flagstring = function (id, flags) {
        var pos = this.find_cell_meta(id);
        if (pos != null) {
            this.content =
                this.content.slice(0, pos.start + 37) +
                    flags +
                    this.content.slice(pos.end);
        }
    };
    SageWS.prototype.remove_cell_flag = function (id, flag) {
        var s = this.get_cell_flagstring(id);
        if (s != null && s.includes(flag)) {
            this.set_cell_flagstring(id, s.replace(new RegExp(flag, "g"), ""));
        }
    };
    SageWS.prototype.set_cell_flag = function (id, flag) {
        var s = this.get_cell_flagstring(id);
        if (s != null && !s.includes(flag)) {
            this.set_cell_flagstring(id, s + flag);
        }
    };
    return SageWS;
}());
exports.SageWS = SageWS;
function sagews(content) {
    return new SageWS(content);
}
exports.sagews = sagews;
