"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Evaluator = void 0;
//##############################################################################
//
//    CoCalc: Collaborative Calculation in the Cloud
//    Copyright (C) 2016, Sagemath Inc., AGPLv3.
//
//##############################################################################
/*
Evaluation of code with streaming output built on both the clients and
server (local hub) using a sync_table.  This evaluator is associated
to a syncdoc editing session, and provides code evaluation that
may be used to enhance the experience of document editing.
*/
var stringify = require("json-stable-stringify");
var awaiting_1 = require("awaiting");
var util_1 = require("../../table/util");
var sagews_1 = require("../../../sagews");
var misc_1 = require("../../../misc");
var Evaluator = /** @class */ (function () {
    function Evaluator(syncdoc, client, create_synctable) {
        this.state = "init";
        this.table_options = [];
        this.last_call_time = new Date(0);
        this.syncdoc = syncdoc;
        this.client = client;
        this.create_synctable = create_synctable;
        if (this.syncdoc.data_server == "project") {
            // options only supported for project...
            this.table_options = [{ ephemeral: true, persistent: true }];
        }
    }
    Evaluator.prototype.init = function () {
        return __awaiter(this, void 0, void 0, function () {
            var i, o;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        i = this.init_eval_inputs();
                        o = this.init_eval_outputs();
                        return [4 /*yield*/, Promise.all([i, o])];
                    case 1:
                        _a.sent();
                        if (this.client.is_project()) {
                            this.init_project_evaluator();
                        }
                        this.set_state("ready");
                        return [2 /*return*/];
                }
            });
        });
    };
    Evaluator.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.inputs_table != null)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.inputs_table.close()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!(this.outputs_table != null)) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.outputs_table.close()];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        if (this.sage_session != null) {
                            this.sage_session.close();
                        }
                        misc_1.close(this);
                        this.set_state("closed");
                        return [2 /*return*/];
                }
            });
        });
    };
    Evaluator.prototype.dbg = function (_f) {
        if (this.client.is_project()) {
            return this.client.dbg("Evaluator." + _f);
        }
        else {
            return function () {
                var _ = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    _[_i] = arguments[_i];
                }
            };
        }
    };
    Evaluator.prototype.init_eval_inputs = function () {
        return __awaiter(this, void 0, void 0, function () {
            var query, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        query = {
                            eval_inputs: [
                                {
                                    string_id: this.syncdoc.get_string_id(),
                                    input: null,
                                    time: null,
                                    user_id: null,
                                },
                            ],
                        };
                        _a = this;
                        return [4 /*yield*/, this.create_synctable(query, this.table_options, 0)];
                    case 1:
                        _a.inputs_table = _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    Evaluator.prototype.init_eval_outputs = function () {
        return __awaiter(this, void 0, void 0, function () {
            var query, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        query = {
                            eval_outputs: [
                                {
                                    string_id: this.syncdoc.get_string_id(),
                                    output: null,
                                    time: null,
                                    number: null,
                                },
                            ],
                        };
                        _a = this;
                        return [4 /*yield*/, this.create_synctable(query, this.table_options, 0)];
                    case 1:
                        _a.outputs_table = _b.sent();
                        this.outputs_table.setMaxListeners(200); // in case of many evaluations at once.
                        return [2 /*return*/];
                }
            });
        });
    };
    Evaluator.prototype.set_state = function (state) {
        this.state = state;
    };
    Evaluator.prototype.assert_not_closed = function () {
        if (this.state === "closed") {
            throw Error("closed");
        }
    };
    Evaluator.prototype.assert_is_project = function () {
        if (!this.client.is_project()) {
            throw Error("BUG -- this code should only run in the project.");
        }
    };
    Evaluator.prototype.assert_is_browser = function () {
        if (this.client.is_project()) {
            throw Error("BUG -- this code should only run in the web browser.");
        }
    };
    // If given, cb below is called repeatedly with results as they appear.
    Evaluator.prototype.call = function (opts) {
        var _this = this;
        this.assert_not_closed();
        this.assert_is_browser();
        var dbg = this.dbg("call");
        dbg(opts.program, opts.input, opts.cb != undefined);
        var time = this.client.server_time();
        // Perturb time if it is <= last time when this client did an evaluation.
        // We do this so that the time below is different than anything else.
        if (time <= this.last_call_time) {
            // slightly later
            time = new Date(this.last_call_time.valueOf() + 1);
        }
        // make time be congruent to our uid
        this.last_call_time = time;
        var user_id = this.syncdoc.get_my_user_id();
        var obj = {
            string_id: this.syncdoc.get_string_id(),
            time: time,
            user_id: user_id,
            input: misc_1.copy_without(opts, "cb"),
        };
        dbg(JSON.stringify(obj));
        this.inputs_table.set(obj);
        // root cause of https://github.com/sagemathinc/cocalc/issues/1589
        this.inputs_table.save();
        if (opts.cb == null) {
            // Fire and forget -- no need to listen for responses.
            dbg("no cb defined, so fire and forget");
            return;
        }
        // Listen for output until we receive a message with mesg.done true.
        var messages = {};
        // output may appear in random order, so we use mesg_number
        // to sort it out.
        var mesg_number = 0;
        var send = function (mesg) {
            dbg("send", mesg);
            if (mesg.done) {
                _this.outputs_table.removeListener("change", handle_output);
            }
            if (opts.cb != null) {
                opts.cb(mesg);
            }
        };
        var handle_output = function (keys) {
            var e_1, _a;
            // console.log("handle_output #{to_json(keys)}")
            dbg("handle_output", keys);
            _this.assert_not_closed();
            try {
                for (var keys_1 = __values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                    var key = keys_1_1.value;
                    var t = misc_1.from_json(key);
                    if (t[1].valueOf() != time.valueOf()) {
                        dbg("not our eval", t[1].valueOf(), time.valueOf());
                        continue;
                    }
                    var x = _this.outputs_table.get(key);
                    if (x == null) {
                        dbg("x is null");
                        continue;
                    }
                    var y = x.get("output");
                    if (y == null) {
                        dbg("y is null");
                        continue;
                    }
                    dbg("y = ", JSON.stringify(y.toJS()));
                    var mesg = y.toJS();
                    if (mesg == null) {
                        dbg("probably never happens, but makes typescript happy.");
                        continue;
                    }
                    // OK, we called opts.cb on output mesg with the given timestamp and user_id...
                    delete mesg.id; // waste of space
                    // Messages may arrive in somewhat random order.  This *DOES HAPPEN*,
                    // since changes are output from the project by computing a diff of
                    // a synctable, and then an array of objects sent out... and
                    // the order in that diff is random.
                    // E.g. this in a Sage worksheet would break:
                    //    for i in range(20): print i; sys.stdout.flush()
                    if (t[2] !== mesg_number) {
                        // Not the next message, so put message in the
                        // set of messages that arrived too early.
                        dbg("put message in holding", t[2], mesg_number);
                        messages[t[2]] = mesg;
                        continue;
                    }
                    // Finally, the right message to handle next.
                    // Inform caller of result
                    send(mesg);
                    mesg_number += 1;
                    // Then, push out any messages that arrived earlier
                    // that are ready to send.
                    while (messages[mesg_number] != null) {
                        send(messages[mesg_number]);
                        delete messages[mesg_number];
                        mesg_number += 1;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        };
        this.outputs_table.on("change", handle_output);
    };
    Evaluator.prototype.execute_sage_code_hook = function (output_uuid) {
        var _this = this;
        this.assert_is_project();
        var dbg = this.dbg("execute_sage_code_hook('" + output_uuid + "')");
        dbg();
        this.assert_not_closed();
        // We track the output_line from within this project, and compare
        // to what is set in the document (by the user).  If they go out
        // of sync for a while, we fill in the result.
        // TODO: since it's now possible to know whether or not users are
        // connected... maybe we could use that instead?
        var output_line = sagews_1.MARKERS.output;
        var hook = function (mesg) {
            dbg("processing mesg '" + misc_1.to_json(mesg) + "'");
            var content = _this.syncdoc.to_str();
            var i = content.indexOf(sagews_1.MARKERS.output + output_uuid);
            if (i === -1) {
                // no cell anymore, so do nothing further right now.
                return;
            }
            i += 37;
            var n = content.indexOf("\n", i);
            if (n === -1) {
                // corrupted? -- don't try further right now.
                return;
            }
            // This is what the frontend also does:
            output_line +=
                stringify(misc_1.copy_without(mesg, ["id", "event"])) + sagews_1.MARKERS.output;
            if (output_line.length - 1 <= n - i) {
                // Things are looking fine (at least, the line is longer enough).
                // TODO: try instead comparing actual content, not just length?
                // Or maybe don't... since this stupid code will all get deleted anyways
                // when we rewrite sagews handling.
                return;
            }
            dbg("browser client didn't maintain sync promptly. fixing");
            dbg("sage_execute_code: i=" + i + ", n=" + n + ", output_line.length=" + output_line.length);
            dbg("output_line='" + output_line + "', sync_line='" + content.slice(i, n) + "'");
            var x = content.slice(0, i);
            content = x + output_line + content.slice(n);
            if (mesg.done) {
                var j = x.lastIndexOf(sagews_1.MARKERS.cell);
                if (j !== -1) {
                    j = x.lastIndexOf("\n", j);
                    var cell_id = x.slice(j + 2, j + 38);
                    //dbg("removing a cell flag: before='#{content}', cell_id='#{cell_id}'")
                    var S = sagews_1.sagews(content);
                    S.remove_cell_flag(cell_id, sagews_1.FLAGS.running);
                    S.set_cell_flag(cell_id, sagews_1.FLAGS.this_session);
                    content = S.content;
                }
            }
            //dbg("removing a cell flag: after='#{content}'")
            _this.syncdoc.from_str(content);
            _this.syncdoc.commit();
        };
        return function (mesg) {
            setTimeout(function () { return hook(mesg); }, 5000);
        };
    };
    Evaluator.prototype.handle_input_change = function (key) {
        var _a, _b;
        var _this = this;
        this.assert_not_closed();
        this.assert_is_project();
        var dbg = this.dbg("handle_input_change");
        dbg("change: " + key);
        var t = misc_1.from_json(key);
        var number, string_id, time;
        var id = (_a = [t[0], t[1], 0], _b = __read(_a, 3), string_id = _b[0], time = _b[1], number = _b[2], _a);
        if (this.outputs_table.get(util_1.to_key(id)) != null) {
            dbg("already being handled");
            return;
        }
        dbg("no outputs yet with key " + misc_1.to_json(id));
        var r = this.inputs_table.get(key);
        if (r == null) {
            dbg("deleting from input?");
            throw Error("deleting from input not implemented");
            // happens when deleting from input table (if that is
            // ever supported, e.g., for maybe trimming old evals...)
            return;
        }
        var input = r.get("input");
        if (input == null) {
            throw Error("input must be specified");
            return;
        }
        var x = input.toJS();
        dbg("x = ", x);
        if (x == null) {
            throw Error("BUG: can't happen");
            return;
        }
        if (x.program == null || x.input == null) {
            this.outputs_table.set({
                string_id: string_id,
                time: time,
                number: number,
                output: {
                    error: "must specify both program and input",
                    done: true,
                },
            });
            this.outputs_table.save();
            return;
        }
        var f;
        switch (x.program) {
            case "sage":
                f = this.evaluate_using_sage;
                break;
            case "shell":
                f = this.evaluate_using_shell;
                break;
            default:
                this.outputs_table.set({
                    string_id: string_id,
                    time: time,
                    number: number,
                    output: {
                        error: "no program '" + x.program + "'",
                        done: true,
                    },
                });
                this.outputs_table.save();
                return;
        }
        f = f.bind(this);
        var hook;
        if (x.program === "sage" &&
            x.input.event === "execute_code" &&
            x.input.output_uuid != null) {
            hook = this.execute_sage_code_hook(x.input.output_uuid);
        }
        else {
            // no op
            hook = function (_) { };
        }
        f(x.input, function (output) {
            _this.assert_not_closed();
            dbg("got output='" + misc_1.to_json(output) + "'; id=" + misc_1.to_json(id));
            hook(output);
            _this.outputs_table.set({ string_id: string_id, time: time, number: number, output: output });
            _this.outputs_table.save();
            number += 1;
        });
    };
    // Runs only in the project
    Evaluator.prototype.init_project_evaluator = function () {
        var _this = this;
        this.assert_is_project();
        var dbg = this.dbg("init_project_evaluator");
        dbg("init");
        this.inputs_table.on("change", function (keys) {
            var e_2, _a;
            try {
                for (var keys_2 = __values(keys), keys_2_1 = keys_2.next(); !keys_2_1.done; keys_2_1 = keys_2.next()) {
                    var key = keys_2_1.value;
                    _this.handle_input_change(key);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (keys_2_1 && !keys_2_1.done && (_a = keys_2.return)) _a.call(keys_2);
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
        /* CRITICAL: it's very important to handle all the inputs
           that may have happened just moments before
           this object got created.  Why? The first input is
           the user trying to frickin' evaluate a cell
           in their worksheet to start things running... and they
           might somehow do that moments before the worksheet
           gets opened on the backend; if we don't do the
           following, then often this eval is missed, and
           confusion and frustration ensues. */
        var v = this.inputs_table.get();
        if (v != null) {
            dbg("handle " + v.size + " pending evaluations");
            v.forEach(function (_, key) {
                if (key != null) {
                    _this.handle_input_change(key);
                }
            });
        }
    };
    Evaluator.prototype.ensure_sage_session_exists = function () {
        if (this.sage_session != null)
            return;
        this.dbg("ensure_sage_session_exists")();
        // This code only runs in the project, where client
        // has a sage_session method.
        this.sage_session = this.client.sage_session({
            path: this.syncdoc.get_path(),
        });
    };
    // Runs only in the project
    Evaluator.prototype.evaluate_using_sage = function (input, cb) {
        return __awaiter(this, void 0, void 0, function () {
            var dbg;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_is_project();
                        dbg = this.dbg("evaluate_using_sage");
                        dbg();
                        // TODO: input also may have -- uuid, output_uuid, timeout
                        if (input.event === "execute_code") {
                            input = misc_1.copy_with(input, ["code", "data", "preparse", "event", "id"]);
                            dbg("ensure sage session is running, so we can actually execute the code");
                        }
                        return [4 /*yield*/, this.ensure_sage_session_exists()];
                    case 1:
                        _a.sent();
                        if (!(input.event === "execute_code")) return [3 /*break*/, 3];
                        if (!!this.sage_session.is_running()) return [3 /*break*/, 3];
                        return [4 /*yield*/, awaiting_1.callback(this.sage_session.init_socket)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        dbg("send call to backend sage session manager", misc_1.to_json(input));
                        this.sage_session.call({ input: input, cb: cb });
                        return [2 /*return*/];
                }
            });
        });
    };
    // Runs only in the project
    Evaluator.prototype.evaluate_using_shell = function (input, cb) {
        this.assert_is_project();
        var dbg = this.dbg("evaluate_using_shell");
        dbg();
        input.cb = function (err, output) {
            if (output == null) {
                output = {};
            }
            if (err) {
                output.error = err;
            }
            output.done = true;
            cb(output);
        };
        this.client.shell(input);
    };
    return Evaluator;
}());
exports.Evaluator = Evaluator;
