"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncDoc = void 0;
/*
SyncDoc -- the core class for editing with a synchronized document.

This code supports both string-doc and db-doc, for editing both
strings and small database tables efficiently, with history,
undo, save to disk, etc.

This code is run *both* in browser clients and under node.js
in projects, and behaves slightly differently in each case.
*/
/* OFFLINE_THRESH_S - If the client becomes disconnected from
   the backend for more than this long then---on reconnect---do
   extra work to ensure that all snapshots are up to date (in
   case snapshots were made when we were offline), and mark the
   sent field of patches that weren't saved.   I.e., we rebase
   all offline changes. */
var OFFLINE_THRESH_S = 5 * 60; // 5 minutes.
/* How often the local hub will autosave this file to disk if
   it has it open and there are unsaved changes.  This is very
   important since it ensures that a user that edits a file but
   doesn't click "Save" and closes their browser (right after
   their edits have gone to the database), still has their
   file saved to disk soon.  This is important, e.g., for homework
   getting collected and not missing the last few changes.  It turns
   out this is what people expect.
   Set to 0 to disable. (But don't do that.) */
var LOCAL_HUB_AUTOSAVE_S = 45;
// const LOCAL_HUB_AUTOSAVE_S = 5;
// How big of files we allow users to open using syncstrings.
var MAX_FILE_SIZE_MB = 5;
var events_1 = require("events");
var lodash_1 = require("lodash");
var immutable_1 = require("immutable");
var awaiting_1 = require("awaiting");
var async_utils_1 = require("../../../async-utils");
var async_wait_1 = require("../../../async-wait");
var misc_1 = require("../../../misc");
var evaluator_1 = require("./evaluator");
var ipywidgets_state_1 = require("./ipywidgets-state");
var schema = require("../../../schema");
var sorted_patch_list_1 = require("./sorted-patch-list");
var util_1 = require("./util");
var export_1 = require("./export");
var SyncDoc = /** @class */ (function (_super) {
    __extends(SyncDoc, _super);
    function SyncDoc(opts) {
        var e_1, _a;
        var _b;
        var _this = _super.call(this) || this;
        // This id is used for equality test and caching.
        _this.id = misc_1.uuid();
        // Throttling of incoming upstream patches from project to client.
        _this.patch_interval = 250;
        // This is what's actually output by setInterval -- it's
        // not an amount of time.
        _this.project_autosave_timer = 0;
        // throttling of change events -- e.g., is useful for course
        // editor where we have hundreds of changes and the UI gets
        // overloaded unless we throttle and group them.
        _this.change_throttle = 0;
        _this.cursors = false; // if true, also provide cursor tracking functionality
        _this.cursor_map = immutable_1.Map();
        _this.cursor_last_time = new Date(0);
        _this.state = "init";
        _this.last_user_change = misc_1.minutes_ago(60);
        _this.last_save_to_disk_time = new Date(0);
        _this.settings = immutable_1.Map();
        _this.syncstring_save_state = "";
        _this.load_full_history_done = false;
        // patches that this client made during this editing session.
        _this.my_patches = {};
        _this.patch_update_queue = [];
        _this.persistent = false;
        _this.data_server = "project";
        _this.last_has_unsaved_changes = undefined;
        _this.ephemeral = false;
        _this.sync_is_disabled = false;
        if (opts.string_id === undefined) {
            _this.string_id = schema.client_db.sha1(opts.project_id, opts.path);
        }
        else {
            _this.string_id = opts.string_id;
        }
        try {
            for (var _c = __values([
                "project_id",
                "path",
                "client",
                "patch_interval",
                "file_use_interval",
                "change_throttle",
                "cursors",
                "doctype",
                "from_patch_str",
                "persistent",
                "data_server",
                "ephemeral",
            ]), _d = _c.next(); !_d.done; _d = _c.next()) {
                var field = _d.value;
                if (opts[field] != undefined) {
                    _this[field] = opts[field];
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
        _this._from_str = opts.from_str;
        // Initialize to time when we create the syncstring, so we don't
        // see our own cursor when we refresh the browser (before we move
        // to update this).
        _this.cursor_last_time = (_b = _this.client) === null || _b === void 0 ? void 0 : _b.server_time();
        async_utils_1.reuse_in_flight_methods(_this, [
            "save",
            "save_to_disk",
            "load_from_disk",
            "handle_patch_update_queue",
            "sync_remote_and_doc",
        ]);
        if (_this.change_throttle) {
            _this.emit_change = lodash_1.throttle(_this.emit_change, _this.change_throttle);
        }
        _this.setMaxListeners(100);
        _this.init();
        return _this;
    }
    /*
    Initialize everything.
    This should be called *exactly* once by the constructor,
    and no other time.  It tries to set everything up.  If
    the browser isn't connected to the network, it'll wait
    until it is (however long, etc.).  If this fails, it closes
    this SyncDoc.
    */
    SyncDoc.prototype.init = function () {
        return __awaiter(this, void 0, void 0, function () {
            var log, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_not_closed("init");
                        log = this.dbg("init");
                        log("initializing all tables...");
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 5]);
                        //const t0 = new Date();
                        return [4 /*yield*/, this.init_all()];
                    case 2:
                        //const t0 = new Date();
                        _a.sent();
                        return [3 /*break*/, 5];
                    case 3:
                        err_1 = _a.sent();
                        if (this.state == "closed") {
                            return [2 /*return*/];
                        }
                        log("WARNING -- error initializing " + err_1);
                        // completely normal that this could happen on frontend - it just means
                        // that we closed the file before finished opening it...
                        if (this.state != "closed") {
                            log("Error -- NOT caused by closing during the init_all, so we report it.");
                            this.emit("error", err_1);
                        }
                        return [4 /*yield*/, this.close()];
                    case 4:
                        _a.sent();
                        return [2 /*return*/];
                    case 5:
                        // Success -- everything perfectly initialized with no issues.
                        this.set_state("ready");
                        this.init_watch();
                        this.emit_change(); // from nothing to something.
                        return [2 /*return*/];
                }
            });
        });
    };
    /* Set this user's cursors to the given locs. */
    SyncDoc.prototype.set_cursor_locs = function (locs, side_effect) {
        if (side_effect === void 0) { side_effect = false; }
        if (this.state != "ready")
            return;
        if (this.cursors_table == null) {
            throw Error("cursors are not enabled");
        }
        var x = {
            string_id: this.string_id,
            user_id: this.my_user_id,
            locs: locs,
        };
        if (!side_effect) {
            x.time = this.client.server_time();
        }
        if (x.time != null) {
            this.cursor_last_time = x.time;
        }
        this.cursors_table.set(x, "none");
        this.cursors_table.save();
    };
    SyncDoc.prototype.init_file_use_interval = function () {
        var _this = this;
        var is_chat = misc_1.filename_extension(this.path) === "sage-chat";
        if (this.file_use_interval == null) {
            if (is_chat) {
                this.file_use_interval = 10 * 1000;
            }
            else {
                this.file_use_interval = 60 * 1000;
            }
        }
        if (!this.file_use_interval || !this.client.is_user()) {
            // file_use_interval has to be nonzero, and we only do
            // this for browser user.
            return;
        }
        //const dbg = this.dbg('init_file_use_interval')
        var action;
        if (is_chat) {
            action = "chat";
        }
        else {
            action = "edit";
        }
        var file_use = function () { return __awaiter(_this, void 0, void 0, function () {
            var user_is_active, tm;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!is_chat) return [3 /*break*/, 2];
                        return [4 /*yield*/, awaiting_1.delay(100)];
                    case 1:
                        _a.sent(); // wait a little so my_patches and gets updated.
                        user_is_active = false;
                        for (tm in this.my_patches) {
                            if (new Date(parseInt(tm)) > this.last_user_change) {
                                user_is_active = true;
                                break;
                            }
                        }
                        if (!user_is_active) {
                            return [2 /*return*/];
                        }
                        _a.label = 2;
                    case 2:
                        this.last_user_change = new Date();
                        this.client.mark_file({
                            project_id: this.project_id,
                            path: this.path,
                            action: action,
                            ttl: this.file_use_interval,
                        });
                        return [2 /*return*/];
                }
            });
        }); };
        this.throttled_file_use = lodash_1.throttle(file_use, this.file_use_interval, {
            leading: true,
        });
        this.on("user-change", this.throttled_file_use);
    };
    SyncDoc.prototype.set_state = function (state) {
        this.state = state;
        this.emit(state);
    };
    SyncDoc.prototype.get_state = function () {
        return this.state;
    };
    SyncDoc.prototype.get_project_id = function () {
        return this.project_id;
    };
    SyncDoc.prototype.get_path = function () {
        return this.path;
    };
    SyncDoc.prototype.get_string_id = function () {
        return this.string_id;
    };
    SyncDoc.prototype.get_my_user_id = function () {
        return this.my_user_id != null ? this.my_user_id : 0;
    };
    SyncDoc.prototype.assert_not_closed = function (desc) {
        if (this.state === "closed") {
            //console.trace();
            throw Error("must not be closed -- " + desc);
        }
    };
    SyncDoc.prototype.set_doc = function (doc, exit_undo_mode) {
        if (exit_undo_mode === void 0) { exit_undo_mode = true; }
        if (doc.is_equal(this.doc)) {
            // no change.
            return;
        }
        if (exit_undo_mode)
            this.undo_state = undefined;
        // console.log(`sync-doc.set_doc("${doc.to_str()}")`);
        this.doc = doc;
        this.emit_change();
    };
    // Convenience function to avoid having to do
    // get_doc and set_doc constantly.
    SyncDoc.prototype.set = function (x) {
        this.set_doc(this.doc.set(x));
    };
    SyncDoc.prototype.delete = function (x) {
        this.set_doc(this.doc.delete(x));
    };
    SyncDoc.prototype.get = function (x) {
        return this.doc.get(x);
    };
    SyncDoc.prototype.get_one = function (x) {
        return this.doc.get_one(x);
    };
    // Return underlying document, or undefined if document
    // hasn't been set yet.
    SyncDoc.prototype.get_doc = function () {
        if (this.doc == null) {
            throw Error("doc must be set");
        }
        return this.doc;
    };
    // Set this doc from its string representation.
    SyncDoc.prototype.from_str = function (value) {
        // console.log(`sync-doc.from_str("${value}")`);
        this.doc = this._from_str(value);
    };
    // Return string representation of this doc,
    // or exception if not yet ready.
    SyncDoc.prototype.to_str = function () {
        if (this.doc == null) {
            throw Error("doc must be set");
        }
        return this.doc.to_str();
    };
    SyncDoc.prototype.count = function () {
        return this.doc.count();
    };
    // Version of the document at a given point in time; if no
    // time specified, gives the version right now.
    // If not fully initialized, will throw exception.
    SyncDoc.prototype.version = function (time) {
        this.assert_table_is_ready("patches");
        misc_1.assertDefined(this.patch_list);
        return this.patch_list.value(time);
    };
    /* Compute version of document if the patches at the given times
       were simply not included.  This is a building block that is
       used for implementing undo functionality for client editors. */
    SyncDoc.prototype.version_without = function (times) {
        this.assert_table_is_ready("patches");
        misc_1.assertDefined(this.patch_list);
        return this.patch_list.value(undefined, undefined, times);
    };
    // Revert document to what it was at the given point in time.
    // There doesn't have to be a patch at exactly that point in
    // time -- if there isn't it just uses the patch before that
    // point in time.
    SyncDoc.prototype.revert = function (time) {
        this.set_doc(this.version(time));
    };
    /* Undo/redo public api.
    Calling this.undo and this.redo returns the version of
    the document after the undo or redo operation, and records
    a commit changing to that.
    The first time calling this.undo switches into undo
    state in which additional
    calls to undo/redo move up and down the stack of changes made
    by this user during this session.
  
    Call this.exit_undo_mode() to exit undo/redo mode.
  
    Undo and redo *only* impact changes made by this user during
    this session.  Other users edits are unaffected, and work by
    this same user working from another browser tab or session is
    also unaffected.
  
    Finally, undo of a past patch by definition means "the state
    of the document" if that patch was not applied.  The impact
    of undo is NOT that the patch is removed from the patch history.
    Instead, it records a new patch that is what would have happened
    had we replayed history with the patches being undone not there.
  
    Doing any set_doc explicitly exits undo mode automatically.
    */
    SyncDoc.prototype.undo = function () {
        var prev = this._undo();
        this.set_doc(prev, false);
        this.commit();
        return prev;
    };
    SyncDoc.prototype.redo = function () {
        var next = this._redo();
        this.set_doc(next, false);
        this.commit();
        return next;
    };
    SyncDoc.prototype._undo = function () {
        this.assert_is_ready("_undo");
        var state = this.undo_state;
        if (state == null) {
            // not in undo mode
            state = this.undo_state = this.init_undo_state();
        }
        if (state.pointer === state.my_times.length) {
            // pointing at live state (e.g., happens on entering undo mode)
            var value = this.version(); // last saved version
            var live = this.doc;
            if (!live.is_equal(value)) {
                // User had unsaved changes, so last undo is to revert to version without those.
                state.final = value.make_patch(live); // live redo if needed
                state.pointer -= 1; // most recent timestamp
                return value;
            }
            else {
                // User had no unsaved changes, so last undo is version without last saved change.
                var tm = state.my_times[state.pointer - 1];
                state.pointer -= 2;
                if (tm != null) {
                    state.without.push(tm);
                    return this.version_without(state.without);
                }
                else {
                    // no undo information during this session
                    return value;
                }
            }
        }
        else {
            // pointing at particular timestamp in the past
            if (state.pointer >= 0) {
                // there is still more to undo
                state.without.push(state.my_times[state.pointer]);
                state.pointer -= 1;
            }
            return this.version_without(state.without);
        }
    };
    SyncDoc.prototype._redo = function () {
        this.assert_is_ready("_redo");
        var state = this.undo_state;
        if (state == null) {
            // nothing to do but return latest live version
            return this.get_doc();
        }
        if (state.pointer === state.my_times.length) {
            // pointing at live state -- nothing to do
            return this.get_doc();
        }
        else if (state.pointer === state.my_times.length - 1) {
            // one back from live state, so apply unsaved patch to live version
            var value = this.version();
            if (value == null) {
                // see remark in undo -- do nothing
                return this.get_doc();
            }
            state.pointer += 1;
            return value.apply_patch(state.final);
        }
        else {
            // at least two back from live state
            state.without.pop();
            state.pointer += 1;
            if (state.final == null && state.pointer === state.my_times.length - 1) {
                // special case when there wasn't any live change
                state.pointer += 1;
            }
            return this.version_without(state.without);
        }
    };
    SyncDoc.prototype.in_undo_mode = function () {
        return this.undo_state != null;
    };
    SyncDoc.prototype.exit_undo_mode = function () {
        this.undo_state = undefined;
    };
    SyncDoc.prototype.init_undo_state = function () {
        if (this.undo_state != null) {
            return this.undo_state;
        }
        var my_times = misc_1.keys(this.my_patches).map(function (x) { return new Date(parseInt(x)); });
        my_times.sort(misc_1.cmp_Date);
        return (this.undo_state = {
            my_times: my_times,
            pointer: my_times.length,
            without: [],
        });
    };
    SyncDoc.prototype.save_to_disk_autosave = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, err_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("save_to_disk_autosave");
                        dbg();
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.save_to_disk()];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        err_2 = _a.sent();
                        dbg("failed -- " + err_2);
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /* Make it so the local hub project will automatically save
       the file to disk periodically. */
    SyncDoc.prototype.init_project_autosave = function () {
        // Do not autosave sagews until we resolve
        //   https://github.com/sagemathinc/cocalc/issues/974
        // Similarly, do not autosave ipynb because of
        //   https://github.com/sagemathinc/cocalc/issues/5216
        if (!LOCAL_HUB_AUTOSAVE_S ||
            !this.client.is_project() ||
            this.project_autosave_timer ||
            misc_1.endswith(this.path, ".sagews") ||
            misc_1.endswith(this.path, ".ipynb.sage-jupyter2")) {
            return;
        }
        // Explicit cast due to node vs browser typings.
        this.project_autosave_timer = (setInterval(this.save_to_disk_autosave.bind(this), LOCAL_HUB_AUTOSAVE_S * 1000));
    };
    // account_id of the user who made the edit at
    // the given point in time.
    SyncDoc.prototype.account_id = function (time) {
        this.assert_is_ready("account_id");
        return this.users[this.user_id(time)];
    };
    /* Approximate time when patch with given timestamp was
       actually sent to the server; returns undefined if time
       sent is approximately the timestamp time.  Only defined
       when there is a significant difference, due to editing
       when offline! */
    SyncDoc.prototype.time_sent = function (time) {
        this.assert_table_is_ready("patches");
        misc_1.assertDefined(this.patch_list);
        return this.patch_list.time_sent(time);
    };
    // Integer index of user who made the edit at given
    // point in time.
    SyncDoc.prototype.user_id = function (time) {
        this.assert_table_is_ready("patches");
        misc_1.assertDefined(this.patch_list);
        return this.patch_list.user_id(time);
    };
    SyncDoc.prototype.syncstring_table_get_one = function () {
        if (this.syncstring_table == null) {
            throw Error("syncstring_table must be defined");
        }
        var t = this.syncstring_table.get_one();
        if (t == null) {
            // project has not initialized it yet.
            return immutable_1.Map();
        }
        return t;
    };
    /* The project calls set_initialized once it has checked for
       the file on disk; this way the frontend knows that the
       syncstring has been initialized in the database, and also
       if there was an error doing the check.
     */
    SyncDoc.prototype.set_initialized = function (error, read_only, size) {
        return __awaiter(this, void 0, void 0, function () {
            var init;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_table_is_ready("syncstring");
                        this.dbg("set_initialized")();
                        init = { time: this.client.server_time(), size: size, error: error };
                        this.syncstring_table.set({
                            string_id: this.string_id,
                            project_id: this.project_id,
                            path: this.path,
                            init: init,
                            read_only: read_only,
                            last_active: this.client.server_time(),
                        });
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /* List of timestamps of the versions of this string in the sync
       table that we opened to start editing (so starts with what was
       the most recent snapshot when we started).  The list of timestamps
       is sorted from oldest to newest. */
    SyncDoc.prototype.versions = function () {
        this.assert_table_is_ready("patches");
        var v = [];
        var s = this.patches_table.get();
        if (s == null) {
            // shouldn't happen do to assert_is_ready above.
            throw Error("patches_table must be initialized");
        }
        s.map(function (x, _) {
            v.push(x.get("time"));
        });
        v.sort(misc_1.cmp_Date);
        return v;
    };
    /* List of all known timestamps of versions of this string, including
       possibly much older versions than returned by this.versions(), in
       case the full history has been loaded.  The list of timestamps
       is sorted from oldest to newest. */
    SyncDoc.prototype.all_versions = function () {
        this.assert_table_is_ready("patches");
        misc_1.assertDefined(this.patch_list);
        return this.patch_list.versions();
    };
    SyncDoc.prototype.last_changed = function () {
        var v = this.versions();
        if (v.length > 0) {
            return v[v.length - 1];
        }
        else {
            return new Date(0);
        }
    };
    SyncDoc.prototype.init_table_close_handlers = function () {
        var e_2, _a;
        var _this = this;
        try {
            for (var _b = __values(["syncstring", "patches", "cursors"]), _c = _b.next(); !_c.done; _c = _b.next()) {
                var x = _c.value;
                var t = this[x + "_table"];
                if (t != null) {
                    t.on("close", function () { return _this.close(); });
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    // Close synchronized editing of this string; this stops listening
    // for changes and stops broadcasting changes.
    SyncDoc.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            var err_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.state == "closed") {
                            return [2 /*return*/];
                        }
                        if (!(this.client.is_user() && this.state == "ready")) return [3 /*break*/, 4];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.save_to_disk()];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        err_3 = _a.sent();
                        return [3 /*break*/, 4];
                    case 4:
                        this.set_state("closed");
                        this.emit("close");
                        // must be after the emits above, so clients know
                        // what happened and can respond.
                        this.removeAllListeners();
                        if (this.throttled_file_use != null) {
                            // Cancel any pending file_use calls.
                            async_utils_1.cancel_scheduled(this.throttled_file_use);
                            this.throttled_file_use.cancel();
                        }
                        if (this.emit_change != null) {
                            // Cancel any pending change emit calls.
                            async_utils_1.cancel_scheduled(this.emit_change);
                        }
                        if (this.project_autosave_timer) {
                            clearInterval(this.project_autosave_timer);
                            this.project_autosave_timer = 0;
                        }
                        this.patch_update_queue = [];
                        if (!(this.syncstring_table != null)) return [3 /*break*/, 6];
                        return [4 /*yield*/, this.syncstring_table.close()];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        if (!(this.patches_table != null)) return [3 /*break*/, 8];
                        return [4 /*yield*/, this.patches_table.close()];
                    case 7:
                        _a.sent();
                        _a.label = 8;
                    case 8:
                        if (!(this.patch_list != null)) return [3 /*break*/, 10];
                        return [4 /*yield*/, this.patch_list.close()];
                    case 9:
                        _a.sent();
                        _a.label = 10;
                    case 10:
                        if (!(this.cursors_table != null)) return [3 /*break*/, 12];
                        return [4 /*yield*/, this.cursors_table.close()];
                    case 11:
                        _a.sent();
                        _a.label = 12;
                    case 12:
                        if (this.client.is_project()) {
                            this.update_watch_path(); // no input = closes it
                        }
                        if (!(this.evaluator != null)) return [3 /*break*/, 14];
                        return [4 /*yield*/, this.evaluator.close()];
                    case 13:
                        _a.sent();
                        _a.label = 14;
                    case 14:
                        if (!(this.ipywidgets_state != null)) return [3 /*break*/, 16];
                        return [4 /*yield*/, this.ipywidgets_state.close()];
                    case 15:
                        _a.sent();
                        _a.label = 16;
                    case 16:
                        misc_1.close(this);
                        this.set_state("closed");
                        return [2 /*return*/];
                }
            });
        });
    };
    // TODO: We **have** to do this on the client, since the backend
    // **security model** for accessing the patches table only
    // knows the string_id, but not the project_id/path.  Thus
    // there is no way currently to know whether or not the client
    // has access to the patches, and hence the patches table
    // query fails.  This costs significant time -- a roundtrip
    // and write to the database -- whenever the user opens a file.
    // This fix should be to change the patches schema somehow
    // to have the user also provide the project_id and path, thus
    // proving they have access to the sha1 hash (string_id), but
    // don't actually use the project_id and path as columns in
    // the table.  This requires some new idea I guess of virtual
    // fields....
    // Also, this also establishes the correct doctype.
    // Since this MUST succeed before doing anything else. This is critical
    // because the patches table can't be opened anywhere if the syncstring
    // object doesn't exist, due to how our security works, *AND* that the
    // patches table uses the string_id, which is a SHA1 hash.
    SyncDoc.prototype.ensure_syncstring_exists_in_db = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("ensure_syncstring_exists_in_db");
                        if (this.ephemeral) {
                            dbg("ephemeral -- nothing to do (since database not used)");
                            return [2 /*return*/];
                        }
                        if (!!this.client.is_connected()) return [3 /*break*/, 2];
                        dbg("wait until connected...", this.client.is_connected());
                        return [4 /*yield*/, async_utils_1.once(this.client, "connected")];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!(this.client.is_user() && !this.client.is_signed_in())) return [3 /*break*/, 4];
                        return [4 /*yield*/, async_utils_1.once(this.client, "signed_in")];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        if (this.state == "closed")
                            return [2 /*return*/];
                        dbg("do syncstring write query...");
                        return [4 /*yield*/, async_utils_1.callback2(this.client.query, {
                                query: {
                                    syncstrings: {
                                        string_id: this.string_id,
                                        project_id: this.project_id,
                                        path: this.path,
                                        doctype: JSON.stringify(this.doctype),
                                    },
                                },
                            })];
                    case 5:
                        _a.sent();
                        dbg("wrote syncstring to db - done.");
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.synctable = function (query, options, throttle_changes) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        this.assert_not_closed("synctable");
                        if (this.persistent && this.data_server == "project") {
                            options = options.concat([{ persistent: true }]);
                        }
                        _a = this.data_server;
                        switch (_a) {
                            case "project": return [3 /*break*/, 1];
                            case "database": return [3 /*break*/, 3];
                        }
                        return [3 /*break*/, 5];
                    case 1: return [4 /*yield*/, this.client.synctable_project(this.project_id, query, options, throttle_changes, this.id)];
                    case 2: return [2 /*return*/, _b.sent()];
                    case 3: return [4 /*yield*/, this.client.synctable_database(query, options, throttle_changes)];
                    case 4: return [2 /*return*/, _b.sent()];
                    case 5: throw Error("uknown server " + this.data_server);
                }
            });
        });
    };
    SyncDoc.prototype.init_syncstring_table = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, query, _a, is_not_archived;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        dbg = this.dbg("init_syncstring_table");
                        query = {
                            syncstrings: [
                                {
                                    string_id: this.string_id,
                                    project_id: this.project_id,
                                    path: this.path,
                                    users: null,
                                    last_snapshot: null,
                                    snapshot_interval: null,
                                    save: null,
                                    last_active: null,
                                    init: null,
                                    read_only: null,
                                    last_file_change: null,
                                    doctype: null,
                                    archived: null,
                                    settings: null,
                                },
                            ],
                        };
                        dbg("getting table...");
                        _a = this;
                        return [4 /*yield*/, this.synctable(query, [])];
                    case 1:
                        _a.syncstring_table = _b.sent();
                        if (!(this.ephemeral && this.client.is_project())) return [3 /*break*/, 3];
                        this.syncstring_table.set({
                            string_id: this.string_id,
                            project_id: this.project_id,
                            path: this.path,
                            doctype: JSON.stringify(this.doctype),
                        });
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 2:
                        _b.sent();
                        return [3 /*break*/, 5];
                    case 3:
                        dbg("waiting for, then handling the first update...");
                        return [4 /*yield*/, this.handle_syncstring_update()];
                    case 4:
                        _b.sent();
                        _b.label = 5;
                    case 5:
                        this.syncstring_table.on("change", this.handle_syncstring_update.bind(this));
                        is_not_archived = function () {
                            var ss = _this.syncstring_table_get_one();
                            if (ss != null) {
                                return !ss.get("archived");
                            }
                            else {
                                return false;
                            }
                        };
                        dbg("waiting for syncstring to be not archived");
                        return [4 /*yield*/, this.syncstring_table.wait(is_not_archived, 120)];
                    case 6:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    // Used for internal debug logging
    SyncDoc.prototype.dbg = function (_f) {
        var _a;
        if (_f === void 0) { _f = ""; }
        if (!((_a = this.client) === null || _a === void 0 ? void 0 : _a.is_project()) || this.state == "closed") {
            return function () {
                var _ = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    _[_i] = arguments[_i];
                }
            };
        }
        return this.client.dbg("sync-doc(\"" + this.path + "\")." + _f);
    };
    SyncDoc.prototype.init_all = function () {
        return __awaiter(this, void 0, void 0, function () {
            var log;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.state !== "init") {
                            throw Error("connect can only be called in init state");
                        }
                        log = this.dbg("init_all");
                        log("ensure syncstring exists in database");
                        this.assert_not_closed("init_all -- before ensuring syncstring exists");
                        return [4 /*yield*/, this.ensure_syncstring_exists_in_db()];
                    case 1:
                        _a.sent();
                        log("syncstring_table");
                        this.assert_not_closed("init_all -- before init_syncstring_table");
                        return [4 /*yield*/, this.init_syncstring_table()];
                    case 2:
                        _a.sent();
                        log("patch_list, cursors, evaluator, ipywidgets");
                        this.assert_not_closed("init_all -- before init patch_list, cursors, evaluator, ipywidgets");
                        return [4 /*yield*/, Promise.all([
                                this.init_patch_list(),
                                this.init_cursors(),
                                this.init_evaluator(),
                                this.init_ipywidgets(),
                            ])];
                    case 3:
                        _a.sent();
                        this.assert_not_closed("init_all -- after init patch_list");
                        this.init_table_close_handlers();
                        log("file_use_interval");
                        this.init_file_use_interval();
                        if (!this.client.is_project()) return [3 /*break*/, 5];
                        log("load_from_disk");
                        // This sets initialized, which is needed to be fully ready.
                        // We keep trying this load from disk until sync-doc is closed
                        // or it succeeds.  It may fail if, e.g., the file is too
                        // large or is not readable by the user. They are informed to
                        // fix the problem... and once they do (and wait up to 10s),
                        // this will finish.
                        return [4 /*yield*/, async_utils_1.retry_until_success({
                                f: this.init_load_from_disk.bind(this),
                                max_delay: 10000,
                                desc: "syncdoc -- load_from_disk",
                            })];
                    case 4:
                        // This sets initialized, which is needed to be fully ready.
                        // We keep trying this load from disk until sync-doc is closed
                        // or it succeeds.  It may fail if, e.g., the file is too
                        // large or is not readable by the user. They are informed to
                        // fix the problem... and once they do (and wait up to 10s),
                        // this will finish.
                        _a.sent();
                        log("done loading from disk");
                        this.assert_not_closed("init_all -- load from disk");
                        _a.label = 5;
                    case 5:
                        log("wait_until_fully_ready");
                        return [4 /*yield*/, this.wait_until_fully_ready()];
                    case 6:
                        _a.sent();
                        this.assert_not_closed("init_all -- after waiting until fully ready");
                        if (this.client.is_project()) {
                            log("init autosave");
                            this.init_project_autosave();
                        }
                        this.update_has_unsaved_changes();
                        log("done");
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.init_error = function () {
        var x = this.syncstring_table.get_one();
        if (x != null && x.get("init") != null) {
            return x.get("init").get("error");
        }
        return undefined;
    };
    // wait until the syncstring table is ready to be
    // used (so extracted from archive, etc.),
    SyncDoc.prototype.wait_until_fully_ready = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, err_4, is_init_and_not_archived, init;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("wait_until_fully_ready");
                        dbg();
                        this.assert_not_closed("wait_until_fully_ready");
                        if (!(this.client.is_user() && this.init_error())) return [3 /*break*/, 4];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.syncstring_table.wait(function () { return !_this.init_error(); }, 5)];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        err_4 = _a.sent();
                        return [3 /*break*/, 4];
                    case 4:
                        is_init_and_not_archived = function (t) {
                            _this.assert_not_closed("is_init_and_not_archived");
                            var tbl = t.get_one();
                            if (tbl == null) {
                                dbg("null");
                                return false;
                            }
                            // init must be set in table and archived must NOT be
                            // set, so patches are loaded from blob store.
                            var init = tbl.get("init");
                            if (init && !tbl.get("archived")) {
                                dbg("good to go");
                                return init.toJS();
                            }
                            else {
                                dbg("not init yet");
                                return false;
                            }
                        };
                        dbg("waiting for init...");
                        return [4 /*yield*/, this.syncstring_table.wait(is_init_and_not_archived.bind(this), 0)];
                    case 5:
                        init = _a.sent();
                        dbg("init done");
                        if (init.error) {
                            throw Error(init.error);
                        }
                        misc_1.assertDefined(this.patch_list);
                        if (!(this.client.is_user() && this.patch_list.count() === 0 && init.size)) return [3 /*break*/, 9];
                        dbg("waiting for patches for nontrivial file");
                        // normally this only happens in a later event loop,
                        // so force it now.
                        dbg("handling patch update queue since", this.patch_list.count());
                        return [4 /*yield*/, this.handle_patch_update_queue()];
                    case 6:
                        _a.sent();
                        misc_1.assertDefined(this.patch_list);
                        dbg("done handling, now ", this.patch_list.count());
                        if (!(this.patch_list.count() === 0)) return [3 /*break*/, 9];
                        // wait for a change -- i.e., project loading the file from
                        // disk and making available...  Because init.size > 0, we know that
                        // there must be SOMETHING in the patches table once initialization is done.
                        // This is the root cause of https://github.com/sagemathinc/cocalc/issues/2382
                        return [4 /*yield*/, async_utils_1.once(this.patches_table, "change")];
                    case 7:
                        // wait for a change -- i.e., project loading the file from
                        // disk and making available...  Because init.size > 0, we know that
                        // there must be SOMETHING in the patches table once initialization is done.
                        // This is the root cause of https://github.com/sagemathinc/cocalc/issues/2382
                        _a.sent();
                        dbg("got patches_table change");
                        return [4 /*yield*/, this.handle_patch_update_queue()];
                    case 8:
                        _a.sent();
                        dbg("handled update queue");
                        _a.label = 9;
                    case 9:
                        this.emit("init");
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.assert_table_is_ready = function (table) {
        var t = this[table + "_table"];
        if (t == null || t.get_state() != "connected") {
            throw Error("Table " + table + " must be connected.  string_id=" + this.string_id);
        }
    };
    SyncDoc.prototype.assert_is_ready = function (desc) {
        if (this.state != "ready") {
            throw Error("must be ready -- " + desc);
        }
    };
    SyncDoc.prototype.wait_until_ready = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_not_closed("wait_until_ready");
                        if (!(this.state !== "ready")) return [3 /*break*/, 2];
                        // wait for a state change to ready.
                        return [4 /*yield*/, async_utils_1.once(this, "ready")];
                    case 1:
                        // wait for a state change to ready.
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    /* Calls wait for the corresponding patches SyncTable, if
       it has been defined.  If it hasn't been defined, it waits
       until it is defined, then calls wait.  Timeout only starts
       when patches_table is already initialized.
    */
    SyncDoc.prototype.wait = function (until, timeout) {
        if (timeout === void 0) { timeout = 30; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.wait_until_ready()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, async_wait_1.wait({ obj: this, until: until, timeout: timeout, change_event: "change" })];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /* Delete the synchronized string and **all** patches from the database
       -- basically delete the complete history of editing this file.
       WARNINGS:
         (1) If a project has this string open, then things may be messed
             up, unless that project is restarted.
         (2) Only available for an **admin** user right now!
  
       To use: from a javascript console in the browser as admin, do:
  
         await smc.client.sync_string({
           project_id:'9f2e5869-54b8-4890-8828-9aeba9a64af4',
           path:'a.txt'}).delete_from_database()
  
       Then make sure project and clients refresh.
  
       WORRY: Race condition where constructor might write stuff as
       it is being deleted?
    */
    SyncDoc.prototype.delete_from_database = function () {
        return __awaiter(this, void 0, void 0, function () {
            var queries, v, i;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.ephemeral) {
                            return [2 /*return*/]; // not in database.
                        }
                        queries = [
                            {
                                patches_delete: {
                                    id: [this.string_id],
                                    dummy: null,
                                },
                            },
                            {
                                syncstrings_delete: {
                                    project_id: this.project_id,
                                    path: this.path,
                                },
                            },
                        ];
                        v = [];
                        for (i = 0; i < queries.length; i++) {
                            v.push(async_utils_1.callback2(this.client.query, { query: queries[i] }));
                        }
                        return [4 /*yield*/, Promise.all(v)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.file_is_read_only = function () {
        return __awaiter(this, void 0, void 0, function () {
            var err_5;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, async_utils_1.callback2(this.client.path_access, {
                                path: this.path,
                                mode: "w",
                            })];
                    case 1:
                        _a.sent();
                        // no error -- it is NOT read only
                        return [2 /*return*/, false];
                    case 2:
                        err_5 = _a.sent();
                        // error -- it is read only.
                        return [2 /*return*/, true];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.update_if_file_is_read_only = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this.set_read_only;
                        return [4 /*yield*/, this.file_is_read_only()];
                    case 1:
                        _a.apply(this, [_b.sent()]);
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.init_load_from_disk = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.state == "closed") {
                            // stop trying, no error -- this is assumed
                            // in a retry_until_success elsewhere.
                            return [2 /*return*/];
                        }
                        return [4 /*yield*/, this.load_from_disk_if_newer()];
                    case 1:
                        if (_a.sent()) {
                            throw Error("failed to load from disk");
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.load_from_disk_if_newer = function () {
        return __awaiter(this, void 0, void 0, function () {
            var last_changed, dbg, is_read_only, size, error, stats, err_6;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        last_changed = this.last_changed();
                        dbg = this.dbg("load_from_disk_if_newer");
                        is_read_only = false;
                        size = 0;
                        error = "";
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 9, , 10]);
                        dbg("check if path exists");
                        return [4 /*yield*/, async_utils_1.callback2(this.client.path_exists, { path: this.path })];
                    case 2:
                        if (!_a.sent()) return [3 /*break*/, 8];
                        // the path exists
                        dbg("path exists -- stat file");
                        return [4 /*yield*/, async_utils_1.callback2(this.client.path_stat, {
                                path: this.path,
                            })];
                    case 3:
                        stats = _a.sent();
                        if (!(stats.ctime > last_changed)) return [3 /*break*/, 5];
                        dbg("disk file changed more recently than edits, so loading");
                        return [4 /*yield*/, this.load_from_disk()];
                    case 4:
                        size = _a.sent();
                        dbg("loaded");
                        return [3 /*break*/, 6];
                    case 5:
                        dbg("stick with database version");
                        _a.label = 6;
                    case 6:
                        dbg("checking if read only");
                        return [4 /*yield*/, this.file_is_read_only()];
                    case 7:
                        is_read_only = _a.sent();
                        dbg("read_only", is_read_only);
                        _a.label = 8;
                    case 8: return [3 /*break*/, 10];
                    case 9:
                        err_6 = _a.sent();
                        error = err_6.toString() + " -- " + err_6.stack;
                        return [3 /*break*/, 10];
                    case 10: return [4 /*yield*/, this.set_initialized(error, is_read_only, size)];
                    case 11:
                        _a.sent();
                        dbg("done");
                        return [2 /*return*/, !!error];
                }
            });
        });
    };
    SyncDoc.prototype.patch_table_query = function (cutoff) {
        var query = {
            string_id: this.string_id,
            time: cutoff ? { ">=": cutoff } : null,
            // compressed format patch as a JSON *string*
            patch: null,
            // integer id of user (maps to syncstring table)
            user_id: null,
            // (optional) a snapshot at this point in time
            snapshot: null,
            // (optional) when patch actually sent, which may
            // be later than when made
            sent: null,
            // (optional) timestamp of previous patch sent
            // from this session
            prev: null,
        };
        if (this.doctype.patch_format != null) {
            query.format = this.doctype.patch_format;
        }
        return query;
    };
    SyncDoc.prototype.init_patch_list = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, patch_list, _a, update_has_unsaved_changes, doc;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        dbg = this.dbg("init_patch_list");
                        dbg();
                        this.assert_not_closed("init_patch_list - start");
                        // CRITICAL: note that handle_syncstring_update checks whether
                        // init_patch_list is done by testing whether this.patch_list is defined!
                        // That is why we first define "patch_list" below, then set this.patch_list
                        // to it only after we're done.
                        delete this.patch_list;
                        patch_list = new sorted_patch_list_1.SortedPatchList(this._from_str);
                        dbg("opening the table...");
                        _a = this;
                        return [4 /*yield*/, this.synctable({ patches: [this.patch_table_query(this.last_snapshot)] }, [], this.patch_interval)];
                    case 1:
                        _a.patches_table = _b.sent();
                        this.assert_not_closed("init_patch_list -- after making synctable");
                        update_has_unsaved_changes = lodash_1.debounce(this.update_has_unsaved_changes.bind(this), 500, { leading: true, trailing: true });
                        this.patches_table.on("has-uncommitted-changes", function (val) {
                            _this.emit("has-uncommitted-changes", val);
                        });
                        this.on("change", function () {
                            update_has_unsaved_changes();
                        });
                        this.syncstring_table.on("change", function () {
                            update_has_unsaved_changes();
                        });
                        dbg("adding patches");
                        patch_list.add(this.get_patches());
                        doc = patch_list.value();
                        this.last = this.doc = doc;
                        this.patches_table.on("change", this.handle_patch_update.bind(this));
                        this.patches_table.on("before-change", function () { return _this.emit("before-change"); });
                        this.patches_table.on("saved", this.handle_offline.bind(this));
                        this.patch_list = patch_list;
                        dbg("done");
                        return [2 /*return*/];
                }
            });
        });
    };
    /*
      _check_for_timestamp_collision: (t) =>
          obj = this._my_patches[t]
          if not obj?
              return
          key = this._patches_table.key(obj)
          if obj.patch != this._patches_table.get(key)?.get('patch')
              *console.log("COLLISION! #{t}, #{obj.patch}, #{this._patches_table.get(key).get('patch')}")
              * We fix the collision by finding the nearest time after time that
              * is available, and reinserting our patch at that new time.
              this._my_patches[t] = 'killed'
              new_time = this.patch_list.next_available_time(new Date(t), this._user_id, this._users.length)
              this._save_patch(new_time, JSON.parse(obj.patch))
      */
    SyncDoc.prototype.init_evaluator = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, ext;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("init_evaluator");
                        ext = misc_1.filename_extension(this.path);
                        if (ext !== "sagews") {
                            dbg("done -- only use init_evaluator for sagews");
                            return [2 /*return*/];
                        }
                        dbg("creating the evaluator and waiting for init");
                        this.evaluator = new evaluator_1.Evaluator(this, this.client, this.synctable.bind(this));
                        return [4 /*yield*/, this.evaluator.init()];
                    case 1:
                        _a.sent();
                        dbg("done");
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.init_ipywidgets = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, ext;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("init_evaluator");
                        ext = misc_1.filename_extension(this.path);
                        if (ext != "sage-jupyter2") {
                            dbg("done -- only use ipywidgets for jupyter");
                            return [2 /*return*/];
                        }
                        dbg("creating the ipywidgets state table, and waiting for init");
                        this.ipywidgets_state = new ipywidgets_state_1.IpywidgetsState(this, this.client, this.synctable.bind(this));
                        return [4 /*yield*/, this.ipywidgets_state.init()];
                    case 1:
                        _a.sent();
                        dbg("done");
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.init_cursors = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, query, options, _a, s;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        dbg = this.dbg("init_cursors");
                        if (!this.client.is_user()) {
                            dbg("done -- only users care about cursors.");
                            return [2 /*return*/];
                        }
                        if (!this.cursors) {
                            dbg("done -- do not care about cursors for this syncdoc.");
                            return [2 /*return*/];
                        }
                        dbg("getting cursors ephemeral table");
                        query = {
                            cursors: [
                                {
                                    string_id: this.string_id,
                                    user_id: null,
                                    locs: null,
                                    time: null,
                                },
                            ],
                        };
                        if (this.data_server == "project") {
                            options = [{ ephemeral: true }, { queue_size: 1 }];
                        }
                        else {
                            options = [];
                        }
                        _a = this;
                        return [4 /*yield*/, this.synctable(query, options, 1000)];
                    case 1:
                        _a.cursors_table = _b.sent();
                        this.assert_not_closed("init_cursors -- after making synctable");
                        // cursors now initialized; first initialize the
                        // local this._cursor_map, which tracks positions
                        // of cursors by account_id:
                        dbg("loading initial state");
                        s = this.cursors_table.get();
                        if (s == null) {
                            throw Error("bug -- get should not return null once table initialized");
                        }
                        s.forEach(function (locs, k) {
                            if (locs == null) {
                                return;
                            }
                            var u = JSON.parse(k);
                            if (u != null) {
                                _this.cursor_map = _this.cursor_map.set(_this.users[u[1]], locs);
                            }
                        });
                        this.cursors_table.on("change", this.handle_cursors_change.bind(this));
                        this.set_cursor_locs = lodash_1.debounce(this.set_cursor_locs.bind(this), 2000, {
                            leading: true,
                            trailing: true,
                        });
                        dbg("done");
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.handle_cursors_change = function (keys) {
        var e_3, _a;
        if (this.state === "closed") {
            return;
        }
        try {
            for (var keys_1 = __values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                var k = keys_1_1.value;
                var u = JSON.parse(k);
                if (u == null) {
                    continue;
                }
                var account_id = this.users[u[1]];
                var locs = this.cursors_table.get(k);
                if (locs == null && !this.cursor_map.has(account_id)) {
                    // gone, and already gone.
                    continue;
                }
                if (locs != null) {
                    // changed
                    this.cursor_map = this.cursor_map.set(account_id, locs);
                }
                else {
                    // deleted
                    this.cursor_map = this.cursor_map.delete(account_id);
                }
                this.emit("cursor_activity", account_id);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    /* Returns *immutable* Map from account_id to list
       of cursor positions, if cursors are enabled.
    */
    SyncDoc.prototype.get_cursors = function () {
        var e_4, _a;
        if (!this.cursors) {
            throw Error("cursors are not enabled");
        }
        if (this.cursors_table == null) {
            return immutable_1.Map(); // not loaded yet -- so no info yet.
        }
        var account_id = this.client.client_id();
        var map = this.cursor_map;
        if (map.has(account_id) &&
            this.cursor_last_time >= map.getIn([account_id, "time"])) {
            map = map.delete(account_id);
        }
        // Remove any old cursors, where "old" is more than 1 minute old; this is never useful.
        var cutoff = misc_1.server_minutes_ago(1);
        try {
            for (var _b = __values(map), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 1), a = _d[0];
                if (map.getIn([a, "time"]) < cutoff) {
                    map = map.delete(a);
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return map;
    };
    /* Set settings map.  Used for custom configuration just for
       this one file, e.g., overloading the spell checker language.
     */
    SyncDoc.prototype.set_settings = function (obj) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_is_ready("set_settings");
                        this.syncstring_table.set({
                            string_id: this.string_id,
                            project_id: this.project_id,
                            path: this.path,
                            settings: obj,
                        });
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    // get settings object
    SyncDoc.prototype.get_settings = function () {
        this.assert_is_ready("get_settings");
        return this.syncstring_table_get_one().get("settings", immutable_1.Map());
    };
    /*
    Commits and saves current live syncdoc to backend.  It's safe to
    call this frequently or multiple times at once, since
    it is wrapped in reuseInFlight in the constructor.
  
    Function only returns when there is nothing needing
    saving.
  
    Save any changes we have as a new patch.
    */
    SyncDoc.prototype.save = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, doc;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("save");
                        dbg();
                        if (this.client.is_deleted(this.path, this.project_id)) {
                            dbg("not saving because deleted");
                            return [2 /*return*/];
                        }
                        // We just keep trying while syncdoc is ready and there
                        // are changes that have not been saved (due to this.doc
                        // changing during the while loop!).
                        if (this.doc == null || this.last == null) {
                            dbg("bug -- not ready");
                            throw Error("bug -- cannot save if doc and last are not initialized");
                        }
                        if (this.state == "closed") {
                            // There's nothing to do regarding save if the table is
                            // already closed.  Note that we *do* have to save when
                            // the table is init stage, since the project has to
                            // record the newly opened version of the file to the
                            // database! See
                            //    https://github.com/sagemathinc/cocalc/issues/4986
                            dbg("state=" + this.state + " not ready so not saving");
                            return [2 /*return*/];
                        }
                        _a.label = 1;
                    case 1:
                        if (!!this.doc.is_equal(this.last)) return [3 /*break*/, 5];
                        dbg("something to save");
                        this.emit("user-change");
                        doc = this.doc;
                        if (!this.handle_patch_update_queue_running) return [3 /*break*/, 3];
                        dbg("wait until the update queue is done");
                        return [4 /*yield*/, async_utils_1.once(this, "handle_patch_update_queue_done")];
                    case 2:
                        _a.sent();
                        // but wait until next loop (so as to check that needed
                        // and state still ready).
                        return [3 /*break*/, 1];
                    case 3:
                        dbg("Compute new patch.");
                        return [4 /*yield*/, this.sync_remote_and_doc()];
                    case 4:
                        _a.sent();
                        // Emit event since this syncstring was
                        // changed locally (or we wouldn't have had
                        // to save at all).
                        if (doc.is_equal(this.doc)) {
                            dbg("no change during loop -- done!");
                            return [3 /*break*/, 5];
                        }
                        return [3 /*break*/, 1];
                    case 5: 
                    // Ensure all patches are saved to backend.
                    // We do this after the above, so that creating the newest patch
                    // happens immediately on save, which makes it possible for clients
                    // to save current state without having to wait on an async, which is
                    // useful to ensure specific undo points (e.g., right before a paste).
                    return [4 /*yield*/, this.patches_table.save()];
                    case 6:
                        // Ensure all patches are saved to backend.
                        // We do this after the above, so that creating the newest patch
                        // happens immediately on save, which makes it possible for clients
                        // to save current state without having to wait on an async, which is
                        // useful to ensure specific undo points (e.g., right before a paste).
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.next_patch_time = function () {
        var time = this.client.server_time();
        misc_1.assertDefined(this.patch_list);
        var min_time = this.patch_list.newest_patch_time();
        if (min_time != null && min_time >= time) {
            time = new Date(min_time.valueOf() + 1);
        }
        time = this.patch_list.next_available_time(time, this.my_user_id, this.users.length);
        return time;
    };
    SyncDoc.prototype.commit_patch = function (time, patch) {
        this.assert_not_closed("commit_patch");
        var obj = {
            // version for database
            string_id: this.string_id,
            time: time,
            patch: JSON.stringify(patch),
            user_id: this.my_user_id,
        };
        this.my_patches[time.valueOf()] = obj;
        if (this.doctype.patch_format != null) {
            obj.format = this.doctype.patch_format;
        }
        if (this.save_patch_prev != null) {
            // timestamp of last saved patch during this session
            obj.prev = this.save_patch_prev;
        }
        this.save_patch_prev = time;
        // If in undo mode put the just-created patch in our
        // without timestamp list, so it won't be included
        // when doing undo/redo.
        if (this.undo_state != null) {
            this.undo_state.without.unshift(time);
        }
        //console.log 'saving patch with time ', time.valueOf()
        var x = this.patches_table.set(obj, "none");
        var y = this.process_patch(x, undefined, undefined, patch);
        if (y != null) {
            misc_1.assertDefined(this.patch_list);
            this.patch_list.add([y]);
        }
    };
    /* Create and store in the database a snapshot of the state
       of the string at the given point in time.  This should
       be the time of an existing patch.
    */
    SyncDoc.prototype.snapshot = function (time, force) {
        if (force === void 0) { force = false; }
        return __awaiter(this, void 0, void 0, function () {
            var x, snapshot, obj, success, i, x_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        misc_1.assertDefined(this.patch_list);
                        x = this.patch_list.patch(time);
                        if (x == null) {
                            throw Error("no patch at time " + time);
                        }
                        if (x.snapshot != null && !force) {
                            // there is already a snapshot at this point in time,
                            // so nothing further to do.
                            return [2 /*return*/];
                        }
                        snapshot = this.patch_list.value(time, force).to_str();
                        obj = {
                            string_id: this.string_id,
                            time: time,
                            patch: JSON.stringify(x.patch),
                            snapshot: snapshot,
                            user_id: x.user_id,
                        };
                        if (force) {
                            /* CRITICAL: We are sending the patch/snapshot later, but
                               it was valid.   It's important to make this clear or
                               this.handle_offline will recompute this snapshot and
                               try to update sent on it again, which leads to serious
                               problems!
                            */
                            obj.sent = time;
                        }
                        // also set snapshot in the this.patch_list, which
                        // helps with optimization
                        x.snapshot = obj.snapshot;
                        this.patches_table.set(obj, "none");
                        return [4 /*yield*/, this.patches_table.save()];
                    case 1:
                        _a.sent();
                        if (this.state != "ready")
                            return [2 /*return*/];
                        if (!!this.ephemeral) return [3 /*break*/, 8];
                        success = false;
                        i = 0;
                        _a.label = 2;
                    case 2:
                        if (!(i < 2)) return [3 /*break*/, 7];
                        return [4 /*yield*/, async_utils_1.callback2(this.client.query, {
                                query: {
                                    patches: {
                                        string_id: this.string_id,
                                        time: time,
                                        snapshot: null,
                                    },
                                },
                            })];
                    case 3:
                        x_1 = _a.sent();
                        if (this.state != "ready")
                            return [2 /*return*/];
                        if (!(x_1.query.patches == null || x_1.query.patches.snapshot != snapshot)) return [3 /*break*/, 5];
                        return [4 /*yield*/, awaiting_1.delay((i + 1) * 5000)];
                    case 4:
                        _a.sent();
                        return [3 /*break*/, 6];
                    case 5:
                        success = true;
                        return [3 /*break*/, 7];
                    case 6:
                        i++;
                        return [3 /*break*/, 2];
                    case 7:
                        if (!success) {
                            throw Error("unable to confirm that snapshot was saved to the database");
                        }
                        _a.label = 8;
                    case 8:
                        if (this.state != "ready")
                            return [2 /*return*/];
                        this.syncstring_table.set({
                            string_id: this.string_id,
                            project_id: this.project_id,
                            path: this.path,
                            last_snapshot: time,
                        });
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 9:
                        _a.sent();
                        this.last_snapshot = time;
                        return [2 /*return*/];
                }
            });
        });
    };
    // Have a snapshot every this.snapshot_interval patches, except
    // for the very last interval.
    SyncDoc.prototype.snapshot_if_necessary = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, max_size, interval, time;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("snapshot_if_necessary");
                        if (this.get_state() !== "ready")
                            return [2 /*return*/];
                        max_size = Math.floor(1.2 * MAX_FILE_SIZE_MB * 1000000);
                        interval = this.snapshot_interval;
                        dbg("check if we need to make a snapshot:", { interval: interval, max_size: max_size });
                        misc_1.assertDefined(this.patch_list);
                        time = this.patch_list.time_of_unmade_periodic_snapshot(interval, max_size);
                        if (!(time != null)) return [3 /*break*/, 2];
                        dbg("yes, make a snapshot at time", time);
                        return [4 /*yield*/, this.snapshot(time)];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        dbg("no need to make a snapshot yet");
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /*- x - patch object
      - time0, time1: optional range of times
          return undefined if patch not in this range
      - patch: if given will be used as an actual patch
          instead of x.patch, which is a JSON string.
    */
    SyncDoc.prototype.process_patch = function (x, time0, time1, patch) {
        var t = x.get("time");
        if (!misc_1.is_date(t)) {
            // who knows what is in the database...
            try {
                t = misc_1.ISO_to_Date(t);
                if (isNaN(t)) {
                    // ignore patches with bad times
                    return;
                }
            }
            catch (err) {
                // ignore patches with invalid times
                return;
            }
        }
        var time = t;
        if ((time0 != null && time < time0) || (time1 != null && time > time1)) {
            // out of range
            return;
        }
        var user_id = x.get("user_id");
        var sent = x.get("sent");
        var prev = x.get("prev");
        var size;
        if (patch == null) {
            /* Do **NOT** use misc.from_json, since we definitely
               do not want to unpack ISO timestamps as Date,
               since patch just contains the raw patches from
               user editing.  This was done for a while, which
               led to horrific bugs in some edge cases...
               See https://github.com/sagemathinc/cocalc/issues/1771
            */
            if (x.has("patch")) {
                var p = x.get("patch");
                patch = JSON.parse(p);
                size = p.length;
            }
            else {
                patch = [];
                size = 2;
            }
        }
        else {
            var p = x.get("patch");
            // Looking at other code, I think this JSON.stringify (which
            // would be a waste of time) never gets called in practice.
            size = p != null ? p.length : JSON.stringify(patch).length;
        }
        var obj = {
            time: time,
            user_id: user_id,
            patch: patch,
            size: size,
        };
        var snapshot = x.get("snapshot");
        if (sent != null) {
            obj.sent = sent;
        }
        if (prev != null) {
            obj.prev = prev;
        }
        if (snapshot != null) {
            obj.snapshot = snapshot;
        }
        return obj;
    };
    /* Return all patches with time such that
              time0 <= time <= time1;
       If time0 undefined then sets time0 equal to time of last_snapshot.
       If time1 undefined treated as +oo.
    */
    SyncDoc.prototype.get_patches = function (time0, time1) {
        var _this = this;
        this.assert_table_is_ready("patches");
        if (time0 == null) {
            time0 = this.last_snapshot;
        }
        // m below is an immutable map with keys the string that
        // is the JSON version of the primary key
        // [string_id, timestamp, user_number].
        var m = this.patches_table.get();
        if (m == null) {
            // won't happen because of assert above.
            throw Error("patches_table must be initialized");
        }
        var v = [];
        m.forEach(function (x, _) {
            var p = _this.process_patch(x, time0, time1);
            if (p != null) {
                return v.push(p);
            }
        });
        v.sort(util_1.patch_cmp);
        return v;
    };
    SyncDoc.prototype.has_full_history = function () {
        return !this.last_snapshot || this.load_full_history_done;
    };
    SyncDoc.prototype.load_full_history = function () {
        return __awaiter(this, void 0, void 0, function () {
            var query, result, v;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        //dbg = this.dbg("load_full_history")
                        //dbg()
                        if (this.has_full_history() || this.ephemeral) {
                            //dbg("nothing to do, since complete history definitely already loaded")
                            return [2 /*return*/];
                        }
                        query = this.patch_table_query();
                        return [4 /*yield*/, async_utils_1.callback2(this.client.query, {
                                query: { patches: [query] },
                            })];
                    case 1:
                        result = _a.sent();
                        v = [];
                        // process_patch assumes immutable objects
                        immutable_1.fromJS(result.query.patches).forEach(function (x) {
                            var p = _this.process_patch(x, new Date(0), _this.last_snapshot);
                            if (p != null) {
                                v.push(p);
                            }
                        });
                        misc_1.assertDefined(this.patch_list);
                        this.patch_list.add(v);
                        this.load_full_history_done = true;
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.show_history = function (opts) {
        if (opts === void 0) { opts = {}; }
        misc_1.assertDefined(this.patch_list);
        this.patch_list.show_history(opts);
    };
    SyncDoc.prototype.set_snapshot_interval = function (n) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.syncstring_table.set(this.syncstring_table_get_one().set("snapshot_interval", n));
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /* Check if any patches that just got confirmed as saved
       are relatively old; if so, we mark them as such and
       also possibly recompute snapshots.
    */
    SyncDoc.prototype.handle_offline = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var now, oldest, data_1, data_1_1, obj, _a, _b, snapshot_time, e_5_1;
            var e_6, _c, e_5, _d;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        //dbg = this.dbg("handle_offline")
                        //dbg("data='#{misc.to_json(data)}'")
                        this.assert_not_closed("handle_offline");
                        now = this.client.server_time();
                        oldest = undefined;
                        try {
                            for (data_1 = __values(data), data_1_1 = data_1.next(); !data_1_1.done; data_1_1 = data_1.next()) {
                                obj = data_1_1.value;
                                if (obj.sent) {
                                    // CRITICAL: ignore anything already processed! (otherwise, infinite loop)
                                    continue;
                                }
                                if (now.valueOf() - obj.time.valueOf() >= 1000 * OFFLINE_THRESH_S) {
                                    // patch is "old" -- mark it as likely being sent as a result of being
                                    // offline, so clients could potentially discard it.
                                    obj.sent = now;
                                    this.patches_table.set(obj);
                                    this.patches_table.save();
                                    if (oldest == null || obj.time < oldest) {
                                        oldest = obj.time;
                                    }
                                }
                            }
                        }
                        catch (e_6_1) { e_6 = { error: e_6_1 }; }
                        finally {
                            try {
                                if (data_1_1 && !data_1_1.done && (_c = data_1.return)) _c.call(data_1);
                            }
                            finally { if (e_6) throw e_6.error; }
                        }
                        if (!oldest) return [3 /*break*/, 8];
                        //dbg("oldest=#{oldest}, so check whether any snapshots need to be recomputed")
                        misc_1.assertDefined(this.patch_list);
                        _e.label = 1;
                    case 1:
                        _e.trys.push([1, 6, 7, 8]);
                        _a = __values(this.patch_list.snapshot_times()), _b = _a.next();
                        _e.label = 2;
                    case 2:
                        if (!!_b.done) return [3 /*break*/, 5];
                        snapshot_time = _b.value;
                        if (!(snapshot_time >= oldest)) return [3 /*break*/, 4];
                        //console.log("recomputing snapshot #{snapshot_time}")
                        return [4 /*yield*/, this.snapshot(snapshot_time, true)];
                    case 3:
                        //console.log("recomputing snapshot #{snapshot_time}")
                        _e.sent();
                        _e.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_5_1 = _e.sent();
                        e_5 = { error: e_5_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                        }
                        finally { if (e_5) throw e_5.error; }
                        return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.get_last_save_to_disk_time = function () {
        return this.last_save_to_disk_time;
    };
    SyncDoc.prototype.handle_syncstring_save_state = function (state, time) {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, err_7;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // Called when the save state changes.
                        /* this.syncstring_save_state is used to make it possible to emit a
                           'save-to-disk' event, whenever the state changes
                           to indicate a save completed.
                    
                           NOTE: it is intentional that this.syncstring_save_state is not defined
                           the first time this function is called, so that save-to-disk
                           with last save time gets emitted on initial load (which, e.g., triggers
                           latex compilation properly in case of a .tex file).
                        */
                        if (state === "done" && this.syncstring_save_state !== "done") {
                            this.last_save_to_disk_time = time;
                            this.emit("save-to-disk", time);
                        }
                        dbg = this.dbg("handle_syncstring_save_state");
                        dbg("state=" + state + "; this.syncstring_save_state=" + this.syncstring_save_state + "; this.state=" + state);
                        if (!(this.state === "ready" &&
                            this.client.is_project() &&
                            this.syncstring_save_state !== "requested" &&
                            state === "requested")) return [3 /*break*/, 8];
                        this.syncstring_save_state = state; // only used in the if above
                        dbg("requesting save to disk -- calling save_to_disk");
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, 5, 8]);
                        // Uncomment the following to test simulating a
                        // random failure in save_to_disk:
                        // if (Math.random() < 0.5) throw Error("CHAOS MONKEY!"); // FOR TESTING ONLY.
                        return [4 /*yield*/, this.save_to_disk()];
                    case 2:
                        // Uncomment the following to test simulating a
                        // random failure in save_to_disk:
                        // if (Math.random() < 0.5) throw Error("CHAOS MONKEY!"); // FOR TESTING ONLY.
                        _a.sent();
                        return [3 /*break*/, 8];
                    case 3:
                        err_7 = _a.sent();
                        // CRITICAL: we must unset this.syncstring_save_state (and set the save state);
                        // otherwise, it stays as "requested" and this if statement would never get
                        // run again, thus completely breaking saving this doc to disk.
                        // It is normal behavior that *sometimes* this.save_to_disk might
                        // throw an exception, e.g., if the file is temporarily deleted
                        // or save it called before everything is initialized, or file
                        // is temporarily set readonly, or maybe there is a filesystem error.
                        // Of course, the finally below will also take care of this.  However,
                        // it's nice to record the error here.
                        this.syncstring_save_state = "done";
                        return [4 /*yield*/, this.set_save({ state: "done", error: "" + err_7 })];
                    case 4:
                        _a.sent();
                        dbg("ERROR saving to disk in handle_syncstring_save_state-- " + err_7);
                        return [3 /*break*/, 8];
                    case 5:
                        // No matter what, after the above code is run,
                        // the save state in the table better be "done".
                        // We triple check that here, though of course
                        // we believe the logic in save_to_disk and above
                        // should always accomplish this.
                        dbg("had to set the state to done in finally block");
                        if (!(this.state === "ready" &&
                            (this.syncstring_save_state != "done" ||
                                this.syncstring_table_get_one().getIn(["save", "state"]) != "done"))) return [3 /*break*/, 7];
                        this.syncstring_save_state = "done";
                        return [4 /*yield*/, this.set_save({ state: "done", error: "" })];
                    case 6:
                        _a.sent();
                        _a.label = 7;
                    case 7: return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.handle_syncstring_update = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, data, x;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("handle_syncstring_update");
                        dbg();
                        if (this.state === "closed") {
                            return [2 /*return*/];
                        }
                        data = this.syncstring_table_get_one();
                        x = data != null ? data.toJS() : undefined;
                        if (x != null && x.save != null) {
                            this.handle_syncstring_save_state(x.save.state, x.save.time);
                        }
                        dbg(JSON.stringify(x));
                        if (!(x == null || x.users == null)) return [3 /*break*/, 2];
                        dbg("new_document");
                        return [4 /*yield*/, this.handle_syncstring_update_new_document()];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 2:
                        dbg("update_existing");
                        return [4 /*yield*/, this.handle_syncstring_update_existing_document(x, data)];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.handle_syncstring_update_new_document = function () {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function () {
            var obj;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        // Brand new document
                        this.emit("load-time-estimate", { type: "new", time: 1 });
                        this.last_snapshot = undefined;
                        this.snapshot_interval = (_b = (_a = schema.SCHEMA.syncstrings.user_query) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.fields.snapshot_interval;
                        // Brand new syncstring
                        // TODO: worry about race condition with everybody making themselves
                        // have user_id 0... ?
                        this.my_user_id = 0;
                        this.users = [this.client.client_id()];
                        obj = {
                            string_id: this.string_id,
                            project_id: this.project_id,
                            path: this.path,
                            last_snapshot: this.last_snapshot,
                            users: this.users,
                            doctype: JSON.stringify(this.doctype),
                            last_active: this.client.server_time(),
                        };
                        this.syncstring_table.set(obj);
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 1:
                        _c.sent();
                        this.settings = immutable_1.Map();
                        this.emit("metadata-change");
                        this.emit("settings-change", this.settings);
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.handle_syncstring_update_existing_document = function (x, data) {
        return __awaiter(this, void 0, void 0, function () {
            var settings, client_id;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // Existing document.
                        if (this.path == null) {
                            // We just opened the file -- emit a load time estimate.
                            if (x.archived) {
                                this.emit("load-time-estimate", { type: "archived", time: 3 });
                            }
                            else {
                                this.emit("load-time-estimate", { type: "ready", time: 1 });
                            }
                        }
                        // TODO: handle doctype change here (?)
                        this.last_snapshot = x.last_snapshot;
                        this.snapshot_interval = x.snapshot_interval;
                        this.users = x.users;
                        this.project_id = x.project_id;
                        this.path = x.path;
                        settings = data.get("settings", immutable_1.Map());
                        if (settings !== this.settings) {
                            this.settings = settings;
                            this.emit("settings-change", settings);
                        }
                        client_id = this.client.client_id();
                        this.my_user_id = this.users.indexOf(client_id);
                        if (!(this.my_user_id === -1)) return [3 /*break*/, 2];
                        this.my_user_id = this.users.length;
                        this.users.push(client_id);
                        this.syncstring_table.set({
                            string_id: this.string_id,
                            project_id: this.project_id,
                            path: this.path,
                            users: this.users,
                        });
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        this.emit("metadata-change");
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.init_watch = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.client.is_project()) {
                            return [2 /*return*/];
                        }
                        if (!(this.watch_path !== this.path)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.update_watch_path(this.path)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [4 /*yield*/, this.pending_save_to_disk()];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.pending_save_to_disk = function () {
        return __awaiter(this, void 0, void 0, function () {
            var x, err_8, dbg;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_table_is_ready("syncstring");
                        if (!this.client.is_project()) {
                            return [2 /*return*/];
                        }
                        x = this.syncstring_table.get_one();
                        if (!(x != null && x.getIn(["save", "state"]) === "requested")) return [3 /*break*/, 4];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.save_to_disk()];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        err_8 = _a.sent();
                        dbg = this.dbg("pending_save_to_disk");
                        dbg("ERROR saving to disk in pending_save_to_disk -- " + err_8);
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.update_watch_path = function (path) {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, data, err_9;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("update_watch_path");
                        if (this.file_watcher != null) {
                            // clean up
                            dbg("close");
                            this.file_watcher.close();
                            delete this.file_watcher;
                            delete this.watch_path;
                        }
                        if (path == null) {
                            dbg("not opening another watcher");
                            this.watch_path = path;
                            return [2 /*return*/];
                        }
                        if (this.watch_path != null) {
                            dbg("watch_path already defined");
                            return [2 /*return*/];
                        }
                        dbg("opening watcher...");
                        if (this.state === "closed") {
                            throw Error("must not be closed");
                        }
                        this.watch_path = path;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 5, , 6]);
                        return [4 /*yield*/, async_utils_1.callback2(this.client.path_exists, { path: path })];
                    case 2:
                        if (!!(_a.sent())) return [3 /*break*/, 4];
                        // path does not exist
                        dbg("write '" + path + "' to disk from syncstring in-memory database version");
                        data = this.to_str();
                        return [4 /*yield*/, async_utils_1.callback2(this.client.write_file, { path: path, data: data })];
                    case 3:
                        _a.sent();
                        dbg("wrote '" + path + "' to disk");
                        _a.label = 4;
                    case 4: return [3 /*break*/, 6];
                    case 5:
                        err_9 = _a.sent();
                        // This should happen, e.g, if path is read only.
                        dbg("could NOT write '" + path + "' to disk -- " + err_9);
                        // In this case, can't really setup a file watcher.
                        return [2 /*return*/];
                    case 6:
                        dbg("now requesting to watch file");
                        this.file_watcher = this.client.watch_file({ path: path });
                        this.file_watcher.on("change", this.handle_file_watcher_change.bind(this));
                        this.file_watcher.on("delete", this.handle_file_watcher_delete.bind(this));
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.handle_file_watcher_change = function (ctime) {
        var dbg = this.dbg("handle_file_watcher_change");
        var time = ctime.valueOf();
        dbg("file_watcher: change, ctime=" + time + ", this.save_to_disk_start_ctime=" + this.save_to_disk_start_ctime + ", this.save_to_disk_end_ctime=" + this.save_to_disk_end_ctime);
        if (this.save_to_disk_start_ctime == null ||
            (this.save_to_disk_end_ctime != null &&
                time - this.save_to_disk_end_ctime >= 7 * 1000)) {
            // Either we never saved to disk, or the last attempt
            // to save was at least 7s ago, and it finished,
            // so definitely this change event was not caused by it.
            dbg("load_from_disk since no recent save to disk");
            this.load_from_disk();
            return;
        }
    };
    SyncDoc.prototype.handle_file_watcher_delete = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("handle_file_watcher_delete");
                        this.assert_is_ready("handle_file_watcher_delete");
                        dbg("delete: set_deleted and closing");
                        return [4 /*yield*/, this.client.set_deleted(this.path, this.project_id)];
                    case 1:
                        _a.sent();
                        this.close();
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.load_from_disk = function () {
        return __awaiter(this, void 0, void 0, function () {
            var path, dbg, exists, size, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        path = this.path;
                        dbg = this.dbg("load_from_disk");
                        dbg();
                        return [4 /*yield*/, async_utils_1.callback2(this.client.path_exists, { path: path })];
                    case 1:
                        exists = _a.sent();
                        if (!!exists) return [3 /*break*/, 2];
                        dbg("file no longer exists -- setting to blank");
                        size = 0;
                        this.from_str("");
                        return [3 /*break*/, 6];
                    case 2:
                        dbg("file exists");
                        return [4 /*yield*/, this.update_if_file_is_read_only()];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, async_utils_1.callback2(this.client.path_read, {
                                path: path,
                                maxsize_MB: MAX_FILE_SIZE_MB,
                            })];
                    case 4:
                        data = _a.sent();
                        size = data.length;
                        dbg("got it -- length=" + size);
                        this.from_str(data);
                        // we also know that this is the version on disk, so we update the hash
                        this.commit();
                        return [4 /*yield*/, this.set_save({
                                state: "done",
                                error: "",
                                hash: misc_1.hash_string(data),
                            })];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6: 
                    // save new version (just set via from_str) to database.
                    return [4 /*yield*/, this.save()];
                    case 7:
                        // save new version (just set via from_str) to database.
                        _a.sent();
                        return [2 /*return*/, size];
                }
            });
        });
    };
    SyncDoc.prototype.set_save = function (x) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_table_is_ready("syncstring");
                        // set timestamp of when the save happened; this can be useful
                        // for coordinating running code, etc.... and is just generally useful.
                        x.time = new Date().valueOf();
                        this.syncstring_table.set(this.syncstring_table_get_one().set("save", immutable_1.fromJS(x)));
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.set_read_only = function (read_only) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_table_is_ready("syncstring");
                        this.syncstring_table.set(this.syncstring_table_get_one().set("read_only", read_only));
                        return [4 /*yield*/, this.syncstring_table.save()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.is_read_only = function () {
        this.assert_table_is_ready("syncstring");
        return this.syncstring_table_get_one().get("read_only");
    };
    SyncDoc.prototype.wait_until_read_only_known = function () {
        return __awaiter(this, void 0, void 0, function () {
            function read_only_defined(t) {
                var x = t.get_one();
                if (x == null) {
                    return false;
                }
                return x.get("read_only") != null;
            }
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.wait_until_ready()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.syncstring_table.wait(read_only_defined, 5 * 60)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /* Returns true if the current live version of this document has
       a different hash than the version mostly recently saved to disk.
       I.e., if there are changes that have not yet been **saved to
       disk**.  See the other function has_uncommitted_changes below
       for determining whether there are changes that haven't been
       commited to the database yet.  Returns *undefined* if
       initialization not even done yet. */
    SyncDoc.prototype.has_unsaved_changes = function () {
        if (this.state !== "ready") {
            return;
        }
        try {
            return this.hash_of_saved_version() !== this.hash_of_live_version();
        }
        catch (err) {
            // This could happen, e.g. when syncstring_table isn't connected
            // in some edge case. Better to just say we don't know then crash
            // everything. See https://github.com/sagemathinc/cocalc/issues/3577
            return;
        }
    };
    // Returns hash of last version saved to disk (as far as we know).
    SyncDoc.prototype.hash_of_saved_version = function () {
        if (this.state !== "ready") {
            return;
        }
        return this.syncstring_table_get_one().getIn(["save", "hash"]);
    };
    /* Return hash of the live version of the document,
       or undefined if the document isn't loaded yet.
       (TODO: write faster version of this for syncdb, which
       avoids converting to a string, which is a waste of time.) */
    SyncDoc.prototype.hash_of_live_version = function () {
        if (this.state !== "ready") {
            return;
        }
        return misc_1.hash_string(this.doc.to_str());
    };
    /* Return true if there are changes to this syncstring that
       have not been committed to the database (with the commit
       acknowledged).  This does not mean the file has been
       written to disk; however, it does mean that it safe for
       the user to close their browser.
    */
    SyncDoc.prototype.has_uncommitted_changes = function () {
        if (this.state !== "ready") {
            return false;
        }
        return this.patches_table.has_uncommitted_changes();
    };
    // Commit any changes to the live document to
    // history as a new patch.  Returns true if there
    // were changes and false otherwise.   This works
    // fine offline, and does not wait until anything
    // is saved to the network, etc.
    SyncDoc.prototype.commit = function () {
        if (this.last == null || this.doc == null || this.last.is_equal(this.doc)) {
            return false;
        }
        this.emit("user-change");
        var patch = this.last.make_patch(this.doc); // must be nontrivial
        this.last = this.doc;
        // ... and save that to patches table
        var time = this.next_patch_time();
        this.commit_patch(time, patch);
        this.save(); // so eventually also gets sent out.
        return true;
    };
    /* Initiates a save of file to disk, then waits for the
       state to change. */
    SyncDoc.prototype.save_to_disk = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, err_10, error;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.state != "ready") {
                            // We just make save_to_disk a successful
                            // no operation, if the document is either
                            // closed or hasn't finished opening, since
                            // there's a lot of code that tries to save
                            // on exit/close or automatically, and it
                            // is difficult to ensure it all checks state
                            // properly.
                            return [2 /*return*/];
                        }
                        dbg = this.dbg("save_to_disk");
                        if (!this.client.is_deleted(this.path, this.project_id)) return [3 /*break*/, 2];
                        dbg("not saving to disk because deleted");
                        return [4 /*yield*/, this.set_save({ state: "done", error: "" })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                    case 2:
                        dbg("initiating the save");
                        if (!!this.has_unsaved_changes()) return [3 /*break*/, 4];
                        dbg("no unsaved changes, so don't save");
                        // CRITICAL: this optimization is assumed by
                        // autosave, etc.
                        return [4 /*yield*/, this.set_save({ state: "done", error: "" })];
                    case 3:
                        // CRITICAL: this optimization is assumed by
                        // autosave, etc.
                        _a.sent();
                        return [2 /*return*/];
                    case 4:
                        if (this.is_read_only()) {
                            dbg("read only, so can't save to disk");
                            // save should fail if file is read only and there are changes
                            throw Error("can't save readonly file with changes to disk");
                        }
                        if (!this.client.is_user()) return [3 /*break*/, 6];
                        dbg("browser client -- sending any changes over network");
                        return [4 /*yield*/, this.save()];
                    case 5:
                        _a.sent();
                        dbg("save done; now do actual save to the *disk*.");
                        this.assert_is_ready("save_to_disk - after save");
                        _a.label = 6;
                    case 6:
                        _a.trys.push([6, 8, , 9]);
                        return [4 /*yield*/, this.save_to_disk_aux()];
                    case 7:
                        _a.sent();
                        return [3 /*break*/, 9];
                    case 8:
                        err_10 = _a.sent();
                        error = "save to disk failed -- " + err_10;
                        dbg(error);
                        if (this.client.is_project()) {
                            this.set_save({ error: error, state: "done" });
                        }
                        return [3 /*break*/, 9];
                    case 9:
                        if (!this.client.is_user()) return [3 /*break*/, 11];
                        dbg("now wait for the save to disk to finish");
                        this.assert_is_ready("save_to_disk - waiting to finish");
                        return [4 /*yield*/, this.wait_for_save_to_disk_done()];
                    case 10:
                        _a.sent();
                        _a.label = 11;
                    case 11:
                        this.update_has_unsaved_changes();
                        return [2 /*return*/];
                }
            });
        });
    };
    /* Export the (currently loaded) history of editing of this
       document to a simple JSON-able object. */
    SyncDoc.prototype.export_history = function (options) {
        if (options === void 0) { options = {}; }
        this.assert_is_ready("export_history");
        var info = this.syncstring_table.get_one();
        if (info == null || !info.has("users")) {
            throw Error("syncstring table must be defined and users initialized");
        }
        var account_ids = info.get("users").toJS();
        misc_1.assertDefined(this.patch_list);
        return export_1.export_history(account_ids, this.patch_list, options);
    };
    SyncDoc.prototype.update_has_unsaved_changes = function () {
        if (this.state != "ready") {
            // This can happen, since this is called by a debounced function.
            // Make it a no-op in case we're not ready.
            // See https://github.com/sagemathinc/cocalc/issues/3577
            return;
        }
        var cur = this.has_unsaved_changes();
        if (cur !== this.last_has_unsaved_changes) {
            this.emit("has-unsaved-changes", cur);
            this.last_has_unsaved_changes = cur;
        }
    };
    // wait for save.state to change state.
    SyncDoc.prototype.wait_for_save_to_disk_done = function () {
        return __awaiter(this, void 0, void 0, function () {
            function until(table) {
                var done = table.get_one().getIn(["save", "state"]) === "done";
                dbg("checking... done=", done);
                return done;
            }
            var dbg, last_err, f;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbg = this.dbg("wait_for_save_to_disk_done");
                        dbg();
                        last_err = undefined;
                        f = function () { return __awaiter(_this, void 0, void 0, function () {
                            var err_11, err;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        dbg("f");
                                        if (this.state != "ready" ||
                                            this.client.is_deleted(this.path, this.project_id)) {
                                            dbg("not ready or deleted - no longer trying to save.");
                                            return [2 /*return*/];
                                        }
                                        _a.label = 1;
                                    case 1:
                                        _a.trys.push([1, 3, , 4]);
                                        dbg("waiting until done...");
                                        return [4 /*yield*/, this.syncstring_table.wait(until, 15)];
                                    case 2:
                                        _a.sent();
                                        return [3 /*break*/, 4];
                                    case 3:
                                        err_11 = _a.sent();
                                        dbg("timed out after 15s");
                                        throw Error("timed out");
                                    case 4:
                                        if (this.state != "ready" ||
                                            this.client.is_deleted(this.path, this.project_id)) {
                                            dbg("not ready or deleted - no longer trying to save.");
                                            return [2 /*return*/];
                                        }
                                        err = this.syncstring_table_get_one().getIn(["save", "error"]);
                                        if (err) {
                                            dbg("error", err);
                                            last_err = err;
                                            throw Error(err);
                                        }
                                        dbg("done, with no error.");
                                        last_err = undefined;
                                        return [2 /*return*/];
                                }
                            });
                        }); };
                        return [4 /*yield*/, async_utils_1.retry_until_success({
                                f: f,
                                max_tries: 8,
                                desc: "wait_for_save_to_disk_done",
                            })];
                    case 1:
                        _a.sent();
                        if (this.state != "ready" ||
                            this.client.is_deleted(this.path, this.project_id)) {
                            return [2 /*return*/];
                        }
                        if (last_err && typeof this.client.log_error === "function") {
                            this.client.log_error({
                                string_id: this.string_id,
                                path: this.path,
                                project_id: this.project_id,
                                error: "Error saving file -- " + last_err,
                            });
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /* Auxiliary function 2 for saving to disk:
       If this is associated with
       a project and has a filename.
       A user (web browsers) sets the save state to requested.
       The project sets the state to saving, does the save
       to disk, then sets the state to done.
    */
    SyncDoc.prototype.save_to_disk_aux = function () {
        return __awaiter(this, void 0, void 0, function () {
            var err_12;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_is_ready("save_to_disk_aux");
                        if (!this.client.is_user()) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.save_to_disk_user()];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, this.save_to_disk_project()];
                    case 3: return [2 /*return*/, _a.sent()];
                    case 4:
                        err_12 = _a.sent();
                        this.emit("save_to_disk_project", err_12);
                        throw err_12;
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.save_to_disk_user = function () {
        return __awaiter(this, void 0, void 0, function () {
            var x, data, expected_hash;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_is_ready("save_to_disk_user");
                        if (!this.has_unsaved_changes()) {
                            /* Browser client has no unsaved changes,
                               so don't need to save --
                               CRITICAL: this optimization is assumed by autosave.
                             */
                            return [2 /*return*/];
                        }
                        x = this.syncstring_table.get_one();
                        if (x != null && x.getIn(["save", "state"]) === "requested") {
                            // Nothing to do -- save already requested, which is
                            // all the browser client has to do.
                            return [2 /*return*/];
                        }
                        data = this.to_str();
                        expected_hash = misc_1.hash_string(data);
                        return [4 /*yield*/, this.set_save({ state: "requested", error: "", expected_hash: expected_hash })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.save_to_disk_project = function () {
        return __awaiter(this, void 0, void 0, function () {
            var data, hash, path, err, stat, err_13;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.assert_is_ready("save_to_disk_project");
                        data = this.to_str();
                        hash = misc_1.hash_string(data);
                        /*
                        // TODO: put this consistency check back in (?).
                        const expected_hash = this.syncstring_table
                          .get_one()
                          .getIn(["save", "expected_hash"]);
                        */
                        if (hash === this.hash_of_saved_version()) {
                            // No actual save to disk needed; still we better
                            // record this fact in table in case it
                            // isn't already recorded
                            this.set_save({ state: "done", error: "", hash: hash });
                            return [2 /*return*/];
                        }
                        path = this.path;
                        if (!path) {
                            err = "cannot save without path";
                            this.set_save({ state: "done", error: err });
                            throw Error(err);
                        }
                        //dbg("project - write to disk file")
                        // set window to slightly earlier to account for clock
                        // imprecision.
                        // Over an sshfs mount, all stats info is **rounded down
                        // to the nearest second**, which this also takes care of.
                        this.save_to_disk_start_ctime = new Date().valueOf() - 1500;
                        this.save_to_disk_end_ctime = undefined;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        return [4 /*yield*/, async_utils_1.callback2(this.client.write_file, { path: path, data: data })];
                    case 2:
                        _a.sent();
                        this.assert_is_ready("save_to_disk_project -- after write_file");
                        return [4 /*yield*/, async_utils_1.callback2(this.client.path_stat, { path: path })];
                    case 3:
                        stat = _a.sent();
                        this.assert_is_ready("save_to_disk_project -- after path_state");
                        this.save_to_disk_end_ctime = stat.ctime.valueOf() + 1500;
                        this.set_save({
                            state: "done",
                            error: "",
                            hash: misc_1.hash_string(data),
                        });
                        return [3 /*break*/, 5];
                    case 4:
                        err_13 = _a.sent();
                        this.set_save({ state: "done", error: JSON.stringify(err_13) });
                        throw err_13;
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    /*
      When the underlying synctable that defines the state
      of the document changes due to new remote patches, this
      function is called.
      It handles update of the remote version, updating our
      live version as a result.
    */
    SyncDoc.prototype.handle_patch_update = function (changed_keys) {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, changed_keys_1, changed_keys_1_1, key;
            var e_7, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (changed_keys == null || changed_keys.length === 0) {
                            // this happens right now when we do a save.
                            return [2 /*return*/];
                        }
                        dbg = this.dbg("handle_patch_update");
                        dbg(changed_keys);
                        if (this.patch_update_queue == null) {
                            this.patch_update_queue = [];
                        }
                        try {
                            for (changed_keys_1 = __values(changed_keys), changed_keys_1_1 = changed_keys_1.next(); !changed_keys_1_1.done; changed_keys_1_1 = changed_keys_1.next()) {
                                key = changed_keys_1_1.value;
                                this.patch_update_queue.push(key);
                            }
                        }
                        catch (e_7_1) { e_7 = { error: e_7_1 }; }
                        finally {
                            try {
                                if (changed_keys_1_1 && !changed_keys_1_1.done && (_a = changed_keys_1.return)) _a.call(changed_keys_1);
                            }
                            finally { if (e_7) throw e_7.error; }
                        }
                        dbg("Clear patch update_queue in a later event loop...");
                        return [4 /*yield*/, awaiting_1.delay(1)];
                    case 1:
                        _b.sent();
                        return [4 /*yield*/, this.handle_patch_update_queue()];
                    case 2:
                        _b.sent();
                        dbg("done");
                        return [2 /*return*/];
                }
            });
        });
    };
    /*
    Whenever new patches are added to this.patches_table,
    their timestamp gets added to this.patch_update_queue.
    */
    SyncDoc.prototype.handle_patch_update_queue = function () {
        return __awaiter(this, void 0, void 0, function () {
            var dbg, v, _a, _b, key, x, t, p;
            var e_8, _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        dbg = this.dbg("handle_patch_update_queue");
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, , 9, 10]);
                        this.handle_patch_update_queue_running = true;
                        _d.label = 2;
                    case 2:
                        if (!(this.state != "closed" && this.patch_update_queue.length > 0)) return [3 /*break*/, 8];
                        dbg("queue size = ", this.patch_update_queue.length);
                        v = [];
                        try {
                            for (_a = (e_8 = void 0, __values(this.patch_update_queue)), _b = _a.next(); !_b.done; _b = _a.next()) {
                                key = _b.value;
                                x = this.patches_table.get(key);
                                if (x != null) {
                                    t = x.get("time");
                                    // Only need to process patches that we didn't
                                    // create ourselves.
                                    if (t && !this.my_patches["" + t.valueOf()]) {
                                        p = this.process_patch(x);
                                        dbg("patch=" + JSON.stringify(p));
                                        if (p != null) {
                                            v.push(p);
                                        }
                                    }
                                }
                            }
                        }
                        catch (e_8_1) { e_8 = { error: e_8_1 }; }
                        finally {
                            try {
                                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                            }
                            finally { if (e_8) throw e_8.error; }
                        }
                        this.patch_update_queue = [];
                        misc_1.assertDefined(this.patch_list);
                        this.patch_list.add(v);
                        // NOTE: The below sync_remote_and_doc can sometimes
                        // *cause* new entries to be added to this.patch_update_queue.
                        dbg("waiting for remote and doc to sync...");
                        return [4 /*yield*/, this.sync_remote_and_doc()];
                    case 3:
                        _d.sent();
                        if (this.state === "closed")
                            return [2 /*return*/]; // closed during await; nothing further to do
                        dbg("remote and doc now synced");
                        if (!(this.patch_update_queue.length > 0)) return [3 /*break*/, 5];
                        // It is very important that next loop happen in a later
                        // event loop to avoid the this.sync_remote_and_doc call
                        // in this.handle_patch_update_queue above from causing
                        // sync_remote_and_doc to get called from within itself,
                        // due to synctable changes being emited on save.
                        dbg("wait for next event loop");
                        return [4 /*yield*/, awaiting_1.delay(1)];
                    case 4:
                        _d.sent();
                        return [3 /*break*/, 7];
                    case 5:
                        dbg("Patch sent, now make a snapshot if we are due for one.");
                        return [4 /*yield*/, this.snapshot_if_necessary()];
                    case 6:
                        _d.sent();
                        _d.label = 7;
                    case 7: return [3 /*break*/, 2];
                    case 8: return [3 /*break*/, 10];
                    case 9:
                        if (this.state == "closed")
                            return [2 /*return*/]; // got closed, so nothing further to do
                        // OK, done and nothing in the queue
                        // Notify save() to try again -- it may have
                        // paused waiting for this to clear.
                        dbg("done");
                        this.handle_patch_update_queue_running = false;
                        this.emit("handle_patch_update_queue_done");
                        return [7 /*endfinally*/];
                    case 10: return [2 /*return*/];
                }
            });
        });
    };
    /* Disable and enable sync.   When disabled we still
       collect patches from upstream (but do not apply them
       locally), and changes we make are broadcast into
       the patch stream.   When we re-enable sync, all
       patches are put together in the stream and
       everything is synced as normal.  This is useful, e.g.,
       to make it so a user **actively** editing a document is
       not interrupted by being forced to sync (in particular,
       by the 'before-change' event that they use to update
       the live document).
  
       Also, delay_sync will delay syncing local with upstream
       for the given number of ms.  Calling it regularly while
       user is actively editing to avoid them being bothered
       by upstream patches getting merged in.
    */
    SyncDoc.prototype.disable_sync = function () {
        this.sync_is_disabled = true;
    };
    SyncDoc.prototype.enable_sync = function () {
        this.sync_is_disabled = false;
        this.sync_remote_and_doc();
    };
    SyncDoc.prototype.delay_sync = function (timeout_ms) {
        var _this = this;
        if (timeout_ms === void 0) { timeout_ms = 2000; }
        clearTimeout(this.delay_sync_timer);
        this.disable_sync();
        this.delay_sync_timer = setTimeout(function () {
            _this.enable_sync();
        }, timeout_ms);
    };
    /*
      Merge remote patches and live version to create new live version,
      which is equal to result of applying all patches.
  
      Only returns once any newly created patches have
      been sent out.
    */
    SyncDoc.prototype.sync_remote_and_doc = function () {
        return __awaiter(this, void 0, void 0, function () {
            var new_remote;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.last == null || this.doc == null || this.sync_is_disabled) {
                            return [2 /*return*/];
                        }
                        if (!(this.state == "ready")) return [3 /*break*/, 4];
                        // First save any unsaved changes from our live version.
                        // Repeat this until changes stop, since there is an await
                        // in this loop, and user may make changes *during* that
                        // save_patch call, and we must not miss them.
                        this.emit("before-change");
                        _a.label = 1;
                    case 1:
                        if (!!this.last.is_equal(this.doc)) return [3 /*break*/, 4];
                        if (!this.commit()) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.patches_table.save()];
                    case 2:
                        _a.sent();
                        if (this.state != "ready") {
                            return [2 /*return*/];
                        }
                        _a.label = 3;
                    case 3: return [3 /*break*/, 1];
                    case 4:
                        // Compute the global current state of the document,
                        // which is got by applying all patches in order.
                        // It is VERY important to do this, even if the
                        // document is not yet ready, since it is critical
                        // to properly set the state of this.doc to the value
                        // of the patch list (e.g., not doing this 100% breaks
                        // opening a file for the first time on cocalc-docker).
                        misc_1.assertDefined(this.patch_list);
                        new_remote = this.patch_list.value();
                        if (!this.doc.is_equal(new_remote)) {
                            // There is a possibility that live document changed, so
                            // set to new version.
                            this.last = this.doc = new_remote;
                            if (this.state == "ready") {
                                this.emit("after-change");
                                this.emit_change();
                            }
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    SyncDoc.prototype.emit_change = function () {
        this.emit("change", this.doc.changes(this.before_change));
        this.before_change = this.doc;
    };
    return SyncDoc;
}(events_1.EventEmitter));
exports.SyncDoc = SyncDoc;
