"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../util");
describe("test making and applying some patches on strings", function () {
    var s0 = "This is CoCalc! Open source software.  And a website.";
    var s1 = "This is SageMath! Open source software.";
    it("creates and applies a patch that applies cleanly", function () {
        var patch = util_1.make_patch(s0, s1);
        expect(patch).toEqual([
            [
                [
                    [0, " is "],
                    [-1, "CoCalc"],
                    [1, "SageMath"],
                    [0, "! Op"],
                ],
                4,
                4,
                14,
                16,
            ],
            [
                [
                    [0, "are."],
                    [-1, "  And a website."],
                ],
                35,
                35,
                20,
                4,
            ],
        ]);
        expect(util_1.apply_patch(patch, s0)).toEqual([s1, true]); // true=clean
    });
    it("creates a patch that does NOT apply cleanly", function () {
        var patch = util_1.make_patch(s0, s1);
        expect(util_1.apply_patch(patch, "This is CoCalc!")).toEqual([
            "This is SageMath!",
            false,
        ]);
    });
});
describe("test doing a 3-way merge", function () {
    var base = "SageMathCloud is software for using Sage in the cloud.";
    // we change base in two different ways: first name
    var local = "CoCalc is software for using Sage in the cloud.";
    // .... then (independently) the purpose.
    var remote = "SageMathCloud is software for collaborative calculation in the cloud.";
    it("does a three-way merge", function () {
        var merge = util_1.three_way_merge({ base: base, local: local, remote: remote });
        // Merging captures both changes.
        expect(merge).toBe("CoCalc is software for collaborative calculation in the cloud.");
    });
});
describe("Test comparison of patch log entries (compares time and user)", function () {
    var p0 = {
        time: new Date("2019-01-01T22:15:31.539Z"),
        patch: [],
        user_id: 0,
        size: 2,
    };
    var p1 = {
        time: new Date("2019-01-01T22:15:40Z"),
        patch: [],
        user_id: 1,
        size: 2,
    };
    var p2 = {
        time: new Date("2019-01-01T22:15:31.539Z"),
        patch: [],
        user_id: 1,
        size: 2,
    };
    it("compares some patch log entries", function () {
        expect(util_1.patch_cmp(p0, p0)).toBe(0);
        expect(util_1.patch_cmp(p1, p1)).toBe(0);
        expect(util_1.patch_cmp(p0, p1)).toBe(-1);
        expect(util_1.patch_cmp(p1, p0)).toBe(1);
        expect(util_1.patch_cmp(p0, p2)).toBe(-1);
        expect(util_1.patch_cmp(p2, p0)).toBe(1);
    });
});
describe("Test comparing times", function () {
    var t0 = new Date("2019-01-01T22:15:31.539Z");
    var t1 = new Date("2019-01-01T22:15:40Z");
    var t2 = new Date("2019-01-01T22:15:31.539Z");
    it("compares some times", function () {
        expect(util_1.time_cmp(t0, t1)).toBe(-1);
        expect(util_1.time_cmp(t1, t0)).toBe(1);
        expect(util_1.time_cmp(t0, t0)).toBe(0);
        expect(util_1.time_cmp(t0, t2)).toBe(0);
        expect(util_1.time_cmp(t2, t0)).toBe(0);
    });
});
