"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.three_way_merge = exports.time_cmp = exports.patch_cmp = exports.apply_patch = exports.make_patch = exports.decompress_patch = exports.compress_patch = exports.dmp = void 0;
var diff_match_patch = require("smc-util/dmp").diff_match_patch;
exports.dmp = new diff_match_patch();
exports.dmp.Diff_Timeout = 0.2; // computing a diff won't block longer than about 0.2s
// Here's what a diff-match-patch patch looks like
//
// [{"diffs":[[1,"{\"x\":5,\"y\":3}"]],"start1":0,"start2":0,"length1":0,"length2":13},...]
//
// The diff-match-patch library changed the format, but we must keep it the same
// for backward compat and two stay JSON friendly.
var Diff = diff_match_patch.Diff;
function diffs_to_arrays(diffs) {
    var e_1, _a;
    var v = [];
    try {
        for (var diffs_1 = __values(diffs), diffs_1_1 = diffs_1.next(); !diffs_1_1.done; diffs_1_1 = diffs_1.next()) {
            var d = diffs_1_1.value;
            v.push([d[0], d[1]]);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (diffs_1_1 && !diffs_1_1.done && (_a = diffs_1.return)) _a.call(diffs_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return v;
}
function arrays_to_diffs(arrays) {
    var e_2, _a;
    var v = [];
    try {
        for (var arrays_1 = __values(arrays), arrays_1_1 = arrays_1.next(); !arrays_1_1.done; arrays_1_1 = arrays_1.next()) {
            var x = arrays_1_1.value;
            v.push(new Diff(x[0], x[1]));
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (arrays_1_1 && !arrays_1_1.done && (_a = arrays_1.return)) _a.call(arrays_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return v;
}
function compress_patch(patch) {
    return patch.map(function (p) { return [
        diffs_to_arrays(p.diffs),
        p.start1,
        p.start2,
        p.length1,
        p.length2,
    ]; });
}
exports.compress_patch = compress_patch;
function decompress_patch(patch) {
    return patch.map(function (p) { return ({
        diffs: arrays_to_diffs(p[0]),
        start1: p[1],
        start2: p[2],
        length1: p[3],
        length2: p[4],
    }); });
}
exports.decompress_patch = decompress_patch;
// return *a* compressed patch that transforms string s0 into string s1.
function make_patch(s0, s1) {
    return compress_patch(exports.dmp.patch_make(s0, s1));
}
exports.make_patch = make_patch;
// apply a compressed patch to a string.
function apply_patch(patch, s) {
    var e_3, _a;
    var x;
    try {
        x = exports.dmp.patch_apply(decompress_patch(patch), s);
        //console.log('patch_apply ', misc.to_json(decompress_patch(patch)), x)
    }
    catch (err) {
        // If a patch is so corrupted it can't be parsed -- e.g., due to a bug in SMC -- we at least
        // want to make application the identity map (i.e., "best effort"), so
        // the document isn't completely unreadable!
        console.warn("apply_patch -- " + err + ", " + JSON.stringify(patch));
        return [s, false];
    }
    var clean = true;
    try {
        for (var _b = __values(x[1]), _c = _b.next(); !_c.done; _c = _b.next()) {
            var a = _c.value;
            if (!a) {
                clean = false;
                break;
            }
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_3) throw e_3.error; }
    }
    return [x[0], clean];
}
exports.apply_patch = apply_patch;
var cmp_array = require("smc-util/misc").cmp_array;
function patch_cmp(a, b) {
    return cmp_array([a.time.valueOf(), a.user_id], [b.time.valueOf(), b.user_id]);
}
exports.patch_cmp = patch_cmp;
function time_cmp(a, b) {
    var t = a.valueOf() - b.valueOf();
    if (t < 0) {
        return -1;
    }
    else if (t > 0) {
        return 1;
    }
    else {
        return 0;
    }
}
exports.time_cmp = time_cmp;
// Do a 3-way **string** merge by computing patch that transforms
// base to remote, then applying that patch to local.
function three_way_merge(opts) {
    if (opts.base === opts.remote) {
        // trivial special case...
        return opts.local;
    }
    return exports.dmp.patch_apply(exports.dmp.patch_make(opts.base, opts.remote), opts.local)[0];
}
exports.three_way_merge = three_way_merge;
