"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringDocument = void 0;
var util_1 = require("../generic/util");
// Immutable string document that satisfies our spec.
var StringDocument = /** @class */ (function () {
    function StringDocument(value) {
        if (value === void 0) { value = ""; }
        this.value = value;
    }
    StringDocument.prototype.to_str = function () {
        return this.value;
    };
    StringDocument.prototype.is_equal = function (other) {
        return this.value === (other != null ? other.value : undefined);
    };
    StringDocument.prototype.apply_patch = function (patch) {
        return new StringDocument(util_1.apply_patch(patch, this.value)[0]);
    };
    StringDocument.prototype.make_patch = function (other) {
        return util_1.make_patch(this.value, other.value);
    };
    StringDocument.prototype.set = function (x) {
        if (typeof x === "string") {
            return new StringDocument(x);
        }
        throw Error("x must be a string");
    };
    StringDocument.prototype.get = function (_) {
        throw Error("get queries on strings don't have meaning");
    };
    StringDocument.prototype.get_one = function (_) {
        throw Error("get_one queries on strings don't have meaning");
    };
    StringDocument.prototype.delete = function (_) {
        throw Error("delete on strings doesn't have meaning");
    };
    StringDocument.prototype.changes = function (_) {
        // no-op (this is useful for other things, e.g., db-doc)
        return;
    };
    StringDocument.prototype.count = function () {
        return this.value.length;
    };
    return StringDocument;
}());
exports.StringDocument = StringDocument;
