"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
var doc_1 = require("../doc");
describe("create a String document and call methods on it", function () {
    var doc;
    it("creates the doc", function () {
        doc = new doc_1.StringDocument("cocalc");
        expect("" + doc).toBe("[object Object]");
    });
    it("convert to string", function () {
        expect(doc.to_str()).toBe("cocalc");
    });
    it("checks equality", function () {
        expect(doc.is_equal()).toBe(false);
        expect(doc.is_equal(undefined)).toBe(false);
        expect(doc.is_equal(doc)).toBe(true);
        expect(doc.is_equal(new doc_1.StringDocument("cocalc"))).toBe(true);
        expect(doc.is_equal(new doc_1.StringDocument("sagemathcloud"))).toBe(false);
    });
    it("make and apply a patch", function () {
        var patch = doc.make_patch(new doc_1.StringDocument("CoCalc"));
        expect(doc.apply_patch(patch).to_str()).toBe("CoCalc");
    });
    it("set the document (thus making a new one, since immutable!)", function () {
        var d2 = doc.set("CoCalc");
        expect(doc.to_str()).toBe("cocalc");
        expect(d2.to_str()).toBe("CoCalc");
        // also test type checking
        expect(function () {
            doc.set(0);
        }).toThrow("must be a string");
    });
    it("tests that the db-like API all throws errors", function () {
        expect(function () { return doc.get(); }).toThrow("don't have meaning");
        expect(function () { return doc.get_one(); }).toThrow("don't have meaning");
        expect(function () { return doc.delete(); }).toThrow("doesn't have meaning");
        expect(doc.changes(doc)).toBe(undefined);
    });
    it("count gives length", function () {
        expect(doc.count()).toBe(doc.to_str().length);
    });
});
