"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var client_test_1 = require("./client-test");
var sync_1 = require("../sync");
var async_utils_1 = require("../../../../async-utils");
var data_1 = require("./data");
// This mostly tests the trivial minimal edge cases.
describe("create a blank minimal string SyncDoc and call public methods on it", function () {
    var _a = data_1.a_txt(), client_id = _a.client_id, project_id = _a.project_id, path = _a.path, string_id = _a.string_id, init_queries = _a.init_queries;
    var client = new client_test_1.Client(init_queries, client_id);
    var syncstring;
    it("creates the syncstring and wait for it to be ready", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    syncstring = new sync_1.SyncString({ project_id: project_id, path: path, client: client });
                    expect(syncstring.get_state()).toBe("init");
                    return [4 /*yield*/, async_utils_1.once(syncstring, "ready")];
                case 1:
                    _a.sent();
                    expect(syncstring.get_state()).toBe("ready");
                    return [2 /*return*/];
            }
        });
    }); });
    it("call set_cursor_locs, an error since cursors aren't enabled", function () {
        expect(function () { return syncstring.set_cursor_locs([]); }).toThrow("cursors are not enabled");
    });
    it("calls each public get method", function () {
        expect(syncstring.get_state()).toBe("ready");
        expect(syncstring.get_project_id()).toBe(project_id);
        expect(syncstring.get_path()).toBe(path);
        expect(syncstring.get_string_id()).toBe(string_id);
        expect(syncstring.get_my_user_id()).toBe(1);
    });
    it("the db-style get methods all fail on a string", function () {
        expect(function () { return syncstring.get(); }).toThrow("queries on strings don't have meaning");
        expect(function () { return syncstring.get_one(); }).toThrow("queries on strings don't have meaning");
        expect(function () { return syncstring.delete(); }).toThrow("delete on strings doesn't have meaning");
    });
    it("get the underlying doc", function () {
        // via Document
        expect(syncstring.get_doc().to_str()).toBe("");
        // directly
        expect(syncstring.to_str()).toBe("");
    });
    it("get the size via count", function () {
        expect(syncstring.count()).toBe(0);
    });
    it("get current version", function () {
        expect(syncstring.version().to_str()).toBe("");
    });
    it("get version without (removing nothing though)", function () {
        expect(syncstring.version_without([]).to_str()).toBe("");
        expect(syncstring.version_without([new Date()]).to_str()).toBe("");
    });
    it("revert to version now (does nothing - no error)", function () {
        syncstring.revert(new Date());
    });
    it("undo/redo -- nothing to undo yet...", function () {
        expect(syncstring.in_undo_mode()).toBe(false);
        syncstring.undo();
        expect(syncstring.in_undo_mode()).toBe(true);
        syncstring.exit_undo_mode();
        expect(syncstring.in_undo_mode()).toBe(false);
        syncstring.redo(); // no error
    });
    it("account_id of change at given point in time gives error", function () {
        expect(function () { return syncstring.account_id(new Date()); }).toThrow("no patch at");
    });
    it("time sent of change at given point in time gives error", function () {
        expect(function () { return syncstring.time_sent(new Date()); }).toThrow("no patch at");
    });
    it("user_id of change at given point in time gives error", function () {
        expect(function () { return syncstring.user_id(new Date()); }).toThrow("no patch at");
    });
    it("get list of versions (should be empty)", function () {
        expect(syncstring.versions()).toEqual([]);
    });
    it("get all known versions (also empty)", function () {
        expect(syncstring.all_versions()).toEqual([]);
    });
    it("last changed when time began", function () {
        expect(syncstring.last_changed()).toEqual(new Date(0));
    });
    it("check ready state", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    syncstring.assert_is_ready("check ready state");
                    return [4 /*yield*/, syncstring.wait_until_ready()];
                case 1:
                    _a.sent(); // trivial since already ready
                    return [2 /*return*/];
            }
        });
    }); });
    it("wait for an already true condition", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.wait(function () { return true; })];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it("get cursors (error, since cursors not enabled)", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            expect(function () { return syncstring.get_cursors(); }).toThrow("cursors are not enabled");
            return [2 /*return*/];
        });
    }); });
    it("set, then get, something from the settings field", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.set_settings({ foo: { bar: "none" } })];
                case 1:
                    _a.sent();
                    expect(syncstring.get_settings().get("foo").toJS()).toEqual({
                        bar: "none",
                    });
                    return [2 /*return*/];
            }
        });
    }); });
    it("verifies it has the full history already", function () {
        expect(syncstring.has_full_history()).toBe(true);
    });
    it("loads full history (which does basically nothing)", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.load_full_history()];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it("do a save (no-op, since haven't done anything yet)", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.save()];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it("change the snapshot interval", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.set_snapshot_interval(17)];
                case 1:
                    _a.sent();
                    expect(syncstring.snapshot_interval).toBe(17);
                    return [2 /*return*/];
            }
        });
    }); });
    it("read only checks", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.wait_until_read_only_known()];
                case 1:
                    _a.sent(); // no-op
                    expect(syncstring.is_read_only()).toBe(false);
                    return [2 /*return*/];
            }
        });
    }); });
    it("hashes of versions", function () {
        expect(syncstring.hash_of_saved_version()).toBe(0);
        expect(syncstring.hash_of_live_version()).toBe(0);
        expect(syncstring.has_uncommitted_changes()).toBe(false);
    });
    it("saves to disk (no-op, since nothing changed)", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.save_to_disk()];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it("close and clean up", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    expect(syncstring.get_state()).toBe("ready");
                    return [4 /*yield*/, syncstring.close()];
                case 1:
                    _a.sent();
                    expect(syncstring.get_state()).toBe("closed");
                    return [2 /*return*/];
            }
        });
    }); });
});
