"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var client_test_1 = require("./client-test");
var sync_1 = require("../sync");
var async_utils_1 = require("../../../../async-utils");
var data_1 = require("./data");
describe("create syncstring and test doing some edits", function () {
    var _a = data_1.a_txt(), client_id = _a.client_id, project_id = _a.project_id, path = _a.path, init_queries = _a.init_queries;
    var client = new client_test_1.Client(init_queries, client_id);
    var syncstring;
    var v = [
        "SageMathCloud",
        "SageMathCloud -- Collaborative Calculation",
        "CoCalc -- Collaborative Calculation",
    ];
    it("creates the syncstring and wait until ready", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    syncstring = new sync_1.SyncString({ project_id: project_id, path: path, client: client, cursors: true });
                    expect(syncstring.get_state()).toBe("init");
                    return [4 /*yield*/, async_utils_1.once(syncstring, "ready")];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it("sets the value and commit change", function () { return __awaiter(void 0, void 0, void 0, function () {
        var i;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    i = 0;
                    _a.label = 1;
                case 1:
                    if (!(i < v.length)) return [3 /*break*/, 4];
                    syncstring.from_str(v[i]);
                    expect(syncstring.to_str()).toBe(v[i]);
                    expect(syncstring.count()).toBe(v[i].length);
                    expect(syncstring.versions().length).toBe(i);
                    return [4 /*yield*/, syncstring.save()];
                case 2:
                    _a.sent();
                    expect(syncstring.versions().length).toBe(i + 1);
                    _a.label = 3;
                case 3:
                    i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    it("gets the value at each past version", function () {
        var vers = syncstring.versions();
        for (var i = 0; i < v.length; i++) {
            expect(syncstring.version(vers[i]).to_str()).toEqual(v[i]);
        }
    });
    it("undo it all", function () {
        for (var i = 0; i < v.length; i++) {
            syncstring.undo();
            var e = v[v.length - i - 2];
            if (e === undefined) {
                e = "";
            }
            expect(syncstring.to_str()).toEqual(e);
            expect(syncstring.in_undo_mode()).toBe(true);
        }
        // we made lots of changes with undo/redo, and these
        // automatically result in new commits.
        expect(syncstring.versions().length).toBe(2 * v.length);
    });
    it("redo it all (undo mode state continues from above)", function () {
        expect(syncstring.in_undo_mode()).toBe(true);
        for (var i = 0; i < v.length; i++) {
            syncstring.redo();
            expect(syncstring.to_str()).toEqual(v[i]);
        }
        syncstring.exit_undo_mode();
        expect(syncstring.in_undo_mode()).toBe(false);
        expect(syncstring.versions().length).toBe(3 * v.length);
    });
    it("revert to each past point in time", function () {
        var vers = syncstring.versions();
        for (var i = 0; i < v.length; i++) {
            syncstring.revert(vers[i]);
            expect(syncstring.to_str()).toEqual(v[i]);
        }
        // correct, since no commits.
        expect(syncstring.versions().length).toBe(3 * v.length);
    });
    it("gets info about patch at a given point in time", function () {
        var t = syncstring.versions()[1];
        expect(syncstring.account_id(t)).toEqual(client_id);
        // time sent is not set since patch wasn't made offline.
        expect(syncstring.time_sent(t)).toBe(undefined);
        expect(syncstring.user_id(t)).toEqual(1);
    });
    it("last_changed is the time of the last version", function () {
        var vers = syncstring.versions();
        expect(syncstring.last_changed()).toEqual(vers[vers.length - 1]);
    });
    it("is not read only", function () {
        expect(syncstring.is_read_only()).toBe(false);
    });
    it("save to disk", function () { return __awaiter(void 0, void 0, void 0, function () {
        var promise;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    expect(syncstring.has_unsaved_changes()).toBe(true);
                    promise = syncstring.save_to_disk();
                    // Mock: we set save to done in the syncstring
                    // table, otherwise the promise will never resolve.
                    syncstring.set_save({
                        state: "done",
                        error: "",
                        hash: syncstring.hash_of_live_version(),
                    });
                    syncstring.syncstring_table.emit("change-no-throttle");
                    return [4 /*yield*/, promise];
                case 1:
                    _a.sent();
                    expect(syncstring.has_unsaved_changes()).toBe(false);
                    return [2 /*return*/];
            }
        });
    }); });
    it("close and clean up", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, syncstring.close()];
                case 1:
                    _a.sent();
                    expect(syncstring.get_state()).toBe("closed");
                    return [2 /*return*/];
            }
        });
    }); });
});
