"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.query_function = void 0;
/*
Determine function that does query.
*/
var DISABLE_STANDBY = true; // if true, never use standby server at all.
var async = require("async");
var awaiting_1 = require("awaiting");
var schema_1 = require("../../schema");
var misc_1 = require("../../misc");
function query_function(client_query, table) {
    var s = schema_1.SCHEMA[table];
    if (s == null) {
        throw Error("unknown table " + table);
    }
    var db_standby = s.db_standby;
    if (DISABLE_STANDBY || !db_standby) {
        // just use default client.query, which queries the master database.
        return client_query;
    }
    function do_query(opts) {
        if (opts == null) {
            throw Error("opts must be an object");
        }
        var read_done = false;
        var change_queue = [];
        function do_initial_read_query(cb) {
            var opts2 = misc_1.copy(opts);
            opts2.standby = true;
            opts2.changes = false;
            var cb_called = false;
            opts2.cb = function (err, resp) {
                return __awaiter(this, void 0, void 0, function () {
                    var x, err_1, change;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                opts.cb(err, resp);
                                if (!!err) return [3 /*break*/, 2];
                                read_done = true;
                                if (!(change_queue.length > 0)) return [3 /*break*/, 2];
                                // CRITICAL: delay, since these must be pushed out in a later event loop.
                                // Without this delay, there will be many random failures.
                                return [4 /*yield*/, awaiting_1.delay(0)];
                            case 1:
                                // CRITICAL: delay, since these must be pushed out in a later event loop.
                                // Without this delay, there will be many random failures.
                                _a.sent();
                                while (change_queue.length > 0) {
                                    x = change_queue.shift();
                                    if (x == null)
                                        break; // make typescript happy.
                                    err_1 = x.err, change = x.change;
                                    opts.cb(err_1, change);
                                }
                                _a.label = 2;
                            case 2:
                                if (!cb_called) {
                                    cb_called = true;
                                    cb(err);
                                }
                                return [2 /*return*/];
                        }
                    });
                });
            };
            client_query(opts2);
        }
        function start_changefeed(cb) {
            var first_resp = true;
            var opts2 = misc_1.copy(opts);
            opts2.standby = false;
            opts2.changes = true;
            opts2.cb = function (err, change) {
                if (read_done) {
                    opts.cb(err, change);
                }
                else {
                    change_queue.push({ err: err, change: change });
                }
                if (first_resp) {
                    first_resp = false;
                    cb(err);
                }
            };
            opts2.options = opts2.options.concat({ only_changes: true });
            client_query(opts2);
        }
        var f;
        if (db_standby === "unsafe") {
            /* If db_standby == 'unsafe', then we do not even require
               the changefeed to be working before doing the full query.
               This will for sure miss all changes from when the query
               finishes until the changefeed starts.  For some
               tables this is fine; for others, not. */
            f = async.parallel;
        }
        else {
            // Otherwise, the query could miss a small amount of data,
            // but only for a tiny window of time.
            f = async.series;
        }
        f([do_initial_read_query, start_changefeed]);
    }
    return do_query;
}
exports.query_function = query_function;
