"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.synctable_no_changefeed = void 0;
/*
Make a SyncTable which does not use a changefeed at all.

It does the initial database read as usual, and also
writes changes as usual, but does not use a changefeed
at all.   Instead changes are injected by calling
a function.

This is used, e.g., by a backend project for implementing a version
of SyncTable, where the project itself handles all changes,
not the database or hubs.   However, data is still persisted
to the central database.

Returned object is not cached in any way.
*/
var events_1 = require("events");
var synctable_1 = require("./synctable");
var misc_1 = require("../../misc");
function synctable_no_changefeed(query, options, client, throttle_changes) {
    if (options == null) {
        options = [];
    }
    var client2 = new ClientNoChangefeed(client);
    return new synctable_1.SyncTable(query, options, client2, throttle_changes, true, false);
}
exports.synctable_no_changefeed = synctable_no_changefeed;
var ClientNoChangefeed = /** @class */ (function (_super) {
    __extends(ClientNoChangefeed, _super);
    function ClientNoChangefeed(client) {
        var _this = _super.call(this) || this;
        misc_1.bind_methods(_this, [
            "query",
            "dbg",
            "query_cancel",
            "emit_connected",
            "emit_signed_in",
        ]);
        _this.client = client;
        // These MUST be after the binds above, obviously.
        client.on("connected", _this.emit_connected);
        client.on("signed_in", _this.emit_signed_in);
        return _this;
    }
    ClientNoChangefeed.prototype.emit_connected = function () {
        this.emit("connected");
    };
    ClientNoChangefeed.prototype.emit_signed_in = function () {
        this.emit("signed_in");
    };
    ClientNoChangefeed.prototype.is_project = function () {
        return this.client.is_project();
    };
    ClientNoChangefeed.prototype.touch_project = function (project_id) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.client.touch_project(project_id)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    ClientNoChangefeed.prototype.is_connected = function () {
        return this.client.is_connected();
    };
    ClientNoChangefeed.prototype.is_signed_in = function () {
        return this.client.is_signed_in();
    };
    ClientNoChangefeed.prototype.server_time = function () {
        return this.client.server_time();
    };
    ClientNoChangefeed.prototype.dbg = function (s) {
        return this.client.dbg(s);
    };
    ClientNoChangefeed.prototype.query = function (opts) {
        if (opts.changes) {
            this.changefeed_query(opts);
        }
        else {
            this.client.query(opts);
        }
    };
    ClientNoChangefeed.prototype.changefeed_query = function (opts) {
        opts.changes = false;
        this.client.query(opts);
    };
    ClientNoChangefeed.prototype.query_cancel = function (_) {
        // no op since no changefeed.
        this.client.removeListener("connected", this.emit_connected);
        this.client.removeListener("signed_in", this.emit_signed_in);
    };
    ClientNoChangefeed.prototype.alert_message = function (opts) {
        if (this.client.alert_message != null) {
            this.client.alert_message(opts);
        }
    };
    return ClientNoChangefeed;
}(events_1.EventEmitter));
