"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.synctable_no_database = void 0;
/*
Make a SyncTable which does not use a changefeed or the central
database at all.

The initial read waits on the client calling a function to provide
the initial data, and all changes are also injected by explicitly
calling a function.  An event is emitted when a new change is made
that has to get saved.

This is used to implement the browser side of project specific
SyncTables.  It's also obviously useful for unit testing.
*/
var events_1 = require("events");
var synctable_1 = require("./synctable");
var misc_1 = require("../../misc");
function synctable_no_database(query, options, client, throttle_changes, initial_get_query, project_id) {
    if (throttle_changes === void 0) { throttle_changes = undefined; }
    if (initial_get_query === void 0) { initial_get_query = []; }
    if (options == null) {
        options = [];
    }
    var client2 = new ClientNoDatabase(client, initial_get_query);
    return new synctable_1.SyncTable(query, options, client2, throttle_changes, true, true, project_id);
}
exports.synctable_no_database = synctable_no_database;
var ClientNoDatabase = /** @class */ (function (_super) {
    __extends(ClientNoDatabase, _super);
    function ClientNoDatabase(client, initial_get_query) {
        var _this = _super.call(this) || this;
        _this.connected = true;
        _this.initial_get_query = initial_get_query;
        misc_1.bind_methods(_this, ["query", "dbg", "query_cancel"]);
        _this.client = client;
        return _this;
    }
    ClientNoDatabase.prototype.set_connected = function (connected) {
        var event = connected && this.connected != connected;
        this.connected = connected;
        if (event) {
            this.emit("signed_in");
            this.emit("connected");
        }
    };
    ClientNoDatabase.prototype.is_project = function () {
        return this.client.is_project();
    };
    ClientNoDatabase.prototype.touch_project = function (project_id) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.client.touch_project(project_id)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    ClientNoDatabase.prototype.is_connected = function () {
        return this.connected;
    };
    ClientNoDatabase.prototype.is_signed_in = function () {
        return this.connected;
    };
    ClientNoDatabase.prototype.server_time = function () {
        return this.client.server_time();
    };
    ClientNoDatabase.prototype.dbg = function (s) {
        return this.client.dbg(s);
    };
    ClientNoDatabase.prototype.query = function (opts) {
        var _a;
        if (opts.options && opts.options.length === 1 && opts.options[0].set) {
            if (this.connected) {
                // set query -- totally ignore.
                opts.cb();
            }
            else {
                opts.cb("disconnected");
            }
        }
        else {
            // get query -- returns predetermined result (default: empty)
            var table = misc_1.keys(opts.query)[0];
            opts.cb(undefined, { query: (_a = {}, _a[table] = this.initial_get_query, _a) });
        }
    };
    ClientNoDatabase.prototype.query_cancel = function (_) { };
    ClientNoDatabase.prototype.alert_message = function (opts) {
        if (this.client.alert_message != null) {
            this.client.alert_message(opts);
        }
    };
    return ClientNoDatabase;
}(events_1.EventEmitter));
