"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var changefeed_1 = require("../changefeed");
describe("first test of the public API of the Changefeed class", function () {
    var options = [];
    var query = {
        system_notifications: { id: null, time: null, text: null, priority: null },
    };
    var table = "system_notifications";
    var got = {};
    var init_val = [
        { id: "0", time: new Date(0), text: "foo", priority: "low" },
    ];
    function do_query(opts) {
        var _a;
        got.do_query = opts;
        opts.cb(undefined, { query: (_a = {}, _a[table] = init_val, _a) });
    }
    function query_cancel(opts) {
        got.query_cancel = opts;
        opts.cb();
    }
    var changefeed;
    var opts = {
        do_query: do_query,
        query_cancel: query_cancel,
        options: options,
        query: query,
        table: table,
    };
    it("creates the changefeed", function () {
        changefeed = new changefeed_1.Changefeed(opts);
        expect(changefeed.get_state()).toBe("disconnected");
    });
    it("initializes the changefeed", function () { return __awaiter(void 0, void 0, void 0, function () {
        var init;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, changefeed.connect()];
                case 1:
                    init = _a.sent();
                    expect(changefeed.get_state()).toBe("connected");
                    expect(init).toBe(init_val);
                    return [2 /*return*/];
            }
        });
    }); });
    it("causes an update", function (done) {
        changefeed.on("update", function (x) {
            expect(x).toEqual({ action: "insert", new_val: { text: "bar" } });
            done();
        });
        got.do_query.cb(undefined, { action: "insert", new_val: { text: "bar" } });
    });
    it("ends the changefeed via query_cancel event", function (done) {
        changefeed.on("close", function () {
            expect(changefeed.get_state()).toBe("closed");
            done();
        });
        got.do_query.cb(undefined, { event: "query_cancel" });
    });
    it("creates changefeed again", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    changefeed = new changefeed_1.Changefeed(opts);
                    return [4 /*yield*/, changefeed.connect()];
                case 1:
                    _a.sent();
                    expect(changefeed.get_state()).toBe("connected");
                    return [2 /*return*/];
            }
        });
    }); });
    it("ends the changefeed via an error", function (done) {
        changefeed.on("close", function () {
            expect(changefeed.get_state()).toBe("closed");
            done();
        });
        got.do_query.cb("fatal");
    });
    it("creates changefeed again", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    changefeed = new changefeed_1.Changefeed(opts);
                    return [4 /*yield*/, changefeed.connect()];
                case 1:
                    _a.sent();
                    expect(changefeed.get_state()).toBe("connected");
                    return [2 /*return*/];
            }
        });
    }); });
    it("ends the changefeed via an error", function (done) {
        changefeed.on("close", function () {
            expect(changefeed.get_state()).toBe("closed");
            done();
        });
        got.do_query.cb("fatal");
    });
    it("creates changefeed again", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    changefeed = new changefeed_1.Changefeed(opts);
                    return [4 /*yield*/, changefeed.connect()];
                case 1:
                    _a.sent();
                    expect(changefeed.get_state()).toBe("connected");
                    return [2 /*return*/];
            }
        });
    }); });
    it("ends the changefeed by explicitly calling close", function (done) {
        changefeed.on("close", function () {
            expect(changefeed.get_state()).toBe("closed");
            done();
        });
        changefeed.close();
    });
});
