"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var synctable_1 = require("../synctable");
var async_utils_1 = require("../../../async-utils");
var client_test_1 = require("./client-test");
describe("tests public API of a system_notifications SyncTable", function () {
    var synctable;
    var notifications = [
        {
            id: "123e4567-e89b-12d3-a456-426655440000",
            time: new Date(),
            text: "This is a message.",
            priority: "low",
        },
        {
            id: "123e4567-e89b-12d3-a456-426655440001",
            time: new Date(),
            text: "This is a second message.",
            priority: "high",
        },
    ];
    var client = new client_test_1.ClientTest(notifications);
    var query = {
        system_notifications: [
            { id: null, time: null, text: null, priority: null },
        ],
    };
    test("create the synctable", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    // last 0 is to disable change throttling, which messes up jest.
                    synctable = new synctable_1.SyncTable(query, [], client, 0);
                    expect(synctable.get_state()).toBe("disconnected");
                    return [4 /*yield*/, async_utils_1.once(synctable, "connected")];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    test("get query the synctable", function () {
        var _a;
        var x = synctable.get();
        if (x == null) {
            throw Error("must be defined since synctable is connected");
        }
        expect(x.toJS()).toEqual((_a = {},
            _a[notifications[0].id] = notifications[0],
            _a[notifications[1].id] = notifications[1],
            _a));
    });
    test("get_one query the synctable", function () {
        var x = synctable.get_one();
        if (x == null) {
            throw Error("must be defined since synctable is connected");
        }
        expect(x.toJS()).toEqual(notifications[0]);
    });
    test("get_one query for one primary key", function () {
        var x = synctable.get_one(notifications[0].id);
        if (x == null) {
            throw Error("must be defined since synctable is connected");
        }
        expect(x.toJS()).toEqual(notifications[0]);
        expect(x).toBe(synctable.get(notifications[0].id));
    });
    test("get_one query for other primary key", function () {
        var x = synctable.get_one(notifications[1].id);
        if (x == null) {
            throw Error("must be defined since synctable is connected");
        }
        expect(x.toJS()).toEqual(notifications[1]);
        // also the get is the same when there is an arg.
        expect(x).toBe(synctable.get(notifications[1].id));
    });
    test("get_one query for other primary key", function () {
        var x = synctable.get_one("foo");
        expect(x).toBe(undefined);
        // also the get is the same when there is an arg.
        expect(x).toBe(synctable.get("foo"));
    });
    test("does not have uncommitted changes", function () {
        expect(synctable.has_uncommitted_changes()).toBe(false);
    });
    test("making change to invalid field raises error", function () {
        expect(function () { return synctable.set({ foobar: "medium" }); }).toThrow("Cannot coerce: no field 'foobar' in table system_notifications");
    });
    test("making change to field not in query (even though it is valid) raises error", function () {
        expect(function () { return synctable.set({ done: true }); }).toThrow("coerce");
    });
    test("make change; then has uncommitted changes", function () {
        expect(client.set_queries.length).toBe(0);
        synctable.set({ id: notifications[1].id, priority: "medium" });
        // Set does not cause a database write (via save).
        expect(client.set_queries.length).toBe(0);
        expect(synctable.has_uncommitted_changes()).toBe(true);
    });
    test("save change; then does not have uncommitted changes", function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, synctable.save()];
                case 1:
                    _a.sent();
                    // Set causes a database write:
                    expect(client.set_queries.length).toBe(1);
                    expect(synctable.has_uncommitted_changes()).toBe(false);
                    return [2 /*return*/];
            }
        });
    }); });
    test("waiting for a condition to be satisfied", function () { return __awaiter(void 0, void 0, void 0, function () {
        function satisfy_condition() {
            synctable.set({ id: notifications[1].id, priority: "high" });
            synctable.save();
        }
        function until(s) {
            var priority = s.get(notifications[1].id).get("priority");
            return priority === "high";
        }
        var p;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    p = synctable.wait(until);
                    satisfy_condition();
                    return [4 /*yield*/, p];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    test("a change event", function (done) { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    synctable.once("change", function (keys) {
                        expect(keys).toEqual(["123e4567-e89b-12d3-a456-426655440001"]);
                        done();
                    });
                    synctable.set({ id: notifications[1].id, priority: "medium" });
                    return [4 /*yield*/, synctable.save()];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    test("closing the synctable", function () { return __awaiter(void 0, void 0, void 0, function () {
        var n;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    n = client.set_queries.length;
                    expect(synctable.get_state()).toBe("connected");
                    synctable.set({ id: notifications[1].id, priority: "low" });
                    expect(synctable.has_uncommitted_changes()).toBe(true);
                    synctable.close();
                    return [4 /*yield*/, async_utils_1.once(synctable, "closed")];
                case 1:
                    _a.sent();
                    expect(client.set_queries.length).toBe(n + 1); // final save happened
                    return [2 /*return*/];
            }
        });
    }); });
    test("closed synctable -- has the right state", function () {
        expect(synctable.get_state()).toBe("closed");
    });
    test("closed synctable -- most public API functions throw an error", function () { return __awaiter(void 0, void 0, void 0, function () {
        var err_1, err_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    expect(function () { return synctable.set({ priority: "medium" }); }).toThrow("can't set until table is initialized");
                    expect(function () { return synctable.get(); }).toThrow("closed");
                    expect(function () { return synctable.get_one(); }).toThrow("table not yet initialized");
                    expect(synctable.has_uncommitted_changes()).toBe(false); // does not throw
                    return [4 /*yield*/, synctable.close()];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, synctable.wait(function () { return true; })];
                case 3:
                    _a.sent();
                    return [3 /*break*/, 5];
                case 4:
                    err_1 = _a.sent();
                    expect(err_1.toString()).toContain("closed");
                    return [3 /*break*/, 5];
                case 5:
                    _a.trys.push([5, 7, , 8]);
                    return [4 /*yield*/, synctable.save()];
                case 6:
                    _a.sent();
                    return [3 /*break*/, 8];
                case 7:
                    err_2 = _a.sent();
                    expect(err_2.toString()).toContain("closed");
                    return [3 /*break*/, 8];
                case 8: return [2 /*return*/];
            }
        });
    }); });
    // some errors...
    test("try create synctable with an invalid query and get exception", function () {
        var invalid_query = {
            // invalid since missing id primary key.
            system_notifications: [{ time: null, text: null, priority: null }],
        };
        expect(function () { return new synctable_1.SyncTable(invalid_query, [], client, 0); }).toThrow("primary key");
    });
    test("try create synctable with another invalid query and get exception", function () {
        var invalid_query = {
            // invalid since extra foo key.
            system_notifications: [
                { id: null, foo: null, time: null, text: null, priority: null },
            ],
        };
        expect(function () { return new synctable_1.SyncTable(invalid_query, [], client, 0); }).toThrow("field in the schema");
    });
});
