"use strict";
/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.to_key = exports.parse_query = void 0;
var misc_1 = require("../../misc");
var schema_1 = require("../../schema");
// Parse query description to allow for some convenient shortcuts
// TODO: document them here!
function parse_query(query) {
    var _a, _b, _c;
    query = misc_1.deep_copy(query);
    if (typeof query === "string") {
        // name of a table -- get all fields
        var s = schema_1.SCHEMA[query];
        if (s == null)
            throw Error("no schema for query \"" + query + "\"");
        if (s.user_query == null)
            throw Error("user_query not defined for query \"" + query + "\"");
        if (s.user_query.get == null)
            throw Error("user_query.get not defined for query \"" + query + "\"");
        var v = misc_1.copy(s.user_query.get.fields);
        for (var k in v) {
            v[k] = null;
        }
        return _a = {}, _a[query] = [v], _a;
    }
    else {
        var k = misc_1.keys(query);
        if (k.length !== 1) {
            throw Error("must specify exactly one table");
        }
        var table = k[0];
        if (!misc_1.is_array(query[table])) {
            return _b = {}, _b[table] = [query[table]], _b;
        }
        else {
            return _c = {}, _c[table] = query[table], _c;
        }
    }
}
exports.parse_query = parse_query;
var json_stable_stringify = require("json-stable-stringify");
function to_key(x) {
    if (typeof x === "object") {
        return json_stable_stringify(x);
    }
    else {
        return x;
    }
}
exports.to_key = to_key;
