/*
 *  This file is part of CoCalc: Copyright © 2020 Sagemath, Inc.
 *  License: AGPLv3 s.t. "Commons Clause" – see LICENSE.md for details
 */
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var PROJECT_UPGRADES = require("./schema").PROJECT_UPGRADES;
var misc = require("./misc");
// This is used by the frontend in r_account.  It's also used by the backend
// to double check the claims of the frontend.
// stripe_subscriptions_data = stripe_customer?.subscriptions?.data
function get_total_upgrades(stripe_subscriptions_data) {
    var e_1, _a;
    var subs = stripe_subscriptions_data;
    if (subs == null) {
        return {};
    }
    // always running isn't in any of the subscriptions and I don't want to edit benefits
    // for all of them, so just put a zero 0 (since this whole old upgrades thing
    // will eventually go away). https://github.com/sagemathinc/cocalc/issues/4802
    var total = { always_running: 0 };
    try {
        for (var subs_1 = __values(subs), subs_1_1 = subs_1.next(); !subs_1_1.done; subs_1_1 = subs_1.next()) {
            var sub = subs_1_1.value;
            if (sub.status != "active" && sub.status != "trialing") {
                // not yet paid for or no longer available.
                continue;
            }
            var info = PROJECT_UPGRADES.subscription[sub.plan.id.split("-")[0]];
            if (info == null) {
                // there are now some subscriptions that have nothing to do with "upgrades" (e.g., for licenses).
                continue;
            }
            var benefits = info.benefits;
            // TODO: add a quantity third field to map_sum to make this not so naive...
            for (var q = 0; q < sub.quantity; q++) {
                total = misc.map_sum(total, benefits);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (subs_1_1 && !subs_1_1.done && (_a = subs_1.return)) _a.call(subs_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return total;
}
exports.get_total_upgrades = get_total_upgrades;
//
// INPUT:
//    subscriptions = {standard:2, premium:1, course:2, ...}
//    projects = {project_id:{cores:1, network:1, ...}, ...}
//
// OUTPUT:
//     {available:{cores:10, network:3, ...},   excess:{project_id:{cores:2, ...}}  }
//
function available_upgrades(stripe_subscriptions_data, projects) {
    var e_2, _a, _b;
    var project_id, upgrades;
    var available = get_total_upgrades(stripe_subscriptions_data); // start with amount available being your quota
    var excess = {}; // nothing exceeds quota
    // sort projects by project_id so that excess will be well defined
    var v = [];
    for (project_id in projects) {
        upgrades = projects[project_id];
        v.push({ project_id: project_id, upgrades: upgrades });
    }
    v.sort(function (a, b) { return misc.cmp(a.project_id, b.project_id); });
    try {
        for (var v_1 = __values(v), v_1_1 = v_1.next(); !v_1_1.done; v_1_1 = v_1.next()) {
            _b = v_1_1.value, project_id = _b.project_id, upgrades = _b.upgrades;
            for (var prop in upgrades) {
                var curval = upgrades[prop];
                if (available[prop] == null) {
                    available[prop] = 0;
                } // ensure that available is defined for this prop
                if (curval <= available[prop]) {
                    // if the current value for this project is within what is left, just subtract it off
                    available[prop] -= curval;
                }
                else {
                    // otherwise, it goes over, so record by how much in excess, then set available to 0.
                    if (excess[project_id] == null) {
                        excess[project_id] = {};
                    }
                    excess[project_id][prop] = curval - available[prop];
                    available[prop] = 0;
                }
            }
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (v_1_1 && !v_1_1.done && (_a = v_1.return)) _a.call(v_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return { available: available, excess: excess };
}
exports.available_upgrades = available_upgrades;
// INPUT: same as above, but also a single project_id
//
// OUTPUT:  Returns the maximum amount for each upgrade setting that a control
// (which starts at 0) for configuring that setting for the project should go up to.
//
//      {cores:2, network:1, disk_quota:2000, memory:1000}
//
//
function upgrade_maxes(stripe_subscriptions_data, projects, project_id) {
    var _a = available_upgrades(stripe_subscriptions_data, projects), available = _a.available, excess = _a.excess;
    var allocated = projects[project_id];
    var maxes = {};
    for (var param in available) {
        var avail = available[param];
        var max = PROJECT_UPGRADES.max_per_project[param]; // the maximum allowed for this param for any project
        var alloc = allocated[param] != null ? allocated[param] : 0; // how much has already been allocated to this project
        maxes[param] = Math.min(alloc + avail, max);
    }
    return maxes;
}
exports.upgrade_maxes = upgrade_maxes;
